/*
 * @geminiBubbleMapFrame.java Version 1.1 01/24/2014
 *
 * Copyright (c) 2012 Kansas Geological Survey
 * 1930 Constant Avenue, Lawrence, Kansas, 66047, U.S.A.
 * All Rights Reserved.
 */

package gemini.gui;

import java.util.Observer;
import java.util.Observable;
import java.awt.*;
import java.awt.event.*;

import javax.swing.*;
import javax.swing.event.*;

import cmn.cmnStruct;                            // GLOBAL Common Data Structure

import iqstrat.iqstratHeadersListStruct;         // Well Header List Structure
import iqstrat.iqstratHeadersStruct;             // Well Header Data Structure

import gemini.plot.geminiPlotLeases;             // Lease Production Bubble Map Plot

import production.productionStruct;              // Production Data Structure
import production.productionListStruct;          // Production List Structure
import production.io.ReadProductionXMLFile;      // Read Production XML File

import production.plot.productionPlotListStruct; // Production Plot List Structure
import production.plot.productionPlotStruct;     // Production Plot Data Structure
import production.productionBuildPlotData;       // Production Build Plot Data
import production.gui.productionControlFrame;    // Production Plot Control Dialog

import util.utilLandGridListStruct;              // TRS Land Grid List Structure
import util.utilWaitAnimation;                   // Wait for Data Animation
import util.SwingWorker;                         // Swing Worker to control Animation

/** Class geminiBubbleMapFrame
 *  <p> This file will display lease production history as bubble map.
 *
 *  @version 1.1 01/24/2014
 *  @author  John R. Victorine
 */

public class geminiBubbleMapFrame extends JFrame implements ActionListener, Observer
{
  public static final int OIL      = production.productionStruct.OIL;
  public static final int GAS      = production.productionStruct.GAS;

  public static final int STOP     = gemini.plot.geminiPlotLeases.STOP;
  public static final int PAUSE    = gemini.plot.geminiPlotLeases.PAUSE;
  public static final int RUN      = gemini.plot.geminiPlotLeases.RUN;

  public static final int PREVIOUS = 0;
  public static final int NEXT     = 1;

  // Input Variables

  private String                   sTitle    = "";
  private cmnStruct                stStruct  = null; // Global Common Data
  private utilLandGridListStruct   stGrid    = null; // Land Grid List Data
  private iqstratHeadersListStruct stWells   = null; // Well List Data
  private int                      iGrid     = -1;   // Grid

  // Global Variables

  private Observable               notifier  = null; // Observable Object this Frame
  private geminiPlotLeases         panel     = null; // Plot Wells Panel

  private iqstratHeadersStruct     stWell    = null; // Selected Well

  private int                      iList     = 0;    // Total Number of Lease Production Records
  private productionListStruct     stList[]  = null; // List of Lease Production List Data Structures
  private productionPlotListStruct stPlot    = null; // Production Plot List Structure
  private productionControlFrame   plot      = null; // Production Plot Control Dialog

  private int                      iProduct  = OIL;  // Product to display
  private int                      iStart    = 2012; // Starting Year of Production Data
  private int                      iEnd      = 0;    // Ending Year of Production Data
  private double                   dMinCum   = 0.0;  // Minimum Cumulative value
  private double                   dMaxCum   = 0.0;  // Maximum Cumulative value
  private double                   dMedium   = 0.0;  // Medium Value of the Cumlative
  private double                   dMean     = 0.0;  // Mean (Average) Value of the Cumulative
  private double                   dMinProd  = 0.0;  // Minimum Production value
  private double                   dMaxProd  = 0.0;  // Maximum Production value

  private double                   dXMax     = 0.0;
  private double                   dXMin     = 0.0;
  private double                   dYMax     = 0.0;
  private double                   dYMin     = 0.0;

  private int                      iAction   = STOP; // Set Button action (default stop)
  private Timer                    timer     = null; // Timer for Animation
  private int                      iCurrentYear = 0; // Current Year for the bubble map

  private boolean                  bProduct[] = { false, false };

  // Screen Width

  private int iControlWidth = 400;  // Control Panel Width
  private int iScreenWidth  = 0;    // Screen Width
  private int iScreenHigh   = 0;    // Scree Hieght

  // Global Widgets

  private JLabel       lbl      = new JLabel();        // Latitude & Longitude Label
  private JLabel       lblYear  = new JLabel();        // Year Being Displayed
  private JLabel       lblAvg   = new JLabel();        // Minimimum, Medium, Mean, Maximum Values

  private JRadioButton rbOIL    = new JRadioButton();  // Oil Production Plot Data
  private JRadioButton rbGAS    = new JRadioButton();  // Gas Production Plot Data

  private JButton      btnStop  = new JButton();       // Stop Animation Action
  private JButton      btnPause = new JButton();       // Pause Animation Action
  private JButton      btnRun   = new JButton();       // Run Animation Action

  private JButton      btnPrev  = null;                // Previous Year Action
  private JButton      btnNext  = null;                // Next Year Action

  private JTextField   txtYear  = new JTextField();    // Start Year

  /** Constructor geminiKSMapCountyPanel()
   *  <p> This is the Constructor for this class
   *  @param sTitle   - Title
   *  @param stStruct - Global Common Data Structure
   *  @param stGrid   - Utility Land Grid List Data Structure
   *  @param stWells  - Well Header List Data Structure
   *  @param iGrid    - Plot Text with Grid or NOT.
   */

  public geminiBubbleMapFrame( int                      iControlWidth,
                               String                   sTitle,
	                           cmnStruct                stStruct,
                               utilLandGridListStruct   stGrid,
                               iqstratHeadersListStruct stWells,
                               int iGrid  )
  {
    int delay = 1000; //milliseconds

    try
    {
	  this.iControlWidth = iControlWidth;
      this.sTitle        = sTitle;
      this.stStruct      = stStruct;
      this.stGrid        = stGrid;
      this.stWells       = stWells;
      this.iGrid         = iGrid;

      jbInit();

      LoadProductionData();

      if (bProduct[0])
      {
		iProduct = OIL;
		rbOIL.setSelected(true);
	  }
	  else
	  {
		iProduct = GAS;
		rbGAS.setSelected(true);
	  }

      convert(production.productionStruct.PRODUCT[iProduct]);

      setMinMaxValues();

      panel.setProductionData( stPlot, iStart, iEnd,
                               dMinCum,  dMaxCum, dMinProd, dMaxProd,
                               dMedium, dMean);
      setCurrentYear(iStart);

      lbl.setForeground(Color.black);
      lbl.setText("");

      setButtons();

      // Set up a timer that calls this object's action handler.

      timer = new Timer(delay, new animateAction());
      timer.setInitialDelay(0);
      timer.setCoalesce(true);

      // Set Close Window

      addWindowListener( new geminiBubbleMapFrame_WindowListener() );
    }
    catch(Exception ex)
    {
      ex.printStackTrace();
      String sMessage = new String("geminiBubbleMapFrame(): " + ex.getMessage());
      JOptionPane.showMessageDialog((Component) null,
                                    sMessage,
                                    "ERROR",
                                    JOptionPane.ERROR_MESSAGE);
    }
  }

  /** Method jbInit()
   *  <p> This method will create this Panel.
   *  @throws Exception
   */

  private void jbInit() throws Exception
  {
	JPanel   pnlTop     = new JPanel();
	JPanel   pnlBase    = new JPanel();
	JPanel   pnlLatLong = new JPanel();
	JPanel   pnlProduct = new JPanel();
	JPanel   pnlPlay    = new JPanel();
	JPanel   pnlIncPlay = new JPanel();
	int      iWidth     = 0;
	int      iHeight    = 0;

    ButtonGroup  group  = new ButtonGroup();

    // Create an Observable

    notifier = new geminiBubbleMapFrameObservable();

    // Add this Dialog to the notifier as observer

    notifier.addObserver( this );

    // Build Frame

    this.setTitle( sTitle );
    this.getContentPane().setLayout(new BorderLayout());

    pnlTop.setLayout(new BorderLayout());
    pnlTop.setBackground(Color.white);

    pnlLatLong.setBackground(Color.white);

    lbl.setText("PLEASE WAIT - LOADING PRODUCTION DATA ...");
    lbl.setFont(new java.awt.Font("Dialog", 1, 13));
	lbl.setHorizontalTextPosition(JLabel.CENTER);
    lbl.setForeground(Color.red);
    lbl.setBackground(Color.white);

    lblYear.setText("");
    lblYear.setBackground(Color.white);

    lblAvg.setText("");
    lblAvg.setBackground(Color.white);

    Dimension d  = Toolkit.getDefaultToolkit().getScreenSize();
    iScreenWidth = d.width;
    iScreenHigh  = d.height-50;

    iWidth  = (4*(iScreenHigh-50))/5;
    iHeight = iScreenHigh-50;

//    panel = new geminiPlotLeases( stStruct, stWells, stGrid, 800, 850, lbl, 2);
    panel = new geminiPlotLeases( notifier, stStruct, stWells, stGrid,
                                 iScreenWidth-iControlWidth, iScreenHigh-50-40,
                                 lbl, iGrid );
//    panel = new geminiPlotLeases( stStruct, stWells, stGrid, iWidth, iHeight, lbl, 2);

    // Base Panel

    // -- Stop Animation Button

    pnlBase.setLayout(new BorderLayout());
    pnlBase.setBackground(Color.white);

    // .. Product Panel

    rbOIL.setFont(new java.awt.Font("Dialog", 1, 11));
	rbOIL.setText("OIL (BBL)");
	rbOIL.setEnabled(false);
	rbOIL.addActionListener(this);
    group.add(rbOIL);

    rbGAS.setFont(new java.awt.Font("Dialog", 1, 11));
	rbGAS.setText("GAS (MCF)");
	rbGAS.setEnabled(false);
	rbGAS.addActionListener(this);
    group.add(rbGAS);

    // .. Play Panel

    btnStop.setFont(new java.awt.Font("Dialog", 1, 13));
    btnStop.setText("Stop");
    btnStop.setEnabled(false);
    btnStop.addActionListener(this);

    btnPause.setFont(new java.awt.Font("Dialog", 1, 13));
    btnPause.setText("Pause");
    btnPause.setEnabled(false);
    btnPause.addActionListener(this);

    btnRun.setFont(new java.awt.Font("Dialog", 1, 13));
    btnRun.setText("Run");
    btnRun.setEnabled(false);
    btnRun.addActionListener(this);

    // .. Increment Play Panel

    pnlIncPlay.setLayout(new BorderLayout());
    pnlIncPlay.setBackground(Color.white);

    btnPrev = new JButton(createImageIcon(cmn.cmnStruct.PREVIOUS));
    btnPrev.setPreferredSize(new Dimension(27, 27));
    btnPrev.setBackground(Color.white);
    btnPrev.setToolTipText("Previous Year - Increment by 1 year");
    btnPrev.setEnabled(false);
    btnPrev.addActionListener(this);

    txtYear.setText( "" + iCurrentYear );
    txtYear.setColumns(4);
    txtYear.setHorizontalAlignment(JTextField.CENTER);
    txtYear.setFont(new java.awt.Font("Dialog", 1, 13));
    txtYear.setEditable(false);
    txtYear.addFocusListener(new geminiBubbleMapFrameFocusAdapter(this));

    btnNext = new JButton(createImageIcon(cmn.cmnStruct.NEXT));
    btnNext.setPreferredSize(new Dimension(27, 27));
    btnNext.setBackground(Color.white);
    btnNext.setToolTipText("Next Year - Increment by 1 year");
    btnNext.setEnabled(false);
    btnNext.addActionListener(this);

    // Attach Widgets

    this.getContentPane().add(pnlTop,  BorderLayout.NORTH);
    pnlTop.add(pnlLatLong,             BorderLayout.CENTER);
    pnlTop.add(lblAvg,                 BorderLayout.WEST);
    pnlLatLong.add(lbl,                null);
//    pnlTop.add(lbl,                   BorderLayout.CENTER);
    pnlTop.add(lblYear,                BorderLayout.EAST);

    this.getContentPane().add(panel,   BorderLayout.CENTER);

    this.getContentPane().add(pnlBase, BorderLayout.SOUTH);
    pnlBase.add(pnlProduct,            BorderLayout.WEST);
    pnlProduct.add(rbOIL,             null);
    pnlProduct.add(rbGAS,             null);

    pnlBase.add(pnlPlay,               BorderLayout.CENTER);
    pnlPlay.add(btnStop, null);
    pnlPlay.add(btnPause, null);
    pnlPlay.add(btnRun, null);

    pnlBase.add(pnlIncPlay,            BorderLayout.EAST);
    pnlIncPlay.add(btnPrev,            BorderLayout.WEST);
    pnlIncPlay.add(txtYear,            BorderLayout.CENTER);
    pnlIncPlay.add(btnNext,            BorderLayout.EAST);

    // Display the Frame

//    this.setSize(new Dimension(800, 850));
//    this.setLocation((d.width - this.getSize().width)/2,
//                     (d.height - this.getSize().height)/2);

    this.setSize(iScreenWidth-iControlWidth, iScreenHigh);
    this.setLocation(iControlWidth+1, 1);

    this.setResizable(true);
    this.setVisible(true);
  }

  /** METHOD createImageIcon()
   *  <p> This Method will create the Frame for this class.
   *  @param path = the path to the image
   *  @return image icon
   */

  protected ImageIcon createImageIcon(String path)
  {
    java.net.URL imgURL = getClass().getResource(path);
    ImageIcon    image  = null;

    if (imgURL != null)
    {
      image = new ImageIcon(imgURL);
    }
    else
    {
       System.err.println("Couldn't find file: " + path);
    }

    return (image);
  }

  /** METHOD geminiBubbleMapFrame_WindowListener()
   *  <p> This Class will handle Frame Close Action using the X Button on Frame
   */

  public class geminiBubbleMapFrame_WindowListener extends WindowAdapter
  {
    public void windowClosing(WindowEvent e)
    {
      close();
      System.exit(0);
    }
  }

  /** Method close()
   * <p> This method will close all dialogs that are opened by this class.
   */

  public void close()
  {
	sTitle    = null;
    stStruct  = null; // Global Common Data
    stGrid    = null; // Land Grid List Data
    stWells   = null; // Well List Data

    // Global Variables

    notifier = null; // Observable Object this Frame

    if (panel != null)
      panel.close();
    panel    = null; // Plot Wells Panel

    if (iList  > 0)
    {
	  for (int i=0; i<iList; i++)
	  {
		if (stList[i] != null)
		  stList[i].delete();
		stList[i] = null;
	  }
	  stList = null;
	}

	if (stPlot != null)
	  stPlot.delete();
	stPlot = null;

    if (plot != null)
      plot.close();
    plot    = null; // Production Plot Control Dialog

    // Global Widgets

    lbl      = null;
    lblYear  = null;
    lblAvg   = null;

    rbOIL    = null;
    rbGAS    = null;

    btnStop  = null;
    btnPause = null;
    btnRun   = null;

    btnPrev  = null;
    btnNext  = null;

    txtYear  = null;

	dispose();
  }

  /* ===================================================================== *
   * -------------------- WAIT LOAD PRODUCTION DATA ---------------------- *
   * ===================================================================== */

  /** Method WaitLoad()
   * <p> This method will display the Wait Animation
   */

  private void WaitLoad()
  {
    java.net.URL url = null;

    // Get the URL Path to display the Wait Animation

    try
    {
      url = new java.net.URL(cmn.cmnStruct.TOOLS + "/");
    }
    catch(Exception ex)
    {
      System.out.println("Wrong Image Directory" + ex);
    }

    // Start the Wait Animation Dialog

    final utilWaitAnimation pDialog = new utilWaitAnimation(url);
    pDialog.startAnimation();

    final SwingWorker worker = new SwingWorker()
    {
      public Object construct()
      {
//        load();

        return "OK";
      }

      //Runs on the event-dispatching thread.

      public void finished() { pDialog.stopAnimation(); }
    };

    // Start the Wait Animation Thread

    worker.start();  //required for SwingWorker
  }

  /** Method load()
   *  <p> This method will load in the production data for plotting
   */

  public void load()
  {
    int delay = 1000; //milliseconds

    try
    {
      LoadProductionData();

      if (bProduct[0])
      {
		iProduct = OIL;
		rbOIL.setSelected(true);
	  }
	  else
	  {
		iProduct = GAS;
		rbGAS.setSelected(true);
	  }

      setButtons();

      convert(production.productionStruct.PRODUCT[iProduct]);

      setMinMaxValues();

      panel.setProductionData( stPlot, iStart, iEnd,
                               dMinCum,  dMaxCum, dMinProd, dMaxProd,
                               dMedium, dMean);
      setCurrentYear(iStart);

//      getMinMaxPosition();
//      setArea( dXMin, dXMax, dYMin, dYMax );

      // Set up a timer that calls this object's action handler.

      timer = new Timer(delay, new animateAction());
      timer.setInitialDelay(0);
      timer.setCoalesce(true);
    }
    catch(Exception ex)
    {
      ex.printStackTrace();
      String sMessage = new String("geminiBubbleMapFrame.load(): " + ex.getMessage());
      JOptionPane.showMessageDialog((Component) null,
                                    sMessage,
                                    "ERROR",
                                    JOptionPane.ERROR_MESSAGE);
    }
  }

  /* ===================================================================== *
   * ---------------------- READ PRODUCTION DATA ------------------------- *
   * ===================================================================== */

  /** Method LoadProductionData()
   *  <p> This method will Load the Array of Lease Production Records
   */

  private void LoadProductionData()
  {
	if (stWells != null)
	{
	  if (stWells.iCount > 0)
	  {
		stList = new productionListStruct[stWells.iCount];

		for (int i=0; i<stWells.iCount; i++)
		{
		  stList[i] = production.productionUtility.transfer(
			            ReadProductionData( stWells.stItem[i].sKID ) );

		  if (stList[i] != null)
		  {
			stList[i].sType = new String("LEASE");
	        for (int k=0; k<stList[i].iCount; k++)
	        {
			  if (stList[i].stItem[k].sProduct.equals("OIL"))
			    bProduct[0] = true;
			  else if (stList[i].stItem[k].sProduct.equals("GAS"))
			    bProduct[1] = true;
		    }
		  }

	      iList++;
		}
	  }
	}
  }

  /** Method ReadProductionData()
   *  <p> This method will read the Production Data for the selected leases
   *  @param sKID = Lease Primary KEY
   *  @param st   = Lease Production List Structure
   */

  private static productionListStruct ReadProductionData(String sKID)
  {
	productionListStruct  st     = null;
    ReadProductionXMLFile pData  = null;
    String                sQuery = new String("sLease=" + sKID);

    pData  = new ReadProductionXMLFile(production.io.ReadProductionXMLFile.URL);
    st     = pData.Process(cmn.cmnStruct.KGS_PRODUCTION+sQuery);

    return (st);
  }

  /* ===================================================================== *
   * ----------------- CONVERT TO PRODUCTION PLOT DATA ------------------- *
   * ===================================================================== */

  /** Method convert()
   *  <p>  This method will convert the Production Data for Leases to
   *       Production Plot Data for building
   *  @param  sProduct = the product to be plotted
   *  @return stPlot   = The production Plot data Structure
   */

  private productionPlotListStruct convert( String sProduct )
  {
    productionPlotStruct st = null;

    if (stPlot != null)
      stPlot.delete();
    stPlot = null;

	if (iList > 0)
	{
//System.out.println("OIL: "+bProduct[0]+" GAS: "+bProduct[1]);

	  for (int i=0; i<iList; i++)
	  {
//production.productionUtility.print(stList[i]);
		st = production.plot.productionPlotUtility.buildProductionPlotData(
			   sProduct, stWells.stItem[i], stList[i] );
	    if (st == null)
	    {
		  st = new productionPlotStruct();
		}

	    stPlot = production.plot.productionPlotUtility.add(st, stPlot);
	  }
	}

	return (stPlot);
  }

  /** Method setMinMaxValues()
   *  <p> This method will compute the minimum and maximum values for the
   *      production plot lease data.
   */

  private void setMinMaxValues()
  {
	if (stPlot != null)
	{
      iStart   = 2012;
      iEnd     = 0;
	  dMinCum  = 0.0;
	  dMaxCum  = 0.0;
	  dMinProd = 0.0;
	  dMaxProd = 0.0;

	  for (int i=0; i<stPlot.iCount; i++)
	  {
		for (int j=0; j<stPlot.stItem[i].iRows; j++)
		{
		  if (stPlot.stItem[i].iProduct == iProduct)
		  {
		    if ((i==0) && (j==0))
		    {
			  iStart   = 2012;
			  iEnd     = stPlot.stItem[i].iTime[j];
			  dMinCum  = stPlot.stItem[i].dCumulative[j];
			  dMaxCum  = stPlot.stItem[i].dCumulative[j];
			  dMinProd = stPlot.stItem[i].dProduction[j];
			  dMaxProd = stPlot.stItem[i].dProduction[j];
		    }

		    if ((iStart > stPlot.stItem[i].iTime[j]) &&
		        (stPlot.stItem[i].iTime[j] > 1935))
		      iStart = stPlot.stItem[i].iTime[j];

		    if (iEnd < stPlot.stItem[i].iTime[j])
		      iEnd = stPlot.stItem[i].iTime[j];

		    if (dMinCum > stPlot.stItem[i].dCumulative[j])
		      dMinCum = stPlot.stItem[i].dCumulative[j];

		    if (dMaxCum < stPlot.stItem[i].dCumulative[j])
		      dMaxCum = stPlot.stItem[i].dCumulative[j];

		    if (dMinProd > stPlot.stItem[i].dProduction[j])
		      dMinProd = stPlot.stItem[i].dProduction[j];

		    if (dMaxProd < stPlot.stItem[i].dProduction[j])
		      dMaxProd = stPlot.stItem[i].dProduction[j];
		  }
		}
	  }
	}
//System.out.println(iStart+" "+iEnd+" "+iProduct+" Prod: "+dMinProd+" "+dMaxProd+" Cum: "+dMinCum+" "+dMaxCum);
    compute();
  }

  private void compute()
  {
	int    i,k;
	double data[]  = null;
	double dSum    = 0.0;
	double total   = 0.0;

	if (stPlot != null)
	{
	  if (stPlot.iCount > 0)
	  {
		total = stPlot.iCount;
	    data  = new double[stPlot.iCount];

	    for (i=0; i<stPlot.iCount; i++)
	      data[i] = 0.0;

	    for (i=0; i<stPlot.iCount; i++)
	    {
		  if (stPlot.stItem[i].iRows > 0)
		  {
		    data[i] = stPlot.stItem[i].dCumulative[stPlot.stItem[i].iRows-1];
		  }
		}
	    data = bubbleSort(data);

		for (i=0; i<stPlot.iCount; i++)
		{
		  dSum = dSum + data[i];
		}
		dMean = Math.round(dSum / total);

		k = (int) ((total+1) / 2);
		dMedium = data[k];

        lblAvg.setText("Minimum="+dMinCum+" Medium="+dMedium+" Mean="+dMean+" Maximum="+dMaxCum);
	  }
	}
  }

  /** Method bubbleSort()
   * <p> This method will sort in ascending depth order (lowest to highest)
   * @param  st = data list structure
   * @return st = sorted data list structure
   */

  public static double[] bubbleSort(double d[])
  {
    boolean swappedOnPrevRun = true;
    double  dTemp   = 0.0;

    if (d != null)
    {
      if (d.length > 1)
      {
        while(swappedOnPrevRun)
        {
          // this variable keeps track of whether to continue sorting or exit

          swappedOnPrevRun = false;

          // loop through every element in the array, except for the last one

          for(int i=0; i<d.length-1; i++)
          {
            // if current element is greater than the next swap the two elements

            if(d[i] > d[i+1])
            {
              // we don't want the loop to end just yet, we're not done

              swappedOnPrevRun = true;

              // store element i in a temporary variable

              dTemp = d[i];

              // set element i+1 to where i used to be

              d[i] = d[i+1];

              // release the old i from temp into i+1 slot

              d[i+1] = dTemp;
            }
          }
        }
      }
    }

    return (d);
  }

  /* ===================================================================== *
   * -------------------------- GET METHODS ------------------------------ *
   * ===================================================================== */

  /** Methods Get Minimum & Maximum Latitude & Longitude
   *  @dXMin = Minimum Longitude
   *  @dXMax = Maximum Longitude
   *  @dYMin = Minimum Latitude
   *  @dYMax = Maximum Latitude
   */

  public double getMinLatitude()  { return (panel.getMinLatitude()); }
  public double getMaxLatitude()  { return (panel.getMaxLatitude()); }
  public double getMinLongitude() { return (panel.getMinLongitude()); }
  public double getMaxLongitude() { return (panel.getMaxLongitude()); }

  /** Method getWell()
   *  <p> This method will return the selected well.
   */

  public iqstratHeadersStruct getWell() { return (stWell); }

  /** Method getMinMaxPosition()
   *  <p> This method will determine the minimum and maximum position of the
   *      wells that have been passed in.
   *

  private void getMinMaxPosition()
  {
	int    iStart     = 0;
    double dMinLat    = 0.0;
    double dMaxLat    = 0.0;
    double dMinLong   = 0.0;
    double dMaxLong   = 0.0;
    double dLongDelta = 0.0;
    double dLatDelta  = 0.0;

	if (stWells != null)
	{
	  for (int i=0; i<stWells.iCount; i++)
	  {
        if (iStart == 0)
        {
          dMinLat  = stWells.stItem[i].dLatitude;
          dMaxLat  = stWells.stItem[i].dLatitude;
          dMinLong = stWells.stItem[i].dLongitude;
          dMaxLong = stWells.stItem[i].dLongitude;
          iStart   = 1;
        }

        if (dMinLat > stWells.stItem[i].dLatitude)
          dMinLat  = stWells.stItem[i].dLatitude;

        if (dMaxLat < stWells.stItem[i].dLatitude)
          dMaxLat  = stWells.stItem[i].dLatitude;

        if (dMinLong > stWells.stItem[i].dLongitude)
          dMinLong  = stWells.stItem[i].dLongitude;

        if (dMaxLong < stWells.stItem[i].dLongitude)
          dMaxLong  = stWells.stItem[i].dLongitude;
	  }
	}

    dLongDelta = 0.05 * (dMaxLong - dMinLong);
    dLatDelta  = 0.05 * (dMaxLat  - dMinLat);

    dXMin      = dMinLong - dLongDelta;
    dXMax      = dMaxLong + dLongDelta;
    dYMin      = dMinLat  - dLatDelta;
    dYMax      = dMaxLat  + dLatDelta;

System.out.println("Longitude: "+dXMin+" "+dXMax+" <-> Latitude: "+dYMin+" "+dYMax);
  }
*/
  /* ===================================================================== *
   * -------------------------- SET METHODS ------------------------------ *
   * ===================================================================== */

  /** Method setButtons()
   *  <p> This method will enable/disable button, change colors
   */

  private void setButtons()
  {
    if (stPlot.iCount > 0)
    {
      rbOIL.setEnabled(true);
      rbGAS.setEnabled(true);

      btnStop.setEnabled(true);
      btnPause.setEnabled(true);
      btnRun.setEnabled(true);
    }

    btnPrev.setEnabled(false);
    btnNext.setEnabled(false);
    txtYear.setEditable(false);

    switch (iAction)
    {
	  case STOP:
        btnStop.setBackground(new Color(255, 255, 0));
        btnPause.setBackground(new Color(180, 180, 180));
        btnRun.setBackground(new Color(180, 180, 180));
	    break;
	  case PAUSE:
        btnStop.setBackground(new Color(180, 180, 180));
        btnPause.setBackground(new Color(255, 255, 0));
        btnRun.setBackground(new Color(180, 180, 180));

	    btnPrev.setEnabled(true);
	    btnNext.setEnabled(true);
	    txtYear.setEditable(true);
	    break;
	  case RUN:
        btnStop.setBackground(new Color(180, 180, 180));
        btnPause.setBackground(new Color(180, 180, 180));
        btnRun.setBackground(new Color(255, 255, 0));
	    break;
	}

    if (!bProduct[0])
      rbOIL.setEnabled(false);

    if (!bProduct[1])
      rbGAS.setEnabled(false);
  }

  /** Method setWell()
   *  <p> This method will set whether reference well is to be selected or
   *      the edit tops well is to be selected
   *  @param iWell = Well to be selected
   */

  public void setWell( int iWell ) { panel.setWell( iWell ); }

  /** Method setLogType()
   *  <p> This method will set the filter of the wells that are
   *      being plotted.
   *  @param filter = the filter to the Well plot
   */

  public void setLogType(String filter) { panel.setLogType(filter); }

  /** Method setText()
   *  <p> This method will set the filter of the wells that are
   *      being plotted.
   *  @param filter = the filter to the Well plot
   */

  public void setText(int iAdd ) { panel.setText(iAdd); }

  /** Method clearWell()
   *  <p> This method will deselete well
   *  @param iPanel = the Panel cleared
   */

  public void clearWell( int iPanel ) { panel.clearWell(iPanel); }

  /** Method zoom_off()
   *  <p> This method will set the zoom off;
   */

  public void zoom_off() { panel.zoom_off(); }

  /** Method zoom_in()
   *  <p> This method will compute the Minimum & Maximum Latitude & Longitude
   */

  public void zoom_in() { panel.zoom_in(); }

  /** Method zoom_out()
   *  <p> This method will reset the Minimum & Maximum Latitude & Longitude
   */

  public void zoom_out() { panel.zoom_out(); }

  /** Method setField()
   *  <p> This method will set field KID to plot field back ground color
   *  @param sFieldKID = Field KID
   */

  public void setField(String sFieldKID)
  {
	panel.setField(sFieldKID);
	repaint();
  }

  /** Method setArea()
   *  <p> This method will zoom into the area
   */

  public void setArea( double dXMin, double dXMax, double dYMin, double dYMax )
  {
	panel.setArea( dXMin, dXMax, dYMin, dYMax );
  }

  /** Method setProductionPlotData()
   *  <p> This method will set the Production Plot Data
   *  @param iProduct = the Product Identifier
   */

  private void setProductionPlotData(int iProduct)
  {
	this.iProduct = iProduct;

    convert(production.productionStruct.PRODUCT[iProduct]);

    setMinMaxValues();

    panel.setProductionData( stPlot, iStart, iEnd,
                             dMinCum,  dMaxCum, dMinProd, dMaxProd,
                             dMedium, dMean);
    setCurrentYear(iStart);
  }

  /* ===================================================================== *
   * ----------- PRODUCTION PLOT & DECLINE CURVE ANALYSIS ---------------- *
   * ===================================================================== */

  /** Method ProductionPlot()
   * <p> This method will load the Production Plot & Decline Curve Plot
   */

  private void ProductionPlot( int iPlot )
  {
//    if (plot != null)
//      plot.close();
//    plot    = null; // Production Plot Control Dialog

    if (iPlot > -1)
    {
	  if (stList[iPlot].iCount > 2)
	  {
//System.out.println(stList[iPlot].sName + " " + stList[iPlot].sKID);
	    plot = new productionControlFrame( stList[iPlot] );
	  }
	}
  }

  /* ===================================================================== *
   * ----------------------- ANIMATION ACTIONS --------------------------- *
   * ===================================================================== */

  /** Method startAnimation() throws Exception
   *  <p> Starts the timer that generates action events
   * @throws Exception - exception starting the timer
   *
   *  JayaPrakash "J.P" Pakalpati 12/27/2002
   */

  private void startAnimation () throws Exception
  {
    if (!timer.isRunning())
    {
      timer.start();
    }
  }

  /** Method pauseAnimation()
   *  <p> Stops the timer that generate action events
   *
   *  JayaPrakash "J.P" Pakalpati 12/27/2002
   */

  private void pauseAnimation()
  {
    if (timer.isRunning())
    {
      timer.stop();
    }
  }

  /** Method stopAnimation() throws Exception
   *  <p> Stops the timer that generate action events
   *      and resets the current year to initial value
   *      <br>
   *      Note: Current year is the year for which the bubble map is displayed
   *            on the drawing panel
   * @throws Exception - exception stopping the timer
   *
   *  JayaPrakash "J.P" Pakalpati 12/27/2002
   */

  private void stopAnimation()throws Exception
  {
    if (timer.isRunning())
    {
      timer.stop();
      setCurrentYear(iStart);
    }
  }

  /** Class animateAction()
   *  <p> A bubble map is always drawn for the current year value.  To create
   *      a bubble map movie the current year value needs to be advanced
   *      (/changed) after a specified time.  Timer is used for generating
   *      action events after a specified delay.  This class encapsulates
   *      the action event. Whenever an action event is generated, the value
   *      of the current year is incremented and the bubble map is redrawn
   *      creating an animation effect.
   *
   *  JayaPrakash "J.P" Pakalpati 12/27/2002
   */

  class animateAction implements ActionListener
  {
	public void actionPerformed(ActionEvent evt)
	{
	  if (iCurrentYear < iEnd-1)
	    setCurrentYear(iCurrentYear+1);
	  else
	    setCurrentYear(iStart);
	}
  }

  /** Method setCurrentYear(int)
   *  <p> Sets the value of the current year to the specified value.
   *      <br>
   *      Note: Current year is the year for which the bubble map
   *            is displayed on the drawing panel.
   *            A 'bubble map' is a graphical view of cumulative production
   *            for the specified list of leases(bubbles) at a
   *            specific time(year).
   *  @param iYear - year value
   *
   *  JayaPrakash "J.P" Pakalpati 12/27/2002
   */

  private void setCurrentYear(int iYear)
  {
    this.iCurrentYear = iYear;
    lblYear.setText("");
    txtYear.setText("" +this.iCurrentYear);

    if (iAction != STOP)
      lblYear.setText("YEAR: " + this.iCurrentYear);

    panel.setYear(this.iCurrentYear);
  }

  /** Method checkYear()
   *  <p> This method will check to see that the user is not going beyond
   *      the minimum and maximum years of the production data
   *  @param iAction = either Previous or Next Action
   */

   private void checkYear(int iAction)
   {
	 switch (iAction)
	 {
	   case PREVIOUS:
	     if (iCurrentYear-1 < iStart)
	       JOptionPane.showMessageDialog((Component) null,
		                                  "Can not Exceed Minimum Year of Production Data",
		                                  "WARNING",
		                                  JOptionPane.WARNING_MESSAGE);
         else
           setCurrentYear(iCurrentYear-1);
	     break;

	   case NEXT:
	     if (iCurrentYear+1 > iEnd)
	       JOptionPane.showMessageDialog((Component) null,
		                                  "Can not Exceed Maximum Year of Production Data",
		                                  "WARNING",
		                                  JOptionPane.WARNING_MESSAGE);
         else
           setCurrentYear(iCurrentYear+1);
	     break;
	 }
   }

  /* ===================================================================== *
   * ---------------------------- ACTIONS -------------------------------- *
   * ===================================================================== */

  /** METHOD actionPerformed()
   *  <p> This Method will handle all the actions within the Frame.
   *  @param event - Action Event
   */

  public void actionPerformed(ActionEvent event)
  {
    if (event.getSource() == rbOIL) { setProductionPlotData(OIL); }
    if (event.getSource() == rbGAS) { setProductionPlotData(GAS); }

    if (event.getSource() == btnStop)
    {
	  iAction = STOP;
	  panel.setPlay(STOP);

	  try
	  {
		stopAnimation();
	  }
	  catch(Exception e) {}
	}

    if (event.getSource() == btnPause)
    {
	  iAction = PAUSE;
	  panel.setPlay(PAUSE);

	  try
	  {
		pauseAnimation();
	  }
	  catch(Exception e) {}
	}

    if (event.getSource() == btnRun)
    {
	  iAction = RUN;
	  panel.setPlay(RUN);

	  try
	  {
		startAnimation();
	  }
	  catch(Exception e) {}
	}

    if (event.getSource() == btnPrev) { checkYear(PREVIOUS); }
    if (event.getSource() == btnNext) { checkYear(NEXT); }

    setButtons();
  }

  /** METHOD focusLost()
   *  <p> This Method will handle the lost focus event for the text fields.
   *  @param e      = Focus Lost Event
   */

  protected void focusLost(FocusEvent e)
  {
    int    iNumeric = 0;
    String sMessage = new String("");
    String sTemp    = new String("");
    int    iYear    = 0;

    if (e.getSource() == txtYear)
    {
      iNumeric = 1;
      sTemp    = txtYear.getText();
      sMessage = new String("The Year Value is a Numeric Field");
    }

    if (iNumeric == 1)
    {
      if (!cmn.cmnString.isNumeric(sTemp))
      {
        JOptionPane.showMessageDialog((Component) null,
                                       sMessage,
                                       "ERROR",
                                       JOptionPane.ERROR_MESSAGE);
        txtYear.setText(""+iCurrentYear);
	  }
	  else
	  {
		iYear = cmn.cmnString.stringToInt(sTemp);

	    if (iYear < iStart)
	    {
	      JOptionPane.showMessageDialog((Component) null,
		                                "Can not Exceed Minimum Year of Production Data",
		                                "WARNING",
		                                JOptionPane.WARNING_MESSAGE);
		  txtYear.setText(""+iCurrentYear);
	    }
		else if (iYear > iEnd)
		{
	       JOptionPane.showMessageDialog((Component) null,
		                                  "Can not Exceed Maximum Year of Production Data",
		                                  "WARNING",
		                                  JOptionPane.WARNING_MESSAGE);
		  txtYear.setText(""+iCurrentYear);
	    }
        else
           setCurrentYear( iYear );
      }
	}
  }

  /** Method update()
   * <p> This method will handle the Observable Actions
   * @param obj = The Observable making the call
   * @param arg = The returned object
   */

  public void update(Observable obj, Object arg)
  {
	int    icnt = -1;
    String sArg = new String( (String) arg);

    if (sArg.equals("Well Selected"))
    {
	  ProductionPlot( panel.getLeaseID() );
	}
  }
}

/** CLASS geminiBubbleMapFrameObservable()
  *  <p> This Class will handle Observable Actions when action occurs on a
  *      frame or panel
  */

class geminiBubbleMapFrameObservable extends Observable
{
  public void notifyObservers(Object b)
  {
    setChanged();
    super.notifyObservers(b);
  }
}

/** CLASS geminiBubbleMapFrameFocusAdapter()
  *  <p> This Class will handle Actions when focus is lost on a textfield
  */

class geminiBubbleMapFrameFocusAdapter extends java.awt.event.FocusAdapter
{
  geminiBubbleMapFrame adaptee;

  geminiBubbleMapFrameFocusAdapter(geminiBubbleMapFrame adaptee)
  {
    this.adaptee = adaptee;
  }

  public void focusLost(FocusEvent e) { adaptee.focusLost(e); }
}

/*
 *  @version 1.1 01/24/2014
 *  @author  John Victorine
 */
