/*
 * @productionDeclinePanel.java Version 1.1 09/12/2013
 *
 * Copyright (c) 2013 Kansas Geological Survey
 * 1930 Constant Avenue, Lawrence, Kansas, 66047, U.S.A.
 * All Rights Reserved.
 */

package production.gui;

import java.net.*;
import java.awt.*;
import java.awt.image.*;
import java.awt.event.*;
import java.util.Observable;
import javax.swing.*;
import javax.swing.event.*;
import javax.swing.border.*;

import production.plot.productionPlotStruct;  // Production Plot Data Structure
import production.declineStruct;              // Decline Curve Data Structure

/** Class productionDeclinePanel
 *  <p> This Class will create the Decline Curve Calculator Panel.
 *
 *  @version 1.1 09/12/2013
 *  @author  John R. Victorine
 */

public class productionDeclinePanel extends JPanel implements ActionListener
{
  public static final int OIL         = production.productionStruct.OIL;
  public static final int GAS         = production.productionStruct.GAS;

  public static final int OFF         = cmn.cmnStruct.OFF;
  public static final int ON          = cmn.cmnStruct.ON;

  public static final int DAILY       = production.math.declineMath.DAILY;
  public static final int MONTHLY     = production.math.declineMath.MONTHLY;
  public static final int YEARLY      = production.math.declineMath.YEARLY;

  public static final int EXPONENTIAL = production.math.declineMath.EXPONENTIAL;
  public static final int HARMONIC    = production.math.declineMath.HARMONIC;
  public static final int HYPERBOLIC  = production.math.declineMath.HYPERBOLIC;

  // Input Variables

  private Observable           notifier  = null; // Observalbles Object
  private productionPlotStruct stPlot    = null; // Production Plot Data Structure

  // Global Variables

  private declineStruct        stDecline = null; // Decline Curve Data Structure

  // -- Product

  private int    iProduct     = OIL;

  // -- Production Rate for Computation

  private int    iRate        = YEARLY;

  private int    iRateDefault = YEARLY;
  private double dQiDefault   = 0.0;
  private double dQtDefault   = 0.0;

  // -- Decline Curve Equation

  private static final String EQUATION_TEXT[] = production.declineStruct.EQUATION_TEXT;

  // Cumulative & Economic Limit

  private static final double DEFAULT[] = { 15.0, 30.0 };
  private static final String PER[]     = { "BBL Per", "MCF Per" };

  private int    iLimitRate = DAILY;

  // Extend to Limit Data

  public static final int NORMAL = 0;
  public static final int EXTEND = 1;

  public int iExtend = NORMAL;
  public int iYear   = 0;

  // Screen Width

  private int iControlWidth = 400;  // Control Panel Width
  private int iScreenWidth  = 0;    // Screen Width
  private int iScreenHigh   = 0;    // Scree Height

  // Global Frame Widgets

  // -- Center Panel: Decline Curve Analysis

  // .. Decline Curve Methods

  private static final String srbMethods[] = { "Exponential", "Harmonic", "Hyperbolic" };

  private JRadioButton rbMethods[] = null;         // Decline Curve Analysis Methods

  // .. Production Rate for Computations

  private static final String srbRate[] = { "Daily", "Monthly", "Yearly" };

  private JRadioButton rbRate[] = null;            // Production Rate By

  // .. Production Rate for Computation

  private JTextField txtQiRate  = new JTextField();  // Initial Production (Qi) Rate
  private JTextField txtQiYear  = new JTextField();  // Initial Production (Qi) Year

  private JTextField txtQtRate  = new JTextField();  // Present Production (Qt) Rate
  private JTextField txtQtYear  = new JTextField();  // Present Production (Qt) Year

  // .. Decline Curve Equation

  private JLabel     lblEquation = new JLabel();     // Decline Curve Equation

  private JTextField txtK        = new JTextField(); // Decline Constant (K)
  private JTextField txtn        = new JTextField(); // Decline Exponent (n)

  // .. Cumulative Production

  private JLabel     lblActual   = new JLabel();     // Cummulative Production (Actual) Units
  private JLabel     lblComputed = new JLabel();     // Cummulative Production (Computed) Units

  private JTextField txtActual   = new JTextField(); // Cummulative Production (Actual)
  private JTextField txtComputed = new JTextField(); // Cummulative Production (Computed)

  // .. Production at Economic Limit Rate

  private static final String srbLimitRate[] = { "Day", "Month", "Year" };

  private JRadioButton rbLimitRate[] = null;         // Production Limit Rate By

  private JLabel     lblQf     = new JLabel();       // Limit Rate By

  private JTextField txtQfRate = new JTextField();   // Production Economic Limit Rate (Qf)
  private JTextField txtQfDate = new JTextField();   // Production Economic Limit Date

  private JLabel     lblEUR    = new JLabel();       // Cummulative @ Economic Limit (EUR) Units
  private JLabel     lblRR     = new JLabel();       // Remaining Reserves (RR) Units

  private JTextField txtEUR    = new JTextField();   // Cummulative @ Economic Limit (EUR)
  private JTextField txtRR     = new JTextField();   // Remaining Reserves (RR)

  // .. Extend Limit

  private JButton    btnExtend = new JButton();      // Extend to Limit Button

  /** Constructor productionDeclinePanel()
   *  <p> This is the Constructor for this class.
   *  @param notifier = observer event class
   *  @param stPlot   = Production Plot data Structure
   */

  public productionDeclinePanel(Observable notifier, productionPlotStruct stPlot)
  {
    try
    {
	  this.notifier = notifier;
	  this.stPlot   = stPlot;

      setData( stPlot );

      jbInit();
    }
    catch (Exception e)
    {
      e.printStackTrace();
    }
  }

  /** METHOD jbInit()
   *  <p> This Method will create the Frame for this class.
   *  @throws Exception
   */

  private void jbInit() throws Exception
  {
	int i, j;

    JPanel pnlCenter         = new JPanel();

    // Build Production Plot Frame

    this.setLayout(new BorderLayout());

    // Decline Curve Analysis Panel

    pnlCenter = buildDeclineCurveAnalysisPanel();

    // Add Widgets to Frame

    this.add( pnlCenter,    BorderLayout.CENTER );
  }

  /** Method buildDeclineCurveAnalysisPanel()
   *  <p> This method will build the Decline Curve Analysis Panel
   */

  private JPanel buildDeclineCurveAnalysisPanel()
  {
	int i, j;

    // .. Initial Productiont Rate (Qi)

    double Qi         = 0.0;
    int    Qi_year    = 0;

    // .. Final Productiont Rate (Qt)

    double Qt         = 0.0;
    int    Qt_year    = 0;

    // .. Decline Curve Parameters

    double K          = 0.0;  // Decline Constant
    double n          = 0.0;  // Decline Exponent

    int    Qf_year    = 0;    // Production Date @ Economic Limit

    double CUMM_A     = 0.0;  // Cumulative Actual
    double CUMM_C     = 0.0;  // Cumulative Computed

    double EUR        = 0.0;  // Cumulative @ Economic Limit (EUR)
    double RR         = 0.0;  // Remaining Reserves (RR)

	JPanel panel               = new JPanel();
    JPanel pnlTop              = new JPanel();
    JPanel pnlMethods          = new JPanel();

    JPanel pnlProductionRate   = new JPanel();
    JPanel pnlRate             = new JPanel();

    JPanel pnlRateLimits       = new JPanel();
    JPanel pnlQi               = new JPanel();
    JPanel pnlQiRate           = new JPanel();
    JPanel pnlQiYear           = new JPanel();

    JPanel pnlQt               = new JPanel();
    JPanel pnlQtRate           = new JPanel();
    JPanel pnlQtYear           = new JPanel();

    JPanel pnlDecline          = new JPanel();
    JPanel pnlEquation         = new JPanel();
    JPanel pnlParameters       = new JPanel();
    JPanel pnlK                = new JPanel();
    JPanel pnln                = new JPanel();

    JLabel lblt                = new JLabel();     // Variable t

    JPanel pnlCenter           = new JPanel();
    JPanel pnlCumulative       = new JPanel();
    JPanel pnlActual           = new JPanel();
    JPanel pnlComputed         = new JPanel();

    JPanel pnlLimit            = new JPanel();

    JPanel pnlEconomicLimit    = new JPanel();
    JPanel pnlQf               = new JPanel();
    JPanel pnlQfButtons        = new JPanel();
    JPanel pnlQfRate           = new JPanel();
    JPanel pnlQfRateTop        = new JPanel();
    JPanel pnlQfDate           = new JPanel();
    JPanel pnlLimitRate        = new JPanel();

    JLabel lblQfRate           = new JLabel();
    JLabel lblQfDate           = new JLabel();

    JPanel pnlSummary          = new JPanel();
    JPanel pnlEUR              = new JPanel();
    JPanel pnlRR               = new JPanel();

    JPanel pnlBase             = new JPanel();

    ButtonGroup groupMethods   = new ButtonGroup();
    ButtonGroup groupRate      = new ButtonGroup();
    ButtonGroup groupLimitRate = new ButtonGroup();

    TitledBorder titledBorderDecline = new TitledBorder(
        new EtchedBorder(EtchedBorder.RAISED,
                         Color.white,
                         new Color(165, 163, 151)),
        "Production Decline Curve Calculator");
    titledBorderDecline.setTitleFont(new java.awt.Font("Dialog", 1, 14));

    TitledBorder titledBorderMethods = new TitledBorder(
        BorderFactory.createEtchedBorder(Color.white, new Color(165, 163, 151)),
        "Decline Curve Analysis Methods:");
    titledBorderMethods.setTitleFont(new java.awt.Font("Dialog", 1, 13));

    TitledBorder titledBorderRate = new TitledBorder(
        BorderFactory.createEtchedBorder(Color.white, new Color(165, 163, 151)),
        "Enter Production Rate for Computations:");
    titledBorderRate.setTitleFont(new java.awt.Font("Dialog", 1, 13));

    TitledBorder titledBorderQi = new TitledBorder(
        BorderFactory.createEtchedBorder(Color.white, new Color(165, 163, 151)),
        "Initial (Qi):");
    titledBorderQi.setTitleFont(new java.awt.Font("Dialog", 1, 13));

    TitledBorder titledBorderQt = new TitledBorder(
        BorderFactory.createEtchedBorder(Color.white, new Color(165, 163, 151)),
        "Final (Qt):");
    titledBorderQt.setTitleFont(new java.awt.Font("Dialog", 1, 13));

    TitledBorder titledBorderQRate = new TitledBorder(
		BorderFactory.createEmptyBorder(), "Rate:");

    TitledBorder titledBorderQYear = new TitledBorder(
		BorderFactory.createEmptyBorder(), "Year:");

    TitledBorder titledBorderQDate = new TitledBorder(
		BorderFactory.createEmptyBorder(), "Date:");

    TitledBorder titledBorderEquation = new TitledBorder(
        BorderFactory.createEtchedBorder(Color.white, new Color(165, 163, 151)),
        "Decline Curve Equation:");
    titledBorderEquation.setTitleFont(new java.awt.Font("Dialog", 1, 13));

    TitledBorder titledBorderK = new TitledBorder(
        BorderFactory.createEtchedBorder(Color.white, new Color(165, 163, 151)),
        "Constant (K):");
    titledBorderK.setTitleFont(new java.awt.Font("Dialog", 1, 13));

    TitledBorder titledBordern = new TitledBorder(
        BorderFactory.createEtchedBorder(Color.white, new Color(165, 163, 151)),
        "Exponent (n):");
    titledBordern.setTitleFont(new java.awt.Font("Dialog", 1, 13));

    TitledBorder titledBorderCumulative = new TitledBorder(
        BorderFactory.createEtchedBorder(Color.white, new Color(165, 163, 151)),
        "Cumulative Production:");
    titledBorderCumulative.setTitleFont(new java.awt.Font("Dialog", 1, 13));

    TitledBorder titledBorderActual = new TitledBorder(
        BorderFactory.createEtchedBorder(Color.white, new Color(165, 163, 151)),
        "Production (Actual):");
    titledBorderActual.setTitleFont(new java.awt.Font("Dialog", 1, 13));

    TitledBorder titledBorderComputed = new TitledBorder(
        BorderFactory.createEtchedBorder(Color.white, new Color(165, 163, 151)),
        "Present (Computed):");
    titledBorderComputed.setTitleFont(new java.awt.Font("Dialog", 1, 13));

    TitledBorder titledBorderLimit = new TitledBorder(
        BorderFactory.createEtchedBorder(Color.white, new Color(165, 163, 151)),
        "Production at Economic Limit:");
    titledBorderLimit.setTitleFont(new java.awt.Font("Dialog", 1, 13));

    TitledBorder titledBorderEconomic = new TitledBorder(
        BorderFactory.createEtchedBorder(Color.white, new Color(165, 163, 151)),
        "Economic Limit Rate (Qf):");
    titledBorderEconomic.setTitleFont(new java.awt.Font("Dialog", 1, 13));

    TitledBorder titledBorderEUR = new TitledBorder(
        BorderFactory.createEtchedBorder(Color.white, new Color(165, 163, 151)),
        "Economic Limit (EUR):");
    titledBorderEUR.setTitleFont(new java.awt.Font("Dialog", 1, 13));

    TitledBorder titledBorderRR = new TitledBorder(
        BorderFactory.createEtchedBorder(Color.white, new Color(165, 163, 151)),
        "Reserves (RR):");
    titledBorderRR.setTitleFont(new java.awt.Font("Dialog", 1, 13));

    if (stDecline != null)
    {
      Qi         = stDecline.Qi;
      Qi_year    = stDecline.Qi_year;

    // .. Final Productiont Rate (Qt)

      Qt         = stDecline.Qt;
      Qt_year    = stDecline.Qt_year;

      // .. Decline Curve Parameters

      K          = stDecline.K;       // Decline Constant
      n          = stDecline.n;       // Decline Exponent

      CUMM_A     = stDecline.CUMM_A;  // Cumulative Actual
      CUMM_C     = stDecline.CUMM_C;  // Cumulative Computed

      Qf_year    = stDecline.Qf_year; // Economic Limit Year Date

      EUR        = stDecline.EUR;     // Cumulative @ Economic Limit (EUR)
      RR         = stDecline.RR;      // Remaining Reserves (RR)
	}

    // Build Decline Curve Analysis Panel

    panel.setLayout(new BorderLayout());
    panel.setBorder(titledBorderDecline);

    // Top Panel

    pnlTop.setLayout(new BorderLayout());

    // -- Decline Curve Analysis Methods Panel

    pnlMethods.setLayout(new GridLayout());
    pnlMethods.setBorder(titledBorderMethods);

    rbMethods = new JRadioButton[3];

    for (i=0; i<3; i++)
    {
	  rbMethods[i] = new JRadioButton();
      rbMethods[i].setFont(new java.awt.Font("Dialog", 1, 12));
      rbMethods[i].setText(srbMethods[i]);
      rbMethods[i].addActionListener(this);
      if (i==0) rbMethods[i].setSelected(true);

      groupMethods.add(rbMethods[i]);

      pnlMethods.add(rbMethods[i], null);
    }

    // -- Production Rate for Computations

    pnlProductionRate.setLayout(new BorderLayout());
    pnlProductionRate.setBorder(titledBorderRate);

    // .. Production Rate By Panel

    pnlRate.setLayout(new GridLayout(3,1));

    rbRate = new JRadioButton[3];

    for (i=0; i<3; i++)
    {
	  rbRate[i] = new JRadioButton();
      rbRate[i].setFont(new java.awt.Font("Dialog", 1, 12));
      rbRate[i].setText(srbRate[i]);
      rbRate[i].addActionListener(this);
      if (i==2) rbRate[i].setSelected(true);

      groupRate.add(rbRate[i]);

      pnlRate.add(rbRate[i], null);
    }

    // .. Production Rate Limits Panel

    pnlRateLimits.setLayout(new GridLayout());

    // .. .. Initial Production (Qi) Panel

    pnlQi.setLayout(new BorderLayout());
    pnlQi.setBorder(titledBorderQi);

    pnlQiRate.setLayout(new BorderLayout());
    pnlQiRate.setBorder(titledBorderQRate);

    txtQiRate.setText("" + (int) Qi);
    txtQiRate.setHorizontalAlignment(SwingConstants.RIGHT);
    txtQiRate.setFont(new java.awt.Font("Dialog", 1, 12));
    txtQiRate.addFocusListener(new productionDeclinePanelFocusAdapter(this));

    pnlQiYear.setLayout(new BorderLayout());
    pnlQiYear.setBorder(titledBorderQYear);

    txtQiYear.setText("" + Qi_year);
    txtQiYear. setColumns(4);
    txtQiYear.setHorizontalAlignment(SwingConstants.CENTER);
    txtQiYear.setFont(new java.awt.Font("Dialog", 1, 12));
    txtQiYear.addFocusListener(new productionDeclinePanelFocusAdapter(this));

    // .. .. Final Production (Qt) Panel

    pnlQt.setLayout(new BorderLayout());
    pnlQt.setBorder(titledBorderQt);

    pnlQtRate.setLayout(new BorderLayout());
    pnlQtRate.setBorder(titledBorderQRate);

    txtQtRate.setText("" + (int) Qt);
    txtQtRate.setHorizontalAlignment(SwingConstants.RIGHT);
    txtQtRate.setFont(new java.awt.Font("Dialog", 1, 12));
    txtQtRate.addFocusListener(new productionDeclinePanelFocusAdapter(this));

    pnlQtYear.setLayout(new BorderLayout());
    pnlQtYear.setBorder(titledBorderQYear);

    txtQtYear.setText("" + Qt_year);
    txtQtYear. setColumns(4);
    txtQtYear.setHorizontalAlignment(SwingConstants.CENTER);
    txtQtYear.setFont(new java.awt.Font("Dialog", 1, 12));
    txtQtYear.addFocusListener(new productionDeclinePanelFocusAdapter(this));

    // .. Decline Curve Equation Panel

    pnlDecline.setLayout(new BorderLayout());
    pnlDecline.setBorder(titledBorderEquation);

    // .. .. Equation Panel

    pnlEquation.setLayout(new GridLayout(2,1));

    lblEquation.setText(EQUATION_TEXT[stDecline.iMethod]);
    lblEquation.setFont(new java.awt.Font("Dialog", 1, 12));

    lblt.setText("t=time");
    lblt.setHorizontalAlignment(SwingConstants.CENTER);
    lblt.setFont(new java.awt.Font("Dialog", 1, 12));

    // .. .. Decline Curve Parameters Panel

    pnlParameters.setLayout(new GridLayout());

    // .. - .. Decline Constant (K) Panel

    pnlK.setLayout(new BorderLayout());
    pnlK.setBorder(titledBorderK);

    txtK.setText(""+K);
    txtK.setEditable(false);
    txtK.setHorizontalAlignment(SwingConstants.RIGHT);
    txtK.setFont(new java.awt.Font("Dialog", 1, 12));
    txtK.addFocusListener(new productionDeclinePanelFocusAdapter(this));

    // .. - .. Decline Exponent (n) Panel

    pnln.setLayout(new BorderLayout());
    pnln.setBorder(titledBordern);

    txtn.setText(""+n);
    txtn.setEditable(false);
    txtn.setHorizontalAlignment(SwingConstants.RIGHT);
    txtn.setFont(new java.awt.Font("Dialog", 1, 12));
    txtn.addFocusListener(new productionDeclinePanelFocusAdapter(this));

    // Center Panel

    pnlCenter.setLayout(new BorderLayout());

    // .. Cumulative Production Panel

    pnlCumulative.setLayout(new GridLayout());
    pnlCumulative.setBorder(titledBorderCumulative);

    pnlActual.setLayout(new BorderLayout());
    pnlActual.setBorder(titledBorderActual);

    txtActual.setText("" + (int) CUMM_A);
    txtActual.setEditable(false);
    txtActual.setHorizontalAlignment(SwingConstants.RIGHT);
    txtActual.setFont(new java.awt.Font("Dialog", 1, 12));
    txtActual.addFocusListener(new productionDeclinePanelFocusAdapter(this));

    lblActual.setText(production.productionStruct.UNITS[iProduct]);
    lblActual.setFont(new java.awt.Font("Dialog", 1, 12));

    pnlComputed.setLayout(new BorderLayout());
    pnlComputed.setBorder(titledBorderComputed);

    txtComputed.setText("" + (int) CUMM_C);
    txtComputed.setEditable(false);
    txtComputed.setHorizontalAlignment(SwingConstants.RIGHT);
    txtComputed.setFont(new java.awt.Font("Dialog", 1, 12));
    txtComputed.addFocusListener(new productionDeclinePanelFocusAdapter(this));

    lblComputed.setText(production.productionStruct.UNITS[iProduct]);
    lblComputed.setFont(new java.awt.Font("Dialog", 1, 12));

    // .. Production at Economic Limit Panel

    pnlLimit.setLayout(new BorderLayout());
    pnlLimit.setBorder(titledBorderLimit);

    pnlEconomicLimit.setLayout(new BorderLayout());
    pnlEconomicLimit.setBorder(titledBorderEconomic);

    // .. .. Economic Limit Rate (Qf)

    pnlQfRateTop.setLayout(new BorderLayout());
//    pnlQfRateTop.setBorder(titledBorderQRate);

    pnlQfRate.setLayout(new BorderLayout());
    pnlQfRate.setBorder(titledBorderQRate);

    lblQfRate.setText("Rate: ");
    lblQfRate.setFont(new java.awt.Font("Dialog", 1, 12));

    txtQfRate.setText(""+ (int) DEFAULT[iProduct]);
    txtQfRate. setColumns(6);
    txtQfRate.setHorizontalAlignment(SwingConstants.RIGHT);
    txtQfRate.setFont(new java.awt.Font("Dialog", 1, 12));
    txtQfRate.addFocusListener(new productionDeclinePanelFocusAdapter(this));

    // .. .. Economic Limit Rate Center Panel

    pnlQf.setLayout(new BorderLayout());

    // .. - .. Economic Limit Rate (Qf) Buttons Panel

//    pnlLimitRate.setLayout(new GridLayout(3,1));

    lblQf.setText(PER[iProduct]);
    lblQf.setFont(new java.awt.Font("Dialog", 1, 10));

    rbLimitRate = new JRadioButton[3];

    for (i=0; i<3; i++)
    {
	  rbLimitRate[i] = new JRadioButton();
      rbLimitRate[i].setFont(new java.awt.Font("Dialog", 1, 11));
      rbLimitRate[i].setText(srbLimitRate[i]);
      rbLimitRate[i].addActionListener(this);
      if (i==0) rbLimitRate[i].setSelected(true);

      groupLimitRate.add(rbLimitRate[i]);

      pnlLimitRate.add(rbLimitRate[i], null);
    }

    // .. .. Economic Limit Date

    pnlQfDate.setLayout(new BorderLayout());
    pnlQfDate.setBorder(titledBorderQDate);

    lblQfDate.setText("Date: ");
    lblQfDate.setFont(new java.awt.Font("Dialog", 1, 12));

    txtQfDate.setText(""+Qf_year);
    txtQfDate. setColumns(4);
    txtQfDate.setEditable(false);
    txtQfDate.setHorizontalAlignment(SwingConstants.CENTER);
    txtQfDate.setFont(new java.awt.Font("Dialog", 1, 12));
    txtQfDate.addFocusListener(new productionDeclinePanelFocusAdapter(this));

    // .. Cumulative Summary

    pnlSummary.setLayout(new GridLayout());

    // .. .. Cumulative at Economic Limit (EUR) Panel

    pnlEUR.setLayout(new BorderLayout());
    pnlEUR.setBorder(titledBorderEUR);

    txtEUR.setText("" + (int) EUR);
    txtEUR.setEditable(false);
    txtEUR.setHorizontalAlignment(SwingConstants.RIGHT);
    txtEUR.setFont(new java.awt.Font("Dialog", 1, 12));
    txtEUR.addFocusListener(new productionDeclinePanelFocusAdapter(this));

    lblEUR.setText(production.productionStruct.UNITS[iProduct]);
    lblEUR.setFont(new java.awt.Font("Dialog", 1, 12));

    // .. .. Remaining Reserves (RR) Panel

    pnlRR.setLayout(new BorderLayout());
    pnlRR.setBorder(titledBorderRR);

    txtRR.setText("" + (int) RR);
    txtRR.setEditable(false);
    txtRR.setHorizontalAlignment(SwingConstants.RIGHT);
    txtRR.setFont(new java.awt.Font("Dialog", 1, 12));
    txtRR.addFocusListener(new productionDeclinePanelFocusAdapter(this));

    lblRR.setText(production.productionStruct.UNITS[iProduct]);
    lblRR.setFont(new java.awt.Font("Dialog", 1, 12));

    // -- Base Panel

    btnExtend.setFont(new java.awt.Font("Dialog", 1, 11));
    btnExtend.setText("Extend to Limit");
    btnExtend.addActionListener(this);

    // Add Widgets to Frame

    panel.add(pnlTop,                    BorderLayout.NORTH);

    // -- Top Panel

    pnlTop.add(pnlMethods,               BorderLayout.NORTH);

    pnlTop.add(pnlProductionRate,        BorderLayout.CENTER);
    pnlProductionRate.add(pnlRate,       BorderLayout.WEST);

    pnlProductionRate.add(pnlRateLimits, BorderLayout.CENTER);
    pnlRateLimits.add(pnlQi,             null);
    pnlQi.add(pnlQiRate,                 BorderLayout.CENTER);
    pnlQiRate.add(txtQiRate,             BorderLayout.CENTER);

    pnlQi.add(pnlQiYear,                 BorderLayout.EAST);
    pnlQiYear.add(txtQiYear,             BorderLayout.CENTER);

    pnlRateLimits.add(pnlQt,             null);
    pnlQt.add(pnlQtRate,                 BorderLayout.CENTER);
    pnlQtRate.add(txtQtRate,             BorderLayout.CENTER);

    pnlQt.add(pnlQtYear,                 BorderLayout.EAST);
    pnlQtYear.add(txtQtYear,             BorderLayout.CENTER);

    pnlTop.add(pnlDecline,               BorderLayout.SOUTH);
    pnlDecline.add(pnlEquation,          BorderLayout.WEST);
    pnlEquation.add(lblEquation,         null);
    pnlEquation.add(lblt,                null);

    pnlDecline.add(pnlParameters,        BorderLayout.CENTER);
    pnlParameters.add(pnlK,              null);
    pnlK.add(txtK,                       BorderLayout.CENTER);

    pnlParameters.add(pnln,              null);
    pnln.add(txtn,                       BorderLayout.CENTER);

    // -- Center Panel

    panel.add(pnlCenter,                 BorderLayout.CENTER);
    pnlCenter.add(pnlCumulative,         BorderLayout.NORTH);
    pnlCumulative.add(pnlActual,         null);
    pnlActual.add(txtActual,             BorderLayout.CENTER);
    pnlActual.add(lblActual,             BorderLayout.EAST);

    pnlCumulative.add(pnlComputed,       null);
    pnlComputed.add(txtComputed,         BorderLayout.CENTER);
    pnlComputed.add(lblComputed,         BorderLayout.EAST);

    pnlCenter.add(pnlLimit,              BorderLayout.CENTER);
    pnlLimit.add(pnlEconomicLimit,       BorderLayout.NORTH);
//    pnlEconomicLimit.add(pnlQfRateTop,   BorderLayout.NORTH);
    pnlEconomicLimit.add(pnlQfRate,      BorderLayout.WEST);
//    pnlQfRateTop.add(pnlQfRate,          BorderLayout.WEST);
//    pnlQfRate.add(lblQfRate,             BorderLayout.WEST);
    pnlQfRate.add(txtQfRate,             BorderLayout.CENTER);
    pnlQfRate.add(lblQf,                 BorderLayout.EAST);
//    pnlQfRateTop.add(pnlLimitRate,       BorderLayout.CENTER);

    pnlEconomicLimit.add(pnlQf,          BorderLayout.CENTER);
    pnlQf.add(pnlLimitRate,              BorderLayout.CENTER);

    pnlEconomicLimit.add(pnlQfDate,      BorderLayout.EAST);
//    pnlQfDate.add(lblQfDate,             BorderLayout.WEST);
    pnlQfDate.add(txtQfDate,             BorderLayout.CENTER);

    pnlLimit.add(pnlSummary,             BorderLayout.SOUTH);
    pnlSummary.add(pnlEUR,               null);
    pnlEUR.add(txtEUR,                   BorderLayout.CENTER);
    pnlEUR.add(lblEUR,                   BorderLayout.EAST);

    pnlSummary.add(pnlRR,                null);
    pnlRR.add(txtRR,                     BorderLayout.CENTER);
    pnlRR.add(lblRR,                     BorderLayout.EAST);

    // -- Base Panel

    panel.add(pnlBase,                   BorderLayout.SOUTH);
    pnlBase.add(btnExtend,               null);

	return (panel);
  }

  /** Method close()
   * <p> This method will close all dialogs that are opened by this class.
   */

  public void close()
  {
    // Input Variables

    notifier = null;
    stPlot   = null;

    // Global Variables

    stDecline   = null; // Decline Curve Data Structure

    // Global Frame Widgets
    // -- Center Panel: Decline Curve Analysis
    // .. Decline Curve Methods

    rbMethods   = null; // Decline Curve Analysis Methods

    // .. Production Rate for Computations


    rbRate      = null; // Production Rate By

    // .. Production Rate for Computation

    txtQiRate   = null; // Initial Production (Qi) Rate
    txtQiYear   = null; // Initial Production (Qi) Year

    txtQtRate   = null; // Present Production (Qt) Rate
    txtQtYear   = null; // Present Production (Qt) Year

    // .. Decline Curve Equation

    lblEquation = null; // Decline Curve Equation

    txtK        = null; // Decline Constant (K)
    txtn        = null; // Decline Exponent (n)

    // .. Cumulative Production

    lblActual   = null; // Cummulative Production (Actual) Units
    lblComputed = null; // Cummulative Production (Computed) Units

    txtActual   = null; // Cummulative Production (Actual)
    txtComputed = null; // Cummulative Production (Computed)

    // .. Production at Economic Limit Rate

    rbLimitRate = null; // Production Limit Rate By

    lblQf       = null; // Limit Rate By

    txtQfRate   = null; // Production Economic Limit Rate (Qf)
    txtQfDate   = null; // Production Economic Limit Date

    lblEUR      = null; // Cummulative @ Economic Limit (EUR) Units
    lblRR       = null; // Remaining Reserves (RR) Units

    txtEUR      = null; // Cummulative @ Economic Limit (EUR)
    txtRR       = null; // Remaining Reserves (RR)

    // .. Extend Limit

    btnExtend   = null; // Extend to Limit Button
  }

  /* ===================================================================== *
   * -------------------------- GET METHODS ------------------------------ *
   * ===================================================================== */

  /** Method getData()
   *  <p> This method will return the Decline Curve Data Structure
   *  @param stDecline = Decline Curve Data Structure
   */

  public declineStruct getData() { return (stDecline); }

  /** Method extendDecline()
   *  <p> This method will control the outer limits
   *  @param iDate = the outer limit of plot
   */

  public int getDeclineYear()
  {
	int iDate = 0;

	switch (iExtend)
	{
	  case NORMAL:
	    iDate = iYear;
	    break;
      case EXTEND:
	    iDate = stDecline.Qf_year;
	    if (iDate < iYear)
	      iDate = iYear;
        break;
	}

    return (iDate);
  }

  /** Method getRate()
   *  <p> This method will return the rate value
   */

  public int getRate() { return(iRate); }

  /** Method initializeQi()
   *  <p> This method will initialize the Initial production rate (Qi)
   */

  private void initializeQi()
  {
	int i = 0;

	if (stPlot.iRows > 3)
	{
	  i = stPlot.iRows-3;
	}
	else if (stPlot.iRows > 2)
	{
	  i = stPlot.iRows-2;
	}
	else if (stPlot.iRows > 1)
	{
	  i = stPlot.iRows-1;
	}

	if (i > 0)
	{
	  stDecline.Qi = Math.round(
		  production.math.productionMath.multiply(stPlot.dProduction[i], stPlot.iFactor));
      dQiDefault   = stDecline.Qi;

	  stDecline.Qi_year = stPlot.iTime[i];

	  stDecline.CUMM_I = Math.round(
		  production.math.productionMath.multiply(stPlot.dCumulative[i], stPlot.iFactor));
	}
  }

  /** Method initializeQt()
   *  <p> This method will initialize the Present production rate (Qt)
   */

  private void initializeQt()
  {
	int i = 0;

	if (stPlot.iRows > 1)
	{
	  i = stPlot.iRows-1;
	}

	if (i > 0)
	{
	  stDecline.Qt = Math.round(
		  production.math.productionMath.multiply(stPlot.dProduction[i], stPlot.iFactor));
      dQtDefault   = stDecline.Qt;

	  stDecline.Qt_year = stPlot.iTime[i];

	  stDecline.CUMM_A = Math.round(
		  production.math.productionMath.multiply(stPlot.dCumulative[i], stPlot.iFactor));
	}
  }

  /** Method getYearData()
   *  <p> This method will retrieve the Production Rate Data based on Year
   *  @param iQ = Identifier indicating either fine Qi: 0 or Qt: 1
   */

  private void getYearData(int iQ, int iYear)
  {
	if (stPlot != null)
	{
      for (int i=0; i<stPlot.iRows; ++i)
      {
		if (iYear == stPlot.iTime[i])
		{
	      switch (iQ)
	      {
	        case 0: // Get Qi Data
   	          stDecline.Qi = Math.round(production.math.productionMath.multiply(
					           stPlot.dProduction[i], stPlot.iFactor));
              dQiDefault   = stDecline.Qi;
	          stDecline.Qi = production.math.declineMath.computeQf(iRate, iRateDefault, dQiDefault);

	          stDecline.Qi_year = stPlot.iTime[i];

	          stDecline.CUMM_I = Math.round(production.math.productionMath.multiply(
					               stPlot.dCumulative[i], stPlot.iFactor));
	          break;

	        case 1: // Get Qt Data
	          stDecline.Qt = Math.round(production.math.productionMath.multiply(
				               stPlot.dProduction[i], stPlot.iFactor));
              dQtDefault   = stDecline.Qt;
	          stDecline.Qt = production.math.declineMath.computeQf(iRate, iRateDefault, dQtDefault);

	          stDecline.Qt_year = stPlot.iTime[i];
	          break;
		  }
		}
	  }
    }
  }

  /* ===================================================================== *
   * -------------------------- SET METHODS ------------------------------ *
   * ===================================================================== */

  /** Method setData()
   * <p> This method will set the extra data structures
   * @param stPlot = production plot data structure
   */

  public void setData( productionPlotStruct stPlot )
  {
	double Q  = 0.0;

	this.stPlot = stPlot;
	if (stPlot != null)
	{
	  iProduct  = stPlot.iProduct;

	  stDecline = new declineStruct();
	  initializeQi();
	  initializeQt();

      iYear   = stDecline.Qt_year;

      stDecline.decline = production.math.declineMath.computeDecline(
		  stDecline.Qi, stDecline.Qt, stDecline.Qi_year, stDecline.Qt_year);

      stDecline.K = production.math.declineMath.computeK(
		  stDecline.iMethod, iRate, stDecline.n,
          stDecline.Qi, stDecline.Qt, stDecline.Qi_year, stDecline.Qt_year);

      Q = production.math.declineMath.computeQc(
		  stDecline.iMethod, stDecline.n, stDecline.K,
          stDecline.Qi, stDecline.Qt); //, stDecline.Qi_year);

      stDecline.CUMM_C = production.math.productionMath.add(stDecline.CUMM_I, Q);

	  stDecline.Qf = production.math.declineMath.computeQf(iRate, iLimitRate, DEFAULT[iProduct]);

	  stDecline.Qf_year = production.math.declineMath.computeEconomicLimitDate(
	    stDecline.iMethod, iRate, stDecline.n, stDecline.K,
	    stDecline.Qi, stDecline.Qf, stDecline.Qi_year);

      if (stDecline.Qf_year > 0)
      {
        Q = production.math.declineMath.computeQc(
	  	      stDecline.iMethod, stDecline.n, stDecline.K,
              stDecline.Qi, stDecline.Qf); //, stDecline.Qi_year);

        stDecline.EUR = production.math.productionMath.add(stDecline.CUMM_I, Q);
        stDecline.RR  = production.math.productionMath.subtract(stDecline.EUR, stDecline.CUMM_A);
	  }
	}
  }

  /** Method clear()
   *  <p> This method will clear the data from the widgets
   */

  public void clear()
  {
    iRate = YEARLY;
    rbRate[iRate].setSelected(true);

    txtQiRate.setText("");
    txtQiYear.setText("");
    txtQtRate.setText("");
    txtQtYear.setText("");

    rbMethods[EXPONENTIAL].setSelected(true);

    txtn.setText("");
    txtK.setText("");

    txtActual.setText("");
    txtComputed.setText("");

    iLimitRate = DAILY;
    rbLimitRate[iLimitRate].setSelected(true);

    txtQfRate.setText("");
    txtQfDate.setText("");

    txtEUR.setText("");
    txtRR.setText("");
  }

  /** Method resetScreen()
   *  <p> This method will refresh the screen values depending on the data.
   */

  public void resetScreen()
  {
    if(stDecline != null)
    {
      rbRate[iRate].setSelected(true);

      txtQiRate.setText(  "" + (int) stDecline.Qi);
      txtQiYear.setText(  "" + stDecline.Qi_year);
      txtQtRate.setText(  "" + (int) stDecline.Qt);
      txtQtYear.setText(  "" + stDecline.Qt_year);

      rbMethods[stDecline.iMethod].setSelected(true);
      lblEquation.setText(EQUATION_TEXT[stDecline.iMethod]);

      txtn.setText(       "" + stDecline.n);
      txtK.setText(       "" + stDecline.K);

      txtActual.setText(  "" + (int) stDecline.CUMM_A);
      lblActual.setText(production.productionStruct.UNITS[iProduct]);

      txtComputed.setText("" + (int) stDecline.CUMM_C);
      lblComputed.setText(production.productionStruct.UNITS[iProduct]);

      rbLimitRate[iLimitRate].setSelected(true);

      txtQfRate.setText(  "" + DEFAULT[iProduct]);
      lblQf.setText(PER[iProduct]);
      txtQfDate.setText(  "" + stDecline.Qf_year);

      txtEUR.setText(     "" + (int) stDecline.EUR);
      lblEUR.setText(production.productionStruct.UNITS[iProduct]);

      txtRR.setText(      "" + (int) stDecline.RR);
      lblRR.setText(production.productionStruct.UNITS[iProduct]);
	}
  }

  /** Method setScreen()
   *  <p> This method will refresh the screen values depending on the data.
   */

  private void setScreen()
  {
    txtK.setText(       "" + stDecline.K);
    txtActual.setText(  "" + (int) stDecline.CUMM_A);
    txtComputed.setText("" + (int) stDecline.CUMM_C);
    txtQfDate.setText(  "" + stDecline.Qf_year);
    txtEUR.setText(     "" + (int) stDecline.EUR);
    txtRR.setText(      "" + (int) stDecline.RR);
  }

  /** Method setButtons()
   *  <p> This method will enable and disable widgets depending on settings
   */

  private void setButtons()
  {
	stDecline.n = 0.0;
	txtn.setText("" + stDecline.n);
	txtn.setEditable(false);

	if (stDecline.iMethod == HYPERBOLIC)
	{
	  stDecline.n = 1.0;
	  txtn.setText("" + stDecline.n);
  	  txtn.setEditable(true);
    }

    lblEquation.setText(EQUATION_TEXT[stDecline.iMethod]);
  }

  /** Method setRate()
   *  <p> This method will set the rate
   *  @param iNewRate = the Production Rate for Computation
   */

  private void setRate(int iNewRate)
  {
	double Q = 0.0;

	this.iRate      = iNewRate;
	stDecline.iRate = iNewRate;

	stDecline.Qi = production.math.declineMath.computeQf(iNewRate, iRateDefault, dQiDefault);
	stDecline.Qt = production.math.declineMath.computeQf(iNewRate, iRateDefault, dQtDefault);

    txtQiRate.setText(  "" + (int) stDecline.Qi);
    txtQtRate.setText(  "" + (int) stDecline.Qt);

	compute();
  }

  /* ===================================================================== *
   * ---------------------------- COMPUTE -------------------------------- *
   * ===================================================================== */

  /** Method compute()
   * <p> This method will compute the decline variables
   */

  public void compute()
  {
	double Q  = 0.0;
	double Qf = 0.0;

	if (stPlot != null)
	{
      stDecline.decline = production.math.declineMath.computeDecline(
		  stDecline.Qi, stDecline.Qt, stDecline.Qi_year, stDecline.Qt_year);

      stDecline.K = production.math.declineMath.computeK(
		  stDecline.iMethod, iRate, stDecline.n,
          stDecline.Qi, stDecline.Qt, stDecline.Qi_year, stDecline.Qt_year);

      Q = production.math.declineMath.computeQc(
		  stDecline.iMethod, stDecline.n, stDecline.K,
          stDecline.Qi, stDecline.Qt); //, stDecline.Qi_year);

      stDecline.CUMM_C = production.math.productionMath.add(stDecline.CUMM_I, Q);

   	  Qf           = cmn.cmnString.stringToDouble(txtQfRate.getText());
	  stDecline.Qf = production.math.declineMath.computeQf(iRate, iLimitRate, Qf);

	  stDecline.Qf_year = production.math.declineMath.computeEconomicLimitDate(
	    stDecline.iMethod, iRate, stDecline.n, stDecline.K,
	    stDecline.Qi, stDecline.Qf, stDecline.Qi_year);

      stDecline.EUR = 0.0;
      stDecline.RR  = 0.0;

      if (stDecline.Qf_year > 0)
      {
        Q = production.math.declineMath.computeQc(
	  	      stDecline.iMethod, stDecline.n, stDecline.K,
              stDecline.Qi, stDecline.Qf); //, stDecline.Qi_year);

        stDecline.EUR = production.math.productionMath.add(stDecline.CUMM_I, Q);
        stDecline.RR  = production.math.productionMath.subtract(stDecline.EUR, stDecline.CUMM_A);
	  }

      setScreen();

      if (notifier != null)
        notifier.notifyObservers(new String("Decline Curve Data Computed"));
	}
  }

  /* ===================================================================== *
   * ---------------------------- ACTIONS -------------------------------- *
   * ===================================================================== */

  /** METHOD actionPerformed()
   *  <p> This Method will handle all the actions within the Frame.
   *  @param event - Action Event
   */

  public void actionPerformed(ActionEvent event)
  {
	int i=0;

	for (i=0; i<3; i++)
	{
      if (event.getSource() == rbMethods[i])   { stDecline.iMethod = i; }
	}

	for (i=0; i<3; i++)
	{
      if (event.getSource() == rbRate[i])
      {
		setRate(i);
      }
	}

	for (i=0; i<3; i++)
	{
      if (event.getSource() == rbLimitRate[i])
      {
		iLimitRate           = i;
		stDecline.iLimitRate = i;
      }
	}

    if (event.getSource() == btnExtend)
    {
      if (notifier != null)
      {
		switch (iExtend)
		{
		  case NORMAL:
            btnExtend.setText("Reset Plot Limit");
            iExtend = EXTEND;
		    break;
          case EXTEND:
            btnExtend.setText("Extend to Limit");
            iExtend = NORMAL;
            break;
		}

        notifier.notifyObservers(new String("Extend Decline Curve"));
	  }
	}

    setButtons();
	compute();
  }

  /** METHOD focusLost()
   *  <p> This Method will handle the lost focus event for the text fields.
   *  @param e      = Focus Lost Event
   */

  protected void focusLost(FocusEvent e)
  {
    int    iNumeric = 0;
    String sMessage = new String("");
    String sTemp    = new String("");

    if (e.getSource() == txtn)
    {
      iNumeric = 1;
      sTemp    = txtn.getText();
      sMessage = new String("Decline Curve Exponent (n) Value is a Numeric Field");
    }

    if (e.getSource() == txtQiRate)
    {
      iNumeric = 1;
      sTemp    = txtQiRate.getText();
      sMessage = new String("Initial Production Rate (Qi) Value is a Numeric Field");
    }

    if (e.getSource() == txtQiYear)
    {
      iNumeric = 1;
      sTemp    = txtQiYear.getText();
      sMessage = new String("Initial Production Rate (Qi) Year Value is a Numeric Field");
    }

    if (e.getSource() == txtQtRate)
    {
      iNumeric = 1;
      sTemp    = txtQtRate.getText();
      sMessage = new String("Present Production Rate (Qt) Value is a Numeric Field");
    }

    if (e.getSource() == txtQtYear)
    {
      iNumeric = 1;
      sTemp    = txtQtYear.getText();
      sMessage = new String("Present Production Rate (Qt) Year Value is a Numeric Field");
    }

    if (e.getSource() == txtQfRate)
    {
      iNumeric = 1;
      sTemp    = txtQfRate.getText();
      sMessage = new String("Economic Limit Rate (Qf) Value is a Numeric Field");
    }

    if (iNumeric == 1)
    {
      if (!cmn.cmnString.isNumeric(sTemp))
      {
        JOptionPane.showMessageDialog((Component) null,
                                       sMessage,
                                       "ERROR",
                                       JOptionPane.ERROR_MESSAGE);

        if (e.getSource() == txtn)
          txtn.setText("" + (int) stDecline.n);

        if (e.getSource() == txtQiRate)
          txtQiRate.setText("" + (int) stDecline.Qi);

        if (e.getSource() == txtQiYear)
          txtQiYear.setText("" + (int) stDecline.Qi_year);

        if (e.getSource() == txtQtRate)
          txtQtRate.setText("" + (int) stDecline.Qt);

        if (e.getSource() == txtQtYear)
          txtQtYear.setText("" + (int) stDecline.Qt_year);

        if (e.getSource() == txtQfRate)
          txtQfRate.setText("" + (int) stDecline.Qf);
      }
      else
      {
        if (e.getSource() == txtn)
          stDecline.n = cmn.cmnString.stringToDouble(txtn.getText());

        if (e.getSource() == txtQiRate)
          stDecline.Qi = cmn.cmnString.stringToDouble(txtQiRate.getText());

        if (e.getSource() == txtQiYear)
        {
          stDecline.Qi_year = cmn.cmnString.stringToInt(txtQiYear.getText());
          getYearData(0, stDecline.Qi_year);
          txtQiRate.setText("" + (int) stDecline.Qi);
		}

        if (e.getSource() == txtQtRate)
          stDecline.Qt = cmn.cmnString.stringToDouble(txtQtRate.getText());

        if (e.getSource() == txtQtYear)
        {
          stDecline.Qt_year = cmn.cmnString.stringToInt(txtQtYear.getText());
          getYearData(1, stDecline.Qt_year);
          txtQtRate.setText("" + (int) stDecline.Qt);
		}

        if (e.getSource() == txtQfRate)
          stDecline.Qf = cmn.cmnString.stringToDouble(txtQfRate.getText());

        compute();
      }
    }
  }
}

/** CLASS productionDeclinePanelFocusAdapter()
  *  <p> This Class will handle Actions when focus is lost on a textfield
  */

class productionDeclinePanelFocusAdapter extends java.awt.event.FocusAdapter
{
  productionDeclinePanel adaptee;

  productionDeclinePanelFocusAdapter(productionDeclinePanel adaptee)
  {
    this.adaptee = adaptee;
  }

  public void focusLost(FocusEvent e) { adaptee.focusLost(e); }
}

/*
 *  @version 1.1 09/12/2013
 *  @author  John Victorine
 */
