/*
 * @productionPlotPanel.java Version 1.1 09/16/2013
 *
 * Copyright (c) 2013 Kansas Geological Survey
 * 1930 Constant Avenue, Lawrence, Kansas, 66047, U.S.A.
 * All Rights Reserved.
 */

package production.gui;

import java.awt.*;
import java.awt.image.*;
import java.util.Observable;
import javax.swing.*;

import cmn.cmnStruct;
import production.plot.productionPlotStruct;
import production.plot.productionPlot;
import production.declineStruct;              // Decline Curve Data Structure

/** Class productionPlotPanel
 *  <p> This Class will create the stratigraphic Plot.
 *
 *  @version 1.1 09/16/2013
 *  @author  John R. Victorine
 */

public class productionPlotPanel extends JPanel
{
  // Input Variables

  private JFrame parent     = null;

  // Global Variables

  private int iWidth   = 0;
  private int iHeight  = 0;

  private productionPlotStruct stPlot    = null;
  private declineStruct        stDecline = null; // Decline Curve Data Structure
  private productionPlot       plot      = null;

  // Panel Widgets

  private ScrollPane scroll = new ScrollPane();
  private Panel      panel  = new Panel();

  /** Constructor productionPlotPanel()
   *  <p> This is the Constructor for this class.
   *  @param parent    = the frame calling this panel
   *  @param stPlot    = The Production Plot Data Structure
   *  @param stDecline = The Decline Curve Data Structure
   */

  public productionPlotPanel( JFrame parent,
                              productionPlotStruct stPlot,
                              declineStruct stDecline )
  {
    try
    {
      this.parent    = parent;
      this.stPlot    = stPlot;
      this.stDecline = stDecline;

      jbInit();
    }
    catch(Exception e)
    {
      e.printStackTrace();
    }
  }

  /** METHOD jbInit()
   *  <p> This Method will create the Panel for this class.
   *  @throws Exception
   */

  private void jbInit() throws Exception
  {
    Panel pnlCenter = new Panel();

    this.setLayout(new BorderLayout());
    pnlCenter.setLayout(new BorderLayout());

    plot    = new productionPlot( stPlot, stDecline );
    iWidth  = 50 + plot.getPlotWidth();
    iHeight = plot.getPlotHeight();

    panel.setLayout( new BorderLayout() );
    panel.setPreferredSize( new Dimension( iWidth, iHeight ) );

    this.add(pnlCenter,   BorderLayout.CENTER);
    pnlCenter.add(scroll, BorderLayout.CENTER);
    scroll.add(panel,     null);
    panel.add(plot,       BorderLayout.CENTER);
  }


  /** Method close()
   * <p> This method will close all dialogs that are opened by this class.
   */

  public void close()
  {
    stPlot = null;

    if (plot != null)
      plot.close();
    plot    = null;
  }

  /** Method refresh()
   *  <p> This method will refresh the plot
   */

  public void refresh() { if (plot != null) { plot.refresh(); } }

  /* ======================================================================= *
   * ----------------------------- GET METHODS ----------------------------- *
   * ======================================================================= */

  /** Method getPlotWidth()
   * <p> This method will return the Plot Width
   * @return iWidth = The Plot Width
   */

  public int  getPlotWidth()  { return (iWidth); }

  /** Method getPlotHeight()
   * <p> This method will return the Plot Height
   * @return iHeight = The Plot Height
   */

  public int getPlotHeight() { return (iHeight); }

  /** Method getImage()
   * <p> This method will convert this plot to a buffered image
   * @return image = profile plot buffered image
   */

  public BufferedImage getImage()
  {
    BufferedImage image = null;

    if (plot != null) image = plot.getImage();

    return (image);
  }

  /* ======================================================================= *
   * ----------------------------- SET METHODS ----------------------------- *
   * ======================================================================= */

  /** Method setObservable( Observable notifier )
   * <p> This method will set the Observerable Object
   * @param notifier = the observable object
   *

  public void setObservable( Observable notifier )
  {
    if (plot != null) plot.setObservable( notifier );
  }
*/
  /** Method setCommon( cmnStruct stCMN )
   * <p> This method will set the Global Common Data Structure
   *  @param stCMN  = the Global Common Data Structure
   *

  public void setCommon( cmnStruct stCMN )
  {
    if (plot != null) plot.setCommon( stCMN );
  }
*/
  /** Method setTitles()
   * <p> This method will set the titles for this plot
   * @param sTitle1 = The first title
   * @param sTitle2 = The second title
   * @param sTitle3 = The third title
   */

  public void setTitles( String sTitle1, String sTitle2, String sTitle3 )
  {
    plot.setTitles( sTitle1, sTitle2, sTitle3 );
  }

  /** Method setData()
   * <p> This method will modify the plot image with a different product
   *  @param stPlot    = Production Plot Data Structure
   *  @param stDecline = Decline Curve Data Structure
   */

  public void setData(productionPlotStruct stPlot, declineStruct stDecline )
  {
	plot.setData(stPlot, stDecline);
  }

  /** Method setDeclineData()
   *  <p> This method will reset the Decline Curve Data Structure
   *  @param st = Decline Curve Data Structure
   */

  public void setDeclineData(declineStruct st) { plot.setDeclineData(st); }

  /** Method setDeclineData(declineStruct st)
   *  <p> This method will reset the Decline Curve Plot
   */

  public void refreshDecline() { plot.refreshDecline(); }

  /** Method setFactor()
   * <p> This method will change the factor
   * @param iFactor == The factor Identifier
   */

  public void setFactor(int iFactor) { plot.setFactor(iFactor); }

  /* =============================================================== *
   * ------------------ SET CURVES ON/OFF METHODS ------------------ *
   * =============================================================== */

  /** Method setProductionCurve()
   *  <p> This method will set the Production & Cumulative Curves ON/OFF
   *  @param iValue = the value to set the Production Curve
   */

  public void setProductionCurve( int iValue ) { plot.setProductionCurve(iValue); }

  /** Method setDeclineCurve()
   *  <p> This method will set the Decline Curve ON or OFF
   *  @param iValue = the value to set for the decline curve
   */

  public void setDeclineCurve( int iValue ) { plot.setDeclineCurve(iValue);  }

  /** Method setMonthlyAvgCurve()
   *  <p> This method will set the Monthly Average Curve ON or OFF
   *  @param iValue = the value to set for the Monthly Average curve
   */

  public void setMonthlyAvgCurve( int iValue ) { plot.setMonthlyAvgCurve(iValue);  }

  /** Method setDailyAvgCurve()
   *  <p> This method will set the Daily Average Curve ON or OFF
   *  @param iValue = the value to set for the Daily Average curve
   */

  public void setDailyAvgCurve( int iValue ) { plot.setDailyAvgCurve(iValue);  }

  /** Method setWellCurve()
   *  <p> This method will set the Plot Number of Wells Curve ON or OFF
   *  @param iValue = the value to set for the Number of Wells
   */

  public void setWellCurve( int iValue ) { plot.setWellCurve(iValue);  }

  /** Method setRawDataCurve()
   *  <p> This method will set the Plot Raw Data Curve ON or OFF
   *  @param iValue = the value to set for the Raw Data
   */

  public void setRawDataCurve( int iValue ) { plot.setRawDataCurve(iValue); }

  /* =============================================================== *
   * ----------------- SET CURVESX XY AXES METHODS ----------------- *
   * =============================================================== */

  /** Method setX()
   *  <p> This method will set the X-Axis Values
   *  @param dXmax    = The Maximum X Values
   *  @param dXmin    = The Minimum X Values
   *  @param dXincr   = The Increment X Value
   *  @param iXCycles = the Number of X-Cycles
   */

  public void setX(double dXmax, double dXmin, double dXincr, int iXCycles)
  {
	plot.setX( dXmax, dXmin, dXincr, iXCycles);
  }

  /** Method setY()
   *  <p> This method will set the Y-Axis Values
   *  @param dYmax    = The Maximum Y Values
   *  @param dYmin    = The Minimum Y Values
   *  @param dYincr   = The Increment Y Value
   *  @param iYCycles = the Number of Y-Cycles
   */

  public void setY(double dYmax, double dYmin, double dYincr, int iYCycles)
  {
	plot.setY( dYmax, dYmin, dYincr, iYCycles);
  }
}

/*
 *  @version 1.1 09/16/2013
 *  @author  John Victorine
 */
