/*
 * @ReadProductionXMLFile.java Version 1.0 06/22/2004
 *
 * Copyright (c) 2001 John R. Victorine
 * All Rights Reserved.
 */

package production.io;

import java.util.*;
import java.io.*;
import java.net.*;
import java.awt.*;
import java.util.zip.*;
import java.math.BigDecimal;

import org.xml.sax.*;
import org.xml.sax.helpers.DefaultHandler;

import javax.xml.parsers.SAXParserFactory;
import javax.xml.parsers.ParserConfigurationException;
import javax.xml.parsers.SAXParser;

import production.productionStruct;      // Production Data Structure
import production.productionListStruct;  // Production List Structure

/** Class ReadProductionXMLFile()
 * <p> This Class will parse the DST Search XML Data Stream.
 *
 *  @version 1.1 06/22/2004
 *  @author  John R. Victorine
 */

public class ReadProductionXMLFile extends DefaultHandler
{
  public static final int FILE       = 0;
  public static final int URL        = 1;
  public static final int SERVER     = 2;
  public static final int SERVER_ZIP = 3;

  private int iType = FILE;

  public static final String ROOTNAME = "production"; // Root Name of XML
  public static final String RECORDS  = "records";    // Total Number of Rows

  public static final String APP      = "app";        // URL used to query
  public static final String BY       = "v1";         // Search By

  public static final String NAME     = "name";       // Name of Production Data
  public static final String TYPE     = "type";       // Type of Production Data
  public static final String KEY      = "key";        // Primary Key for Data
  public static final String ID       = "id";         // Other ID for Data


  public static final String DATA     = "data";       // ELEMENT test
  public static final String YEAR     = "year";       //   Year of Data
  public static final String MONTH    = "month";      //   Month of Data
  public static final String QUARTER  = "quarter";    //   Quarter of Data
  public static final String PRODUCT  = "product";    //   Product
  public static final String OUTPUT   = "output";     //   quantity
  public static final String SOURCES  = "sources";    //   number of sources contributing

  // List of Production Data

  private int                  iCount   = -1;
  private int                  iRecords = 0;
  private int                  iYear    = 0;
  private int                  iMonth   = 0;
  private int                  iQuarter = 0;
  private productionListStruct stData   = new productionListStruct();

  /** Method ReadProductionXMLFile()
   *  This is the Default Constructor for this class.
   */

  public ReadProductionXMLFile() { iType = FILE; }

  /** Method ReadProductionXMLFile()
   *  This is the Constructor for this class.
   *  @param iType = The source identifier,
   *                 0 = File is an absolute Directory Path + Filename
   *                 1 = Source is a URL Location.
   *                 2 = Server Location
   *                 3 = WebStart Location
   */

  public ReadProductionXMLFile(int iType) { this.iType = iType; }

  /** Method Process()
   *  <p> This method will Process the XML File into a Array List.
   *      It is a double pass method in that it Reads the XML File
   *      once to get the number of lines in the XML File and a
   *      second time to actually read the XML File.
   *  @param  filename = The File to be opened.
   *  @return stData   = The DST Recovery Data
   */

  public productionListStruct Process(String filename)
  {
    Read(filename);

    return (stData);
  }

  /* ==================================================================== *
   * -------------------------- PARSE METHODS --------------------------- *
   * ==================================================================== */

  /** Method Read()
   *  <p> This method will Read & Parse the XML File.
   *  @param filename = the filename of the XML File.
   */

  public void Read(String filename)
  {
    // Use the validating parser

    SAXParserFactory factory = SAXParserFactory.newInstance();
    factory.setValidating(true);

    try
    {
      SAXParser saxParser = factory.newSAXParser();
      if (iType == FILE)
      {
        saxParser.parse( new File(filename), this);
      }
      else if (iType == URL)
      {
        try
        {
          InputSource source = new InputSource(filename);
          saxParser.parse(source, this);
        }
        catch (Exception e)
        {
          System.err.println(e);
        }
      }
      else if (iType == SERVER)
      {
        try
        {
          // Connect to the server

          URL u = new URL(filename);
          URLConnection uc = u.openConnection();
          HttpURLConnection connection = (HttpURLConnection) uc;
          connection.setDoOutput(true);
          connection.setDoInput(true);
          connection.setRequestMethod("POST");

          // Read the response XML Document

          InputStream in = connection.getInputStream();
          InputSource source = new InputSource(in);
          saxParser.parse(source, this);
          in.close();
          connection.disconnect();
        }
        catch (Exception e)
        {
          System.err.println(e);
        }
      }
      else
      {
        try
        {
          // Connect to the server

          URL u = new URL(filename);
          URLConnection uc = u.openConnection();
          HttpURLConnection connection = (HttpURLConnection) uc;
          connection.setDoOutput(true);
          connection.setDoInput(true);
          connection.setRequestMethod("POST");

          // Read the response XML Document

          GZIPInputStream in = new GZIPInputStream(connection.getInputStream());
          InputSource source = new InputSource(in);
          saxParser.parse(source, this);
          in.close();
          connection.disconnect();
        }
        catch (Exception e)
        {
          System.err.println(e);
        }
      }
    }
    catch (SAXParseException spe)
    {
      // Error generated by the parser

      System.out.println("\n** Parsing error"            +
                         ", line " + spe.getLineNumber() +
                         ", uri "  + spe.getSystemId());
      System.out.println("   " + spe.getMessage() );

      // Use the contained exception, if any

      Exception  x = spe;
      if (spe.getException() != null)
        x = spe.getException();
      x.printStackTrace();
    }
    catch (SAXException sxe)
    {
      // Error generated by this application (or a parser-initialization error)

      Exception  x = sxe;
      if (sxe.getException() != null)
        x = sxe.getException();
      x.printStackTrace();
    }
    catch (ParserConfigurationException pce)
    {
      // Parser with specified options can't be built

      pce.printStackTrace();
    }
    catch (IOException ioe)
    {
      // I/O error

      ioe.printStackTrace();
    }
  }

  //===========================================================
  // SAX DocumentHandler methods
  //===========================================================

  /** Method setDocumentLocator()
   * <p> This is the Set Document Locator Method
   * @param l Document Locator
   */

  public void setDocumentLocator(Locator l)
  {
    // Ignore errors
  }

  /** Method startDocument()
   * <p> This is the Start Document Method
   * @throws SAXException
   */

  public void startDocument() throws SAXException
  {
    // Ignore it
  }

  /** Method endDocument()
   * <p> This is the End Document Method
   * @throws SAXException
   */

  public void endDocument() throws SAXException
  {
    // Ignore it
  }

  /** Method startElement()
   * <p> This method will parse the ELEMENT from an XML File
   * @param namespaceURI = Name Space URI
   * @param lName        = Local Name
   * @param qName        = Qualified Name
   * @param attrs        = Attributes
   * @throws SAXException
   */

  public void startElement(String namespaceURI,
                           String lName, // local name
                           String qName, // qualified name
                           Attributes attrs) throws SAXException
  {
    String eName = lName; // element name

    if ("".equals(eName))
      eName = qName; // namespaceAware = false

    if (eName.equals(DATA))
    {
      if (iRecords > 0)
      {
        iYear    = 0;
        iMonth   = 0;
        iQuarter = 0;

        iCount++;
        stData.stItem[iCount] = new productionStruct();
      }
    }

    // Add Attributes to the Well Header Data List Structure

    if (attrs != null)
    {
      for (int i = 0; i < attrs.getLength(); i++)
      {
        String aName = attrs.getLocalName(i); // Attr name
        if ("".equals(aName))
          aName = attrs.getQName(i);

        String sTemp = new String(attrs.getValue(i));
        String sOut  = removeExcess(sTemp);

        if (aName.equals(RECORDS)) // Number of Records
        {
          if (cmn.cmnString.isNumeric(sOut.trim()))
          {
            iRecords      = cmn.cmnString.stringToInt(sOut.trim());
            stData.iCount = iRecords;
            stData.stItem = new productionStruct[iRecords];
          }
        }

        if (aName.equals(APP))  { stData.sApp  = new String(sOut.trim()); }
        if (aName.equals(BY))   { stData.sBy   = new String(sOut.trim()); }

        if (aName.equals(NAME)) { stData.sName = new String(sOut.trim()); }
        if (aName.equals(TYPE)) { stData.sType = new String(sOut.trim()); }
        if (aName.equals(KEY))  { stData.sKID  = new String(sOut.trim()); }
        if (aName.equals(ID))   { stData.sID   = new String(sOut.trim()); }

        // Parse each attribute and add it to the structure

        if (iRecords > 0)
        {
          if ((iCount < iRecords) && (iCount > -1))
          {
            ParseData(aName, sOut.trim(), stData.stItem[iCount]);
		  }
		}

        // End Add Attributes to List
      }
    }
  }

  /** Method endElement()
   * <p> This method will parse the ELEMENT from an XML File
   * @param namespaceURI = Name Space URI
   * @param sName        = Simple Name
   * @param qName        = Qualified Name
   * @throws SAXException
   */

  public void endElement(String namespaceURI,
                         String sName, // simple name
                         String qName  // qualified name
                        ) throws SAXException
  {
    // Ignore it
  }

  /** Method characters()
   * <p> This method will parse the CHARACTERS from an XML File
   * @param buf    = Buffer Character Array holding the characters
   * @param offset = String Offset
   * @param len    = Length of String
   * @throws SAXException
   */

  public void characters(char buf[], int offset, int len) throws SAXException
  {
    // Ignore it
  }

  /** Method ignorableWhitespace()
   * <p> This method will parse the Ignorable White Space from an XML File
   * @param buf    = Buffer Character Array holding the characters
   * @param offset = String Offset
   * @param len    = Length of String
   * @throws SAXException
   */

  public void ignorableWhitespace(char buf[], int offset, int len)
    throws SAXException
  {
    // Ignore it
  }

  /** Method processingInstruction()
   * <p> This method will processing Instructions for a XML File
   * @param target = Target
   * @param data   = Data
   * @throws SAXException
   */

  public void processingInstruction(String target, String data)
    throws SAXException
  {
    // Ignore it
  }

  //===========================================================
  // SAX ErrorHandler methods
  //===========================================================

  /** Method error()
   * <p> This method will treat validation errors as fatal
   * @param e = SAX Parse Exception
   * @throws SAXParseException
   */

  public void error(SAXParseException e) throws SAXParseException { throw e; }

  /** Method warning()
   * <p> This method will treat warnings
   * @param err = SAX Parse Exception
   * @throws SAXParseException
   */

  public void warning(SAXParseException err) throws SAXParseException
  {
    System.out.println("** Warning" +
                       ", line "    + err.getLineNumber() +
                       ", uri "     + err.getSystemId());
    System.out.println("   " + err.getMessage());
  }

  //===========================================================
  // Utility Methods ...
  //===========================================================

  /** Method removeExcess()
   * <p> Remove excess white space within a string
   * @param  sin  = String to be parsed of excess spaces
   * @return sout = String with the excess spaces removed
   */

  private String removeExcess(String sin)
  {
    String sout   = new String("");
    char   chold  = ' ';
    char   ch[];

    ch = sin.toCharArray();

    for (int i=0; i<ch.length; i++)
    {
      if (i==0)
        chold = ch[i];

      if (chold != ' ')
      {
        sout = new String(sout + ch[i]);
      }
      else if (chold == ' ')
      {
        if (ch[i] != ' ')
        {
          sout = new String(sout + ch[i]);
        }
      }

      chold = ch[i];
    }

    return (sout);
  }

  /** Method ParseData()
   * <p> This method will parse the Data Stream for the Data
   * @param sIdentifier = The Attribute Identifier
   * @param sData       = Data String
   * @param st          = Data Structure
   */

  private productionStruct ParseData( String sIdentifier,
                                      String sData,
                                      productionStruct st)
  {
    if (sIdentifier.equals(YEAR))       //   Year of Data
    {
	  st.iYear = 0;
	  iYear    = 0;
	  if (cmn.cmnString.isNumeric(sData))
	  {
		st.iYear = cmn.cmnString.stringToInt(sData);
		iYear    = st.iYear;

		if (iYear > 0)
		  st.date = (double) iYear;
	  }
	}

    if (sIdentifier.equals(MONTH))      //   Month of Data
    {
	  st.iMonth = 0;
	  iMonth    = 0;
	  if (cmn.cmnString.isNumeric(sData))
	  {
		st.iMonth = cmn.cmnString.stringToInt(sData);
		iMonth    = st.iMonth;

        if ((iYear > 0) && (iMonth > 0))
          st.date = getDateByYearMonth( iYear, iMonth);
	  }
	}

    if (sIdentifier.equals(QUARTER))    //   Quarter of Data
    {
	  st.iQuarter = 0;
	  iQuarter    = 0;
	  if (cmn.cmnString.isNumeric(sData))
	  {
		st.iQuarter = cmn.cmnString.stringToInt(sData);
		iQuarter    = st.iQuarter;

        if ((iYear > 0) && (iQuarter > 0) && (iMonth == 0))
          st.date = getDateByYearQuarter( iYear, iQuarter);
	  }
	}

    if (sIdentifier.equals(PRODUCT))    //   Product
    {
	  st.sProduct = new String(sData);
	}

    if (sIdentifier.equals(OUTPUT))     //   quantity
    {
	  st.data = 0.0;
	  if (cmn.cmnString.isNumeric(sData))
	  {
		st.data = cmn.cmnString.stringToDouble(sData);
      }
	}

    if (sIdentifier.equals(SOURCES))    //   number of sources contributing
    {
	  st.iSources = 0;
	  if (cmn.cmnString.isNumeric(sData))
	  {
		st.iSources = cmn.cmnString.stringToInt(sData);
	  }
	}

    return (st);
  }

  /** Method getDateByYearMonth()
   *  <p> This method will compute a Date Number by Year & Month
   *      using Big Decimal Math Package
   *
   *      date = iYear + (iMonth / 12)
   *
   *  @param  iYear  = year of production
   *  @param  iMonth = month of production
   *  @return date   = date equivalent
   */

  public static double getDateByYearMonth( int iYear, int iMonth)
  {
	double     date     = 0.0;
	BigDecimal bgDate   = new BigDecimal( "0.0" );
	BigDecimal bgYear   = new BigDecimal( new String(""+iYear) );
	BigDecimal bgMonth  = new BigDecimal( new String(""+iMonth) );
	BigDecimal bgMonths = new BigDecimal( new String("12") );
	BigDecimal bgFrac   = new BigDecimal( "0.0" );

    bgFrac = bgMonth.divide(bgMonths, 3, java.math.BigDecimal.ROUND_CEILING);
    bgDate = bgYear.add( bgFrac );
	date   = bgDate.doubleValue();

	return (date);
  }


  /** Method getDateByYearQuarter()
   *  <p> This method will compute a Date Number by Year & Quarter
   *      using Big Decimal Math Package
   *
   *      date = iYear + (iQuarter / 4)
   *
   *  @param  iYear    = year of production
   *  @param  iQuarter = quarter of production
   *  @return date     = date equivalent
   */

  public static double getDateByYearQuarter( int iYear, int iQuarter)
  {
	double     date     = 0.0;
	BigDecimal bgDate   = new BigDecimal( "0.0" );
	BigDecimal bgYear   = new BigDecimal( new String(""+iYear) );
	BigDecimal bgMonth  = new BigDecimal( new String(""+iQuarter) );
	BigDecimal bgMonths = new BigDecimal( new String("4") );
	BigDecimal bgFrac   = new BigDecimal( "0.0" );

    bgFrac = bgMonth.divide(bgMonths, 3, java.math.BigDecimal.ROUND_CEILING);
    bgDate = bgYear.add( bgFrac );
	date   = bgDate.doubleValue();

	return (date);
  }
}
