/*
 * @WriteProductionXMLFile.java Version 1.1 09/19/2013
 *
 * Copyright (c) 2007 Kansas Geological Survey
 * 1930 Constant Avenue, Lawrence, Kansas, 66047, U.S.A.
 * All Rights Reserved.
 */

package production.io;

import java.io.*;
import java.awt.*;

import util.utilFileIO;
import production.productionListStruct;
import cmn.cmnString;

/** Class WriteProductionXMLFile
 *  <p> This class will parse the Production List Structure and
 *      write the information to the Production XML File.
 *
 *  @version 1.1 09/19/2013
 *  @author  John R. Victorine
 */

public class WriteProductionXMLFile
{
  public static final String DTD =
    "<?xml version=\"1.0\"?>\n"                        +
    "<!DOCTYPE production [\n"                       +
    "<!ELEMENT production (data*)>\n"                +
    "<!ATTLIST production records CDATA #REQUIRED\n" + // Total # of Records
    "                     key     CDATA #REQUIRED\n" + // Primary Key
    "                     id      CDATA #IMPLIED\n"  + // Other ID Key
    "                     type    CDATA #REQUIRED\n" + // Type of Data
    "                     name    CDATA #REQUIRED\n" + // Name of Data
    "                     app     CDATA #IMPLIED\n"  + // XML Application
    "                     v1      CDATA #IMPLIED\n"  + // Variable 1
    "                     v2      CDATA #IMPLIED\n"  + // Variable 2
    "                     v3      CDATA #IMPLIED>\n" + // Variable 3
    "<!ELEMENT data  EMPTY>\n"                       +
    "<!ATTLIST data  year    CDATA #REQUIRED\n"      + // Year Number
    "                month   CDATA #IMPLIED\n"       + // Month Number 0,1,2,...12
    "                quarter CDATA #IMPLIED\n"       + // Quarter Number 1,2,3...
    "                product CDATA #REQUIRED\n"      + // Product, OIL, GAS, etc.
    "                unit    CDATA #IMPLIED\n"       + // Units of Data BBL or MCF
    "                output  CDATA #REQUIRED\n"      + // Production
    "                sources CDATA #IMPLIED>]>\n";     // Number of Wells

  /** Method write()
   *  <p> This is the method will write the XML String to the user's PC
   *  @param sPath     = The Directory Path for the XML File
   *  @param sFilename = The Filename of the XML File.
   *  @param sXML      = The Headers XML String
   */

  public static void write(String sPath, String sFilename, String sXML)
  {
    utilFileIO pIO  = new utilFileIO();

    // Open the File for write

    pIO.Open( util.utilFileIO.DIRECTORY,
              util.utilFileIO.WRITE,
              sPath,
              sFilename );

    // Write the XML Block to the File

    pIO.Write( sXML );

    // Close the File

    pIO.Close();
  }

  /** Method buildXML()
   *  <p> This method will build the XML File
   *  @param  st   = production list data structure
   *  @return sXML = The Production Data XML
   */

  public static String buildXML(productionListStruct st)
  {
	String sXML = "";
	if (st != null)
	{
  	  sXML = new String(DTD);

  	  sXML = new String(  sXML + "<production records=\"" + st.iCount + "\" ");

      if (st.sKID.length() > 1)
        sXML = new String(sXML + "key=\""  + st.sKID  + "\" ");

      if (st.sID.length() > 1)
        sXML = new String(sXML + "id=\""   + st.sID   + "\" ");

      if (st.sType.length() > 1)
        sXML = new String(sXML + "type=\"" + st.sType + "\" ");

      if (st.sName.length() > 1)
        sXML = new String(sXML + "name=\"" + st.sName + "\" ");

      if (st.sApp.length() > 1)
        sXML = new String(sXML + "app=\""  + st.sApp  + "\" ");

      if (st.sBy.length() > 1)
        sXML = new String(sXML + "v1=\""   + st.sBy   + "\" ");

	  sXML = new String(  sXML + ">\n");

      for (int i=0; i<st.iCount; i++)
      {
		sXML = new String(  sXML + "  <data year=\"" + st.stItem[i].iYear + "\" ");

		if (st.stItem[i].iMonth > 0)
		  sXML = new String(sXML + "month=\""   + st.stItem[i].iMonth   + "\" ");

		if (st.stItem[i].iQuarter > 0)
		  sXML = new String(sXML + "quarter=\"" + st.stItem[i].iQuarter + "\" ");

		sXML = new String(  sXML + "product=\"" + st.stItem[i].sProduct + "\" ");
		sXML = new String(  sXML + "output=\""  + st.stItem[i].data     + "\" ");

		if (st.stItem[i].iSources > 0)
  		  sXML = new String(sXML + "sources=\"" + st.stItem[i].iSources + "\" ");

		sXML = new String(  sXML + "/>\n");
	  }

	  sXML = new String(sXML + "</production>\n");
	}

	return (sXML);
  }

  /** Method buildCSV()
   *  <p> This method will build the CSV File
   *  @param  st   = production list data structure
   *  @return sCSV = Production ASCII CSV String
   */

  public static String buildCSV(productionListStruct st)
  {
	int i=0;
	boolean bMonth   = false;
	boolean bQuarter = false;
	boolean bSources = false;
	String  sCSV     = "";

	if (st != null)
	{
      for (i=0; i<st.iCount; i++)
      {
		if (st.stItem[i].iMonth > 0)
		  bMonth = true;

		if (st.stItem[i].iQuarter > 0)
		  bQuarter = true;

		if (st.stItem[i].iSources > 0)
		  bSources = true;
	  }

      // First Line of CSV File

      if (st.sKID.length() > 1)
        sCSV = new String(sCSV + "KEY=" + st.sKID  + " ");

      if (st.sID.length() > 1)
        sCSV = new String(sCSV + "DOR=" + st.sID   + " ");

      if ((st.sType.length() > 1) && (st.sName.length() > 1))
        sCSV = new String(sCSV + st.sType + "=" + st.sName + " ");

      if ((st.sApp.length() > 1) && (st.sBy.length() > 1))
        sCSV = new String(sCSV + "app="  + st.sApp  + "?" + st.sBy);

      sCSV = new String(sCSV + "\n");

      // Second Line of CSV

      sCSV = new String(sCSV   + "Year, ");

      if (bMonth)
        sCSV = new String(sCSV + "Month, ");

      if (bQuarter)
        sCSV = new String(sCSV + "Quarter, ");

      sCSV = new String(sCSV   + "Product, ");

      if (bSources)
      {
        sCSV = new String(sCSV + "Output, ");
        sCSV = new String(sCSV + "sources\n");
	  }
	  else
        sCSV = new String(sCSV + "Output\n");

      for (i=0; i<st.iCount; i++)
      {
        sCSV = new String(sCSV   + st.stItem[i].iYear    + ", ");

        if (bMonth)
          sCSV = new String(sCSV + st.stItem[i].iMonth   + ", ");

        if (bQuarter)
          sCSV = new String(sCSV + st.stItem[i].iQuarter + ", ");

        sCSV = new String(sCSV   + st.stItem[i].sProduct + ", ");

        if (bSources)
        {
          sCSV = new String(sCSV + st.stItem[i].data     + ", ");
          sCSV = new String(sCSV + st.stItem[i].iSources + "\n");
  	    }
	    else
          sCSV = new String(sCSV + st.stItem[i].data     + "\n");
	  }
	}

	return (sCSV);
  }
}

/*
 *  @version 1.1 09/19/2013
 *  @author  John Victorine
 */
