/*
 * @declineMath.java Version 1.1 09/17/2013
 *
 * Copyright (c) 2013 Kansas Geological Survey
 * 1930 Constant Avenue, Lawrence, Kansas, 66047, U.S.A.
 * All Rights Reserved.
 */

package production.math;

import java.math.BigDecimal;

/** Class declineMath
 *  <p> This Class allows the ability to remove round off errors
 *      in the basic math package
 *
 *  @version 1.0 09/17/2013
 *  @author  John R. Victorine
 */

public class declineMath
{
  public static final int CEILING = java.math.BigDecimal.ROUND_CEILING;
  public static final int FLOOR   = java.math.BigDecimal.ROUND_FLOOR;

  public static final int DAILY       = 0;
  public static final int MONTHLY     = 1;
  public static final int YEARLY      = 2;

  public static final int EXPONENTIAL = 0;
  public static final int HARMONIC    = 1;
  public static final int HYPERBOLIC  = 2;

  /** Method computeDecline(double Qi, double Qt, int Qi_year, int Qt_year)
   *  <p> This method will ocmpute the Decline Rate in Percent per Year
   *
   *                Qi-Qt        100.0
   *     decline =  ----- *  ---------------
   *                  Qi     Qt_year-Qi_year
   *
   *  @param  Qi      = Initial Production Rate
   *  @param  Qt      = Present Production Rate
   *  @param  Qi_year = Year Date of Initial Production Rate
   *  @param  Qt_year = Year Date of Present Production Rate
   *  @return decline = Decline Rate in Percent per year
   */

  public static double computeDecline(double Qi, double Qt, int Qi_year, int Qt_year)
  {
    double     decline   = 0.0;
    BigDecimal bgVal     = java.math.BigDecimal.ZERO;
    BigDecimal bgA1      = java.math.BigDecimal.ZERO;
    BigDecimal bgA2      = java.math.BigDecimal.ZERO;
    BigDecimal bgB1      = java.math.BigDecimal.ZERO;
    BigDecimal bgB2      = java.math.BigDecimal.ZERO;
    BigDecimal bgPer     = new BigDecimal( "100.0" );

    BigDecimal bgQi      = new BigDecimal( new String(""+Qi) );
    BigDecimal bgQt      = new BigDecimal( new String(""+Qt) );
    BigDecimal bgQi_year = new BigDecimal( new String(""+Qi_year) );
    BigDecimal bgQt_year = new BigDecimal( new String(""+Qt_year) );

    if ((Qi > 0.0) && (Qt > 0.0) && (Qi_year > 0) && (Qt_year > 0) && (Qi_year != Qt_year))
    {
	  //  100 * (Qi-Qt)

      bgA1    = bgQi.subtract(bgQt);
      bgA2    = bgPer.multiply(bgA1);

      //  Qi * (Qt_year-Qi_year)

      bgB1    = bgQt_year.subtract(bgQi_year);
      bgB2    = bgQi.multiply(bgB1);
      bgVal   = bgA2.divide(bgB2, 3, CEILING);

      decline = bgVal.doubleValue();
    }

    return (decline);
  }

  /** Method computeTime( int iRate, int Qi_year, int iYear )
   *  <p> This method will compute a time relative to the initial
   *      production year
   *
   *              iYear - Qi_year     Daily:   dFactor = 365.0
   *    dTime =   ---------------  ;  Monthly: dFactor = 12.0
   *                  dFactor         Yearly:  dFactor = 1.0
   *
   *  @param  iRate   = Rate (Daily, Montly, or Yearly) Indicator
   *  @param  Qi_year = Year Date of Initial Production Rate
   *  @param  iYear   = The year Date of the time that needs to be computed
   *  @return iTime   = The time relative to the initial production year
   */

  public static double computeTime( int iRate, int Qi_year, int iYear )
  {
	double     dTime     = 0.0;
    BigDecimal bgVal     = java.math.BigDecimal.ZERO;
    BigDecimal bgA1      = java.math.BigDecimal.ZERO;
    BigDecimal bgFactor  = java.math.BigDecimal.ZERO;
    BigDecimal bgDaily   = new BigDecimal( "365.0" );
    BigDecimal bgMontly  = new BigDecimal( "12.0" );
    BigDecimal bgYearly  = java.math.BigDecimal.ONE;

    BigDecimal bgYear    = new BigDecimal( new String(""+iYear) );
    BigDecimal bgQi_year = new BigDecimal( new String(""+Qi_year) );

    switch (iRate)
    {
      case DAILY:
        bgFactor = bgDaily;
        break;
      case MONTHLY:
        bgFactor = bgMontly;
        break;
      case YEARLY:
        bgFactor = bgYearly;
        break;
	}

    bgA1  = bgYear.subtract(bgQi_year);
    bgVal = bgA1.multiply(bgFactor);

    dTime = bgVal.doubleValue();

    return (dTime);
  }

  /** Method computeYear( int iRate, int Qi_year, double dTime )
   *  <p> This method will compute a year date of the time relative
   *      to the initial production year
   *
   *                         iTime      Daily:   dFactor = 365.0
   *    dTime =   Qi_year + -------  ;  Monthly: dFactor = 12.0
   *                        dFactor     Yearly:  dFactor = 1.0
   *
   *  @param  iRate   = Rate (Daily, Montly, or Yearly) Indicator
   *  @param  Qi_year = Year Date of Initial Production Rate
   *  @param  dTime   = The time relative to the initial production year
   *  @return iYear   = The year date relative to the initial production year
   */

  public static int computeYear( int iRate, int Qi_year, double dTime )
  {
	int        iYear     = 0;
    BigDecimal bgVal     = java.math.BigDecimal.ZERO;
    BigDecimal bgA1      = java.math.BigDecimal.ZERO;
    BigDecimal bgFactor  = java.math.BigDecimal.ZERO;
    BigDecimal bgDaily   = new BigDecimal( "365.0" );
    BigDecimal bgMontly  = new BigDecimal( "12.0" );
    BigDecimal bgYearly  = java.math.BigDecimal.ONE;

    BigDecimal bgTime    = new BigDecimal( new String(""+dTime) );
    BigDecimal bgQi_year = new BigDecimal( new String(""+Qi_year) );

    switch (iRate)
    {
      case DAILY:
        bgFactor = bgDaily;
        break;
      case MONTHLY:
        bgFactor = bgMontly;
        break;
      case YEARLY:
        bgFactor = bgYearly;
        break;
	}

    bgA1  = bgTime.divide(bgFactor, 1, CEILING);  // iTime / dFactor
    bgVal = bgQi_year.add(bgA1);

    iYear = bgVal.intValue();

    return (iYear);
  }

  /** Method computeK(int iType, int iRate, double n, double Qi,
   *                  double Qt, double n, int Qi_year, int Qt_year)
   *  <p> This method will compute the Decline Constant (K)
   *
   *     Exponential:
   *       K = ln(Qi/Qt) / iTime
   *
   *     Harmonic:
   *       K = [(Qi/Qt) - 1.0] / iTime
   *
   *     Hyperbolic:
   *       K = {exp[n * ln(Qi/Qt)] - 1.0} / iTime
   *
   *  @param  iType   = Type of Decline Curve Indicator
   *  @param  iRate   = Rate (Daily, Montly, or Yearly) Indicator
   *  @param  n       = Decline Exponent
   *  @param  Qi      = Initial Production Rate
   *  @param  Qt      = Present Production Rate
   *  @param  Qi_year = Year Date of Initial Production Rate
   *  @param  Qt_year = Year Date of Present Production Rate
   *  @return K       = Decline Constant
   */

  public static double computeK(int iType, int iRate, double n,
                                double Qi, double Qt, int Qi_year, int Qt_year)
  {
    double     K         = 0.0;
    int        iType1    = iType;
    double     dTime     = 0.0;

    double     dRatio    = 0.0;
    double     dLog      = 0.0;
    double     dExp      = 0.0;

    BigDecimal bgRatio   = java.math.BigDecimal.ZERO;
    BigDecimal bgA1      = java.math.BigDecimal.ZERO;
    BigDecimal bgA2      = java.math.BigDecimal.ZERO;
    BigDecimal bgB1      = java.math.BigDecimal.ZERO;
    BigDecimal bgB2      = java.math.BigDecimal.ZERO;
    BigDecimal bgVal     = java.math.BigDecimal.ZERO;

    BigDecimal bgQi      = new BigDecimal( new String(""+Qi) );
    BigDecimal bgQt      = new BigDecimal( new String(""+Qt) );
    BigDecimal bgn       = new BigDecimal( new String(""+n) );
    BigDecimal bgQi_year = new BigDecimal( new String(""+Qi_year) );
    BigDecimal bgQt_year = new BigDecimal( new String(""+Qt_year) );
    BigDecimal bgTime    = java.math.BigDecimal.ZERO;

    dTime  = computeTime( iRate, Qi_year, Qt_year );
    bgTime = new BigDecimal( new String(""+dTime) );

    if ((Qi > 0.0) && (Qt > 0.0) && (Qi_year > 0) && (Qt_year > 0) && (dTime > 0))
    {
	  //   Qi/Qt

	  bgRatio = bgQi.divide(bgQt, 3, CEILING);
	  dRatio  = bgRatio.doubleValue();

	  //   ln(Qi/Qt)

	  dLog   = Math.log(dRatio);
	  bgA1   = new BigDecimal(""+dLog);

      switch (iType)
      {
        case EXPONENTIAL:
        case HARMONIC:
          break;
        case HYPERBOLIC:
          if (n == 0.0)
            iType1 = EXPONENTIAL;
          if (n == 1.0)
            iType1 = HARMONIC;
          break;
	  }

      switch (iType1)
      {
		//  ln(Qi/Qt) / iTime
        case EXPONENTIAL:
          bgVal = bgA1.divide(bgTime, 6, CEILING);
          break;

        //  [(Qi/Qt) - 1.0] / iTime
        case HARMONIC:
          bgA2  = bgRatio.subtract(new BigDecimal("1.0"));
          bgVal = bgA2.divide(bgTime, 6, CEILING);
          break;

        //  {exp[n * ln(Qi/Qt)] - 1.0} / iTime
        case HYPERBOLIC:
          //  n * ln(Qi/Qt)
          bgB1   = bgn.multiply(bgA1);
          dRatio = bgB1.doubleValue();

          // exp[n * ln(Qi/Qt)]
          dExp   = Math.exp(dRatio);
          bgB2   = new BigDecimal(""+dExp);

          // {exp[n * ln(Qi/Qt)] - 1.0} / iTime
          bgA2   = bgB2.subtract(new BigDecimal("1.0"));
          bgVal  = bgA2.divide(bgTime, 6, CEILING);
          break;
	  }

      K = bgVal.doubleValue();
    }

    return (K);
  }

  /** Method computeQc(int iType, double n, double K,
   *                   double Qi, double dRate, int Qi_year)
   *  <p> This method will compute the Cumulative Production Rate (Qc)
   *
   *     Exponential:
   *       Qc = (Qi-dRate) / K
   *
   *     Harmonic:
   *       Qc = Qi * ln(Qi/Rate) / K
   *
   *     Hyperbolic:
   *       Qc = {Qi - dRate * exp[n*ln(Qi/dRate)]} / [(1.0-n) * K]
   *
   *  @param  iType   = Type of Decline Curve Indicator
   *  @param  n       = Decline Exponent
   *  @param  K       = Decline Constant
   *  @param  Qi      = Initial Production Rate
   *  @param  dRate   = Production Rate
   *  @param  Qi_year = Year Date of Initial Production Rate
   *  @return Qc      = Production Rate
   */

  public static double computeQc(int iType, double n, double K,
                                double Qi, double dRate) //, int Qi_year)
  {
    double     Qc        = 0.0;
    int        iType1    = iType;

    double     dRatio    = 0.0;
    double     dLog      = 0.0;
    double     dExp      = 0.0;

    BigDecimal bg1       = java.math.BigDecimal.ONE;
    BigDecimal bgRatio   = java.math.BigDecimal.ZERO;
    BigDecimal bgA1      = java.math.BigDecimal.ZERO;
    BigDecimal bgA2      = java.math.BigDecimal.ZERO;
    BigDecimal bgB1      = java.math.BigDecimal.ZERO;
    BigDecimal bgB2      = java.math.BigDecimal.ZERO;
    BigDecimal bgTemp    = java.math.BigDecimal.ZERO;
    BigDecimal bgVal     = java.math.BigDecimal.ZERO;

    BigDecimal bgQi      = new BigDecimal( new String(""+Qi) );
    BigDecimal bgRate    = new BigDecimal( new String(""+dRate) );
    BigDecimal bgn       = new BigDecimal( new String(""+n) );
    BigDecimal bgK       = new BigDecimal( new String(""+K) );
//    BigDecimal bgQi_year = new BigDecimal( new String(""+Qi_year) );

    if ((Qi > 0.0) && (dRate > 0.0) && (K > 0)) //  && (Qi_year > 0)
    {
	  //   Qi/Qt

	  bgRatio = bgQi.divide(bgRate, 3, CEILING);
	  dRatio  = bgRatio.doubleValue();

	  //   ln(Qi/Qt)

	  dLog   = Math.log(dRatio);
	  bgA1   = new BigDecimal(""+dLog);

      switch (iType)
      {
        case EXPONENTIAL:
        case HARMONIC:
          break;
        case HYPERBOLIC:
          if (n == 0.0)
            iType1 = EXPONENTIAL;
          if (n == 1.0)
            iType1 = HARMONIC;
          break;
	  }

      switch (iType1)
      {
		//  (Q-dRate) / K
        case EXPONENTIAL:
          bgA2  = bgQi.subtract(bgRate);
          bgVal = bgA2.divide(bgK, 3, CEILING);
          break;

        //  Qi*ln(Qi/dRate) / K
        case HARMONIC:
          bgA2  = bgQi.multiply(bgA1);
          bgVal = bgA2.divide(bgK, 3, CEILING);
          break;

        //  {Qi - dRate * exp[n*ln(Qi/dRate)]} / [(1.0-n) * K]
        case HYPERBOLIC:
          //  n * ln(Qi/dRate)
          bgB1   = bgn.multiply(bgA1);
          dRatio = bgB1.doubleValue();

          // exp[n * ln(Qi/dRate)]
          dExp   = Math.exp(dRatio);
          bgB2   = new BigDecimal(""+dExp);

          // {Qi - dRate * exp[n*ln(Qi/dRate)]}
          bgB1   = bgRate.multiply(bgB2);
          bgA2   = bgQi.subtract(bgB1);

          // [(1.0-n) * K]
          bgTemp = bg1.subtract(bgn);
          bgTemp = bgK.multiply(bgTemp);

          //{Qi - dRate * exp[n*ln(Qi/dRate)]} / [(1.0-n) * K]
          bgVal  = bgA2.divide(bgTemp, 3, CEILING);
          break;
	  }

      Qc = bgVal.doubleValue();
    }

    return (Qc);
  }

  /** Method computeEconomicLimitDate(
   *                  int iType, int iRate, double n, double K,
   *                  double Qi, double Qf, int Qi_year )
   *  <p> This method will compute the Economic Limit Date
   *
   *     Exponential:
   *       t = ln(Qi/Qf) / K
   *
   *     Harmonic:
   *       t = [(Qi/Qf) - 1.0] / K
   *
   *     Hyperbolic:
   *       t = {exp[n*ln(Qi/Qf)] - 1.0} / (n * K)
   *
   *       where t = time
   *
   *  @param  iType   = Type of Decline Curve Indicator
   *  @param  iRate   = Rate (Daily, Montly, or Yearly) Indicator
   *  @param  n       = Decline Exponent
   *  @param  K       = Decline Constant
   *  @param  Qi      = Initial Production Rate
   *  @param  Qf      = Econimic Limit Production Rate
   *  @param  Qi_year = Year Date of Initial Production Rate
   *  @return iYear   = Economic Limit Date
   */

  public static int computeEconomicLimitDate(
	  int iType, int iRate, double n, double K, double Qi, double Qf, int Qi_year)
  {
    int        iYear     = 0;
    double     dTime     = 0;
    int        iType1    = iType;

    double     dRatio    = 0.0;
    double     dLog      = 0.0;
    double     dExp      = 0.0;

    BigDecimal bg1       = java.math.BigDecimal.ONE;
    BigDecimal bgRatio   = java.math.BigDecimal.ZERO;
    BigDecimal bgA1      = java.math.BigDecimal.ZERO;
    BigDecimal bgA2      = java.math.BigDecimal.ZERO;
    BigDecimal bgB1      = java.math.BigDecimal.ZERO;
    BigDecimal bgB2      = java.math.BigDecimal.ZERO;
    BigDecimal bgTemp    = java.math.BigDecimal.ZERO;
    BigDecimal bgVal     = java.math.BigDecimal.ZERO;

    BigDecimal bgQi      = new BigDecimal( new String(""+Qi) );
    BigDecimal bgQf      = new BigDecimal( new String(""+Qf) );
    BigDecimal bgn       = new BigDecimal( new String(""+n) );
    BigDecimal bgK       = new BigDecimal( new String(""+K) );

    if ((Qi > 0.0) && (Qf > 0.0) && (K > 0))// && (Qi_year > 0))
    {
	  //   Qi/Qt

	  bgRatio = bgQi.divide(bgQf, 3, CEILING);
	  dRatio  = bgRatio.doubleValue();

	  //   ln(Qi/Qt)

	  dLog   = Math.log(dRatio);
	  bgA1   = new BigDecimal(""+dLog);

      switch (iType)
      {
        case EXPONENTIAL:
        case HARMONIC:
          break;
        case HYPERBOLIC:
          if (n == 0.0)
            iType1 = EXPONENTIAL;
          if (n == 1.0)
            iType1 = HARMONIC;
          break;
	  }

      switch (iType1)
      {
		//  ln(Qi/Qf) / K
        case EXPONENTIAL:
          bgVal = bgA1.divide(bgK, 3, CEILING);
          break;

        //  [(Qi/Qf) - 1.0] / K
        case HARMONIC:
          bgA2  = bgRatio.subtract(bg1);
          bgVal = bgA2.divide(bgK, 3, CEILING);
          break;

        //  {exp[n*ln(Qi/Qf)] - 1.0} / (n * K)
        case HYPERBOLIC:
          //  n * ln(Qi/dRate)
          bgB1   = bgn.multiply(bgA1);
          dRatio = bgB1.doubleValue();

          // exp[n * ln(Qi/dRate)]
          dExp   = Math.exp(dRatio);
          bgB2   = new BigDecimal(""+dExp);

          // {exp[n*ln(Qi/Qf)] - 1.0}
          bgB1   = bgB2.subtract(bg1);

          // n * K
          bgTemp = bgn.subtract(bgK);

          // {exp[n*ln(Qi/Qf)] - 1.0} / (n * K)
          bgVal  = bgB1.divide(bgTemp, 3, CEILING);
          break;
	  }

      dTime = bgVal.doubleValue();
      iYear = computeYear( iRate, Qi_year, dTime );
    }

    return (iYear);
  }

  /** Method computeQ(int iType, int iRate, double n, double K,
   *                  double Qi, int Qi_year, int iYear)
   *  <p> This method will compute the Cumulative Production Rate (Qc)
   *
   *     Exponential:
   *       Q = Qi * exp(-1.0*K*t)
   *
   *     Harmonic:
   *       Q = Qi / (1 + K*t)
   *
   *     Hyperbolic:
   *       Q = Qi / (1+n*K*t)^(1/n)
   *
   *  @param  iType   = Type of Decline Curve Indicator
   *  @param  iRate   = Rate (Daily, Montly, or Yearly) Indicator
   *  @param  n       = Decline Exponent
   *  @param  K       = Decline Constant
   *  @param  Qi      = Initial Production Rate
   *  @param  Qi_year = Year Date of Initial Production Rate
   *  @param  iYear   = Economic Limit Date
   *  @return Q       = Production Rate
   */

  public static double computeQ(int iType, int iRate, double n, double K,
                                double Qi, int Qi_year, int iYear)
  {
    double     Q         = 0.0;
    int        iType1    = iType;
    double     dTime     = 0.0;
    double     dRatio    = 0.0;
    double     dLog      = 0.0;
    double     dExp      = 0.0;

    BigDecimal bg1       = java.math.BigDecimal.ONE;
    BigDecimal bgA1      = java.math.BigDecimal.ZERO;
    BigDecimal bgA2      = java.math.BigDecimal.ZERO;
    BigDecimal bgB1      = java.math.BigDecimal.ZERO;
    BigDecimal bgB2      = java.math.BigDecimal.ZERO;
    BigDecimal bgTemp    = java.math.BigDecimal.ZERO;
    BigDecimal bgVal     = java.math.BigDecimal.ZERO;

    BigDecimal bgQi      = new BigDecimal( new String(""+Qi) );
    BigDecimal bgn       = new BigDecimal( new String(""+n) );
    BigDecimal bgK       = new BigDecimal( new String(""+K) );
    BigDecimal bgQi_year = new BigDecimal( new String(""+Qi_year) );
    BigDecimal bgTime    = java.math.BigDecimal.ZERO;

    if ((Qi > 0.0) && (iYear > 0.0) && (Qi_year > 0) && (K > 0))
    {
	  dTime  = computeTime( iRate, Qi_year, iYear);
	  bgTime = new BigDecimal("" + dTime);

      switch (iType)
      {
        case EXPONENTIAL:
        case HARMONIC:
          break;
        case HYPERBOLIC:
          if (n == 0.0)
            iType1 = EXPONENTIAL;
          if (n == 1.0)
            iType1 = HARMONIC;
          break;
	  }

      switch (iType1)
      {
		//  Qi * exp(-1.0*K*t)
        case EXPONENTIAL:
          // exp(K*t)
          bgA1   = bgK.multiply(bgTime);         // K*t
          dRatio = bgA1.doubleValue();
          dExp   = Math.exp(dRatio);             // exp(K*t)

//          dExp  = Qi/dExp;
//          bgVal  = new BigDecimal(""+dExp);
          bgA2  = new BigDecimal(""+dExp);
          bgVal = bgQi.divide(bgA2, 3, CEILING); // Qi/exp(K*t)
          break;

        //  Qi / (1 + K*t)
        case HARMONIC:
          bgA1  = bgK.multiply(bgTime);          // K*t
          bgA2  = bg1.add(bgA1);                 // 1 + K*t
          bgVal = bgQi.divide(bgA2, 3, CEILING); // Qi/(1 + K*t)
          break;

        //  Qi / (1+n*K*t)^(1/n)
        case HYPERBOLIC:
          // 1+n*K*t
          bgA1  = bgK.multiply(bgTime);          // K*t
          bgA2  = bgn.multiply(bgA1);            // n*K*t
          bgB1  = bg1.add(bgA2);                 // 1+n*K*t

          dRatio = bgB1.doubleValue();
          dLog  = Math.log(dRatio)/n;            // [ln(1+n*K*t)]/n
          dExp  = Math.exp(dLog);                // exp{[ln(1+n*K*t)]/n}
          bgB2  = new BigDecimal(""+dExp);

          bgVal = bgQi.divide(bgB2, 3, CEILING);   // Qi / (1+n*K*t)^(1/n)
          break;
	  }

      Q = bgVal.doubleValue();
    }

    return (Q);
  }

  /** Method computeQf(int iRate, int iLimitRate, double dRate)
   *  <p> This method will set the Qf value based on the Rate &
   *      Limit Rate of the Rate Value passed in.
   *  @param iRate      = The Production Rate of Raw Data
   *  @param iLimitRate = The Economic Limit Rate
   *  @param dRate      = The Rate Value being computed
   */

  public static double computeQf(int iRate, int iLimitRate, double dRate)
  {
	double     Qf     = 0.0;
    BigDecimal bgVal  = java.math.BigDecimal.ZERO;
    BigDecimal bgRate = new BigDecimal( new String(""+dRate) );

    switch (iLimitRate)
    {
      case DAILY:
        switch (iRate)
        {
          case DAILY:
            bgVal = bgRate;
            break;
          case MONTHLY:
            bgVal = bgRate.multiply(new BigDecimal("12.0"));
            break;
          case YEARLY:
            bgVal = bgRate.multiply(new BigDecimal("365.0"));
            break;
      	}
        break;
      case MONTHLY:
        switch (iRate)
        {
          case DAILY:
            bgVal = bgRate.divide(new BigDecimal("30.0"), 3, CEILING);
            break;
          case MONTHLY:
            bgVal = bgRate;
            break;
          case YEARLY:
            bgVal = bgRate.multiply(new BigDecimal("12.0"));
            break;
      	}
        break;
      case YEARLY:
        switch (iRate)
        {
          case DAILY:
            bgVal = bgRate.divide(new BigDecimal("365.0"), 3, CEILING);
            break;
          case MONTHLY:
            bgVal = bgRate.divide(new BigDecimal("12.0"), 3, CEILING);
            break;
          case YEARLY:
            bgVal = bgRate;
            break;
      	}
        break;
	}

    Qf = bgVal.doubleValue();

	return (Qf);
  }
}

/*
 *  @version 1.1 09/17/2013
 *  @author  John Victorine
 */
