/*
 * @productionPlot.java Version 1.1 09/10/2013
 *
 * Copyright (c) 2013 Kansas Geological Survey
 * 1930 Constant Avenue, Lawrence, Kansas, 66047, U.S.A.
 * All Rights Reserved.
 */

package production.plot;

import java.awt.*;
import java.awt.geom.*;
import java.awt.image.*;

import production.plot.productionPlotStruct;  // Production Plot Data Structure
import production.declineStruct;              // Decline Curve Data Structure

/** Class productionPlot
 *  <p> This Class will create the production plot
 *
 *  @version 1.1 09/10/2013
 *  @author  John R. Victorine
 */

public class productionPlot extends Canvas
{
  public static final int OFF        = cmn.cmnStruct.OFF;
  public static final int ON         = cmn.cmnStruct.ON;

  public static final  int OIL       = production.productionStruct.OIL;
  public static final  int GAS       = production.productionStruct.GAS;

  // Grid Type

  public static final int _LINEAR    = production.plot.productionPlotStruct._LINEAR;
  public static final int _LOG       = production.plot.productionPlotStruct._LOG;

  // Plotting Grids for Plotting Curves

  public static final int PRODUCTION = 0; // Production Data
  public static final int CUMULATIVE = 1; // Cumulative Production Data
  public static final int BOTH       = 2; // Both Production & Cumulative Data
  public static final int TIME       = 3; // Time Data

  // Decline Curve Rates

  public static final int DAILY       = production.math.declineMath.DAILY;
  public static final int MONTHLY     = production.math.declineMath.MONTHLY;
  public static final int YEARLY      = production.math.declineMath.YEARLY;

  // Producton Data Variables

  private productionPlotStruct stPlot    = null; // Production Plot Data Structure
  private declineStruct        stDecline = null; // Decline Curve Data Structure

  // -- Grid Settings

  private int    iXAxis         = TIME;
  private int    iYAxis         = BOTH;

  // -- Product

  private int    iProduct       = -1;

  // -- Plot Curve ON/OFF Indicators

  private int    iDeclineCurve  = ON;   // Turn ON/OFF Decline Curve
  private int    i12Avg         = OFF;  // Montly Curve ON/OFF Indicator
  private int    i365Avg        = OFF;  // Daily Curve ON/OFF Indicator
  private int    iWellsCurve    = OFF;  // Number of Wells Curve Off
  private int    iRawCurve      = OFF;  // Raw Production Data Curve

  // -- Production Curve Data

  private int    iTime[]        = null; // Array of Plot Time in pixels
  private int    iProduction[]  = null; // Array of Production Data in pixels
  private int    iCumulative[]  = null; // Array of Cumulative Data in pixels
  private int    iMonthlyAvg[]  = null; // Array of Monthly Average in pixels
  private int    iDailyAvg[]    = null; // Array of Daily Average in pixels

  private int    iRawTime[]     = null; // Array of Raw Time in pixels
  private int    iRawData[]     = null; // Array of Raw Data in pixels

  // -- Plot Factors

  private int    iFactor        = 1;    // Divide by Production factor
  private double dTimeIncrement = 1.0;  // Plot Increment for Time

  // -- Decline Curve Data

  private int    iDeclineRows   = 0;    // Number of Decline Data Rows
  private int    iDecline[]     = null; // Array of Decline Curve Data in pixels
  private int    iDeclineTime[] = null; // Array of Decline Curve Time in pixels

  // -- Number of Wells Data

  private int    iWells[]       = null; // Array of Number of Wells in pixels
  private int    iWellsMin      = 0;    // Minimum Number of Wells
  private int    iWellsMax      = 0;    // Maximum Number of Wells

  // Plot Variables

  private int    iWidth         = 0;    // Actual Plotting Width
  private int    iHeight        = 0;    // Actual plotting Height
  private int    iXBegin        = 75;   // Left Edge of Plot Area
  private int    iYBegin        = 100;  // Top Edge of Plot Area

  /** CONSTRUCTOR iqstratPlot()
   *  <p> This is the constructor for this class.
   *  @param stPlot    = Production Plot Data Structure
   *  @param stDecline = Decline Curve Data Structure
   */

  public productionPlot( productionPlotStruct stPlot, declineStruct stDecline )
  {
	this.stPlot    = stPlot;
	this.stDecline = stDecline;

    if (stPlot != null)
    {
      iProduct = stPlot.iProduct;
      iFactor  = stPlot.iFactor;
    }

	iWidth      = stPlot.iWidth  - iXBegin; // Set the Actual Width
	iHeight     = stPlot.iHeight - iYBegin; // Set the Actual Height

    this.setSize(stPlot.iWidth, stPlot.iHeight);
    this.setBackground(Color.white);

//    dTimeIncrement = stPlot.dXincr;

    // Convert data to pixels

    setRawTimeData();
    setRawProductionData();

    setTimeData();
    setProductionData();
    setMonthlyAvgData();
    setDailyAvgData();
    setCumulativeData();
    setWellData();
    setDeclineData();
  }

  /** Method close()
   * <p> This method will set object to null to force JAVA to reallocate memory
   */

  public void close()
  {
	stPlot = null;

    iDeclineCurve  = OFF;  // Turn ON/OFF Decline Curve
    i12Avg         = OFF;  // Montly Curve ON/OFF Indicator
    i365Avg        = OFF;  // Daily Curve ON/OFF Indicator
    iWellsCurve    = OFF;  // Number of Wells Curve Off
    iRawCurve      = OFF;  // Raw Production Data Curve

  // -- Production Curve Data
    iTime          = null; // Array of Plot Time in pixels
    iProduction    = null; // Array of Production Data in pixels
    iCumulative     = null; // Array of Cumulative Data in pixels
    iMonthlyAvg    = null; // Array of Monthly Average in pixels
    iDailyAvg      = null; // Array of Daily Average in pixels

  // -- Plot Factors

    iFactor        = 1;    // Divide by Production factor
//    dTimeIncrement = 1.0;  // Plot Increment for Time

  // -- Decline Curve Data

    iDeclineRows   = 0;    // Number of Decline Data Rows
    iDecline       = null; // Array of Decline Curve Data in pixels
    iDeclineTime   = null; // Array of Decline Curve Time in pixels

  // -- Number of Wells Data

    iWells         = null; // Array of Number of Wells in pixels
    iWellsMin      = 0;    // Minimum Number of Wells
    iWellsMax      = 0;    // Maximum Number of Wells
  }


  /* =============================================================== *
   * ------------------------- GET METHODS ------------------------- *
   * =============================================================== */

  /** Method getPlotWidth()
   * <p> This method will return the Plot Width
   * @return iWidth = The Plot Width
   */

  public int getPlotWidth()  { return (stPlot.iWidth); }

  /** Method getPlotHeight()
   * <p> This method will return the Plot Height
   * @return iHeight = The Plot Height
   */

  public int getPlotHeight() { return (stPlot.iHeight); }

  /** get X-Axis Methods
   *  <p> These methods will retrieve the X-Axis Plot
   *

  public double getXMaximum()   { return (stPlot.dXmax); }
  public double getXMinimum()   { return (stPlot.dXmin); }
  public double getXIncrement() { return (stPlot.dXincr); }
  public int    getXCycles()    { return (stPlot.dXCycles); }
*/
  /** get Y-Axis Methods
   *  <p> These methods will retrieve the Y-Axis Plot
   *

  public double getYMaximum()   { return (stPlot.dYmax); }
  public double getYMinimum()   { return (stPlot.dYmin); }
  public double getYIncrement() { return (stPlot.dYincr); }
  public int    getYCycles()    { return (stPlot.dYCycles); }
*/
  /** Method getImage()
   * <p> This method will convert this plot to a buffered image
   * @return image = profile plot buffered image
   */

  public BufferedImage getImage()
  {
    BufferedImage image = null;
    int w = this.getWidth();
    int h = this.getHeight();

    image = new BufferedImage( w, h, BufferedImage.TYPE_INT_RGB );

    Graphics2D g2 = image.createGraphics();
    g2.setColor( Color.white );
    g2.fillRect( 0, 0, w, h );
    this.paint(g2);
    g2.dispose();

    return (image);
  }

  /* =============================================================== *
   * ------------------ SET CURVES ON/OFF METHODS ------------------ *
   * =============================================================== */

  /** Method setProductionCurve()
   *  <p> This method will set the Production & Cumulative Curves ON/OFF
   *  @param iValue = the value to set the Production Curve
   */

  public void setProductionCurve( int iValue )
  {
	this.iYAxis = iValue;
	repaint();
  }

  /** Method setDeclineCurve()
   *  <p> This method will set the Decline Curve ON or OFF
   *  @param iValue = the value to set for the decline curve
   */

  public void setDeclineCurve( int iValue )
  {
	this.iDeclineCurve = iValue;
	repaint();
  }

  /** Method setMonthlyAvgCurve()
   *  <p> This method will set the Monthly Average Curve ON or OFF
   *  @param iValue = the value to set for the Monthly Average curve
   */

  public void setMonthlyAvgCurve( int iValue )
  {
	this.i12Avg = iValue;
    repaint();
  }

  /** Method setDailyAvgCurve()
   *  <p> This method will set the Daily Average Curve ON or OFF
   *  @param iValue = the value to set for the Daily Average curve
   */

  public void setDailyAvgCurve( int iValue )
  {
	this.i365Avg = iValue;
	repaint();
  }

  /** Method setWellCurve()
   *  <p> This method will set the Plot Number of Wells Curve ON or OFF
   *  @param iValue = the value to set for the Number of Wells
   */

  public void setWellCurve( int iValue )
  {
	this.iWellsCurve = iValue;
	repaint();
  }

  /** Method setRawDataCurve()
   *  <p> This method will set the Plot Raw Data Curve ON or OFF
   *  @param iValue = the value to set for the Raw Data
   */

  public void setRawDataCurve( int iValue )
  {
	this.iRawCurve = iValue;
	repaint();
  }

  /* =============================================================== *
   * -------------------- SET PUBLIC DATA METHODS ------------------ *
   * =============================================================== */

  /** Method setTitles()
   * <p> This method will set the titles for this plot
   * @param sTitle1 = The first title
   * @param sTitle2 = The second title
   * @param sTitle3 = The third title
   */

  public void setTitles( String sTitle1, String sTitle2, String sTitle3 )
  {
	if (stPlot != null)
	{
	  stPlot.sTitle1 = new String( sTitle1 );
	  stPlot.sTitle2 = new String( sTitle2 );
	  stPlot.sTitle3 = new String( sTitle3 );
	  repaint();
    }
  }


  /** Method setX()
   *  <p> This method will set the X-Axis Values
   *  @param dXmax    = The Maximum X Values
   *  @param dXmin    = The Minimum X Values
   *  @param dXincr   = The Increment X Value
   *  @param iXCycles = the Number of X-Cycles
   */

  public void setX(double dXmax, double dXmin, double dXincr, int iXCycles)
  {
	if (stPlot != null)
	{
	  stPlot.dXmax    = dXmax;
	  stPlot.dXmin    = dXmin;
	  stPlot.dXincr   = dXincr;
	  stPlot.iXCycles = iXCycles;

      setTimeData();
      setRawTimeData();
      setDeclineData();
	  repaint();
	}
  }

  /** Method setY()
   *  <p> This method will set the Y-Axis Values
   *  @param dYmax    = The Maximum Y Values
   *  @param dYmin    = The Minimum Y Values
   *  @param dYincr   = The Increment Y Value
   *  @param iYCycles = the Number of Y-Cycles
   */

  public void setY(double dYmax, double dYmin, double dYincr, int iYCycles)
  {
	if (stPlot != null)
	{
	  stPlot.dYmax    = dYmax;
	  stPlot.dYmin    = dYmin;
	  stPlot.dYincr   = dYincr;
	  stPlot.iYCycles = iYCycles;

      setRawProductionData();

      setProductionData();
      setMonthlyAvgData();
      setDailyAvgData();
      setCumulativeData();
      setDeclineData();
	  repaint();
	}
  }

  /** Method setFactor()
   * <p> This method will change the factor
   * @param iFactor == The factor Identifier
   */

  public void setFactor(int iFactor) { this.iFactor = iFactor; }

  /** Method setData()
   * <p> This method will modify the plot image with a different product
   *  @param stPlot    = Production Plot Data Structure
   *  @param stDecline = Decline Curve Data Structure
   */

  public void setData(productionPlotStruct stPlot, declineStruct stDecline )
  {
	this.stPlot    = stPlot;
	this.stDecline = stDecline;

	if (stPlot != null)
	{
      iProduct = stPlot.iProduct;
      iFactor  = stPlot.iFactor;
/*
	  switch (iYAxis)
	  {
		case PRODUCTION:
		  setProductionVsTimePlot();
		  break;
		case CUMULATIVE:
		  setCumulativeVsTimePlot();
		  break;
		case BOTH:
		  setDefaultPlot();
		  break;
	  }
*/
	}

    setDefaultPlot();
    setDeclineData();
    repaint();
  }

  /** Method setDeclineData(declineStruct st)
   *  <p> This method will reset the Decline Curve Data Structure
   *  @param st = Decline Curve Data Structure
   */

  public void setDeclineData(declineStruct st)
  {
	this.stDecline = st;
    setDeclineData();
  }

  /** Method setDeclineData(declineStruct st)
   *  <p> This method will reset the Decline Curve Plot
   */

  public void refreshDecline()
  {
    setDeclineData();
    repaint();
  }

  /** Method setDefaultPlot()
   *  <p> This method will set the Default Plot Indicator to the original settings
   */

  public void setDefaultPlot()
  {
	iXAxis       = TIME;
	iYAxis       = BOTH;
	stPlot.iXLog = _LINEAR;
	stPlot.iYLog = _LOG;

    setRawTimeData();
    setRawProductionData();

//	setMinMaxValues();
	setTimeData();
	setProductionData();
	setMonthlyAvgData();
	setDailyAvgData();
	setCumulativeData();
	setWellData();
	setDeclineData();
  }

  /** Method setDataVsTimePlot()
   *  <p> This method will set the Data vs Time Plot Indicator to the original settings
   */

  public void setDataVsTimePlot()
  {
	iXAxis       = TIME;
	iYAxis       = BOTH;
	stPlot.iXLog = _LINEAR;

//    stPlot.dXincr = dTimeIncrement;

    setRawTimeData();
    setRawProductionData();

//	setMinMaxValues();
	setTimeData();
	setProductionData();
	setMonthlyAvgData();
	setDailyAvgData();
	setCumulativeData();
	setWellData();
	setDeclineData();
  }

  /** Method setCumulativeVsTimePlot()
   *  <p> This method will set the Cumulative vs Time Plot Indicator to plot
   *      Cumulative only data along the Y-Axis and time along the X-Axis
   */

  public void setCumulativeVsTimePlot()
  {
	iXAxis       = TIME;
	iYAxis       = CUMULATIVE;
	stPlot.iXLog = _LINEAR;

//    stPlot.dXincr = dTimeIncrement;

    setRawTimeData();
    setRawProductionData();

//	setMinMaxValues();
	setTimeData();
	setCumulativeData();
	setWellData();
  }

  /** Method setProductionVsTimePlot()
   *  <p> This method will set the Production vs Time Plot Indicator to plot
   *      Production only data along the Y-Axis and time along the X-Axis
   */

  public void setProductionVsTimePlot()
  {
	iXAxis       = TIME;
	iYAxis       = PRODUCTION;
	stPlot.iXLog = _LINEAR;

//    stPlot.dXincr = dTimeIncrement;

    setRawTimeData();
    setRawProductionData();

//	setMinMaxValues();
	setTimeData();
	setProductionData();
	setMonthlyAvgData();
	setDailyAvgData();
	setWellData();
	setDeclineData();
  }

  /** Method setGrid()
   *  <p> This method will set the Production Data to Linear or Log
   *  @param iGrid = The grid to use
   */

  public void setGrid(int iGrid)
  {
    stPlot.iXLog = _LINEAR;
    stPlot.iYLog = iGrid;

    setRawTimeData();
    setRawProductionData();

//	setMinMaxValues();
    setTimeData();
	setProductionData();
	setMonthlyAvgData();
	setDailyAvgData();
	setCumulativeData();
	setWellData();
	setDeclineData();
  }

  /* =============================================================== *
   * ------------------- SET PRIVATE DATA METHODS ------------------ *
   * =============================================================== */

  /** Method setMinMaxValues()
   *  <p> This method will set the minimum and maximum values based on the identifiers
   *

  private void setMinMaxValues()
  {
	if (stPlot != null)
	{
	  for (int i=0; i<stPlot.iRows; i++)
	  {
 	    if (i==0)
	    {
		  stPlot.dXmin = (double) stPlot.iTime[i];
		  stPlot.dXmax = (double) stPlot.iTime[i];

		  switch (iYAxis)
		  {
			case PRODUCTION:
		      stPlot.dYmin = stPlot.dProduction[i];
		      stPlot.dYmax = stPlot.dProduction[i];
			  break;
			case CUMULATIVE:
		      stPlot.dYmin = stPlot.dCumulative[i];
		      stPlot.dYmax = stPlot.dCumulative[i];
			  break;
			case BOTH:
		      stPlot.dYmin = stPlot.dProduction[i];
		      stPlot.dYmax = stPlot.dProduction[i];
			  break;
  	      }
		}

		if ((double) stPlot.iTime[i] < stPlot.dXmin)
		{
		  stPlot.dXmin = (double) stPlot.iTime[i];
		}

		if ((double) stPlot.iTime[i] > stPlot.dXmax)
		{
		  stPlot.dXmax = (double) stPlot.iTime[i];
		}

		switch (iYAxis)
		{
		  case PRODUCTION:
		    if ((stPlot.dProduction[i] < stPlot.dYmin) &&
		        (stPlot.dProduction[i] != 0.0))
		    {
		      stPlot.dYmin = stPlot.dProduction[i];
			}

		    if (stPlot.dProduction[i] > stPlot.dYmax)
		    {
		      stPlot.dYmax = stPlot.dProduction[i];
			}
		    break;
		  case CUMULATIVE:
		    if (stPlot.dCumulative[i] < stPlot.dYmin)
		    {
		      stPlot.dYmin = stPlot.dCumulative[i];
			}

		    if (stPlot.dCumulative[i] > stPlot.dYmax)
		    {
		      stPlot.dYmax = stPlot.dCumulative[i];
			}
		    break;
		  case BOTH:
		    if ((stPlot.dProduction[i] < stPlot.dYmin) &&
		        (stPlot.dProduction[i] != 0.0))
		    {
		      stPlot.dYmin = stPlot.dProduction[i];
			}

		    if (stPlot.dCumulative[i] > stPlot.dYmax)
		    {
		      stPlot.dYmax = stPlot.dCumulative[i];
			}
		    break;
		}
	  }

      int iCycleL = (int) Math.floor(Math.log(stPlot.dYmin) / Math.log(10.0));
      int iCycleU = (int) Math.ceil(Math.log(stPlot.dYmax) / Math.log(10.0));

      stPlot.iYCycles = (iCycleU-iCycleL);
      if (stPlot.iYCycles == 0)
        stPlot.iYCycles = 1;

      stPlot.dYmin = Math.pow(10.0, iCycleL);
      stPlot.dYmax = Math.pow(10.0, iCycleU);

      if (stPlot.iYLog == _LINEAR)
      {
    	stPlot.dYmin  = 0.0;
		stPlot.dYincr = stPlot.dYmax / 5.0;
	  }
	}
  }
*/
  /** Method setTimeData()
   *  <p> This method will set the Time Data to Pixels
   */

  private void setTimeData()
  {
	int    iRatio = 0;
	double dXmin  = 0.0;
	double dXmax  = 0.0;

    if (stPlot != null)
    {
      dXmin = stPlot.dXmin;
      dXmax = stPlot.dXmax;

      iTime = new int[stPlot.iRows];

      for (int i=0; i<stPlot.iRows; i++)
      {
        iRatio = (int) ((double) iWidth *
                        ((double) stPlot.iTime[i]-dXmin) / (dXmax - dXmin));
    	iTime[i] = iXBegin + iRatio;
	  }
    }
  }

  /** Method setRawTimeData()
   *  <p> This method will set the Raw Time Data to Pixels
   */

  private void setRawTimeData()
  {
	int    iRatio = 0;
	double dXmin  = 0.0;
	double dXmax  = 0.0;

    if (stPlot != null)
    {
      dXmin = stPlot.dXmin;
      dXmax = stPlot.dXmax;

      iRawTime = new int[stPlot.iRawRows];

      for (int i=0; i<stPlot.iRawRows; i++)
      {
        iRatio = (int) ((double) iWidth *
                        ((double) stPlot.dTime[i]-dXmin) / (dXmax - dXmin));
    	iRawTime[i] = iXBegin + iRatio;
	  }
    }
  }

  /** Method setProductionData()
   *  <p> This method will set the Production Data to Pixels
   */

  private void setProductionData()
  {
	int    i = 0;
	int    iRatio   = 0;
	double dYmin    = 0.0;
	double dYmax    = 0.0;
	double dMinimum = 0.0;
	double dMaximum = 0.0;
	double data     = 0.0;

	if (stPlot != null)
	{
	  dMinimum = stPlot.dYmin;
	  dMaximum = stPlot.dYmax;

	  iProduction = new int[stPlot.iRows];

	  if (stPlot.iYLog == _LINEAR)
	  {
		for (i=0; i<stPlot.iRows; i++)
		{
		  data   = stPlot.dProduction[i];
		  iRatio = (int) ((double) iHeight * (data-dMinimum) / (dMaximum-dMinimum));
          iProduction[i] = iHeight + iYBegin - iRatio;
		}
	  }
	  else
	  {
		dYmin = Math.log(dMinimum) / Math.log(10.0);
		dYmax = Math.log(dMaximum) / Math.log(10.0);

		for (i=0; i<stPlot.iRows; i++)
		{
		  data   = Math.log(stPlot.dProduction[i])/Math.log(10.0);
		  iRatio = (int) ((double) iHeight * (data-dYmin) / (dYmax-dYmin));
          iProduction[i] = iHeight + iYBegin - iRatio;
		}
	  }
	}
  }

  /** Method setRawProductionData()
   *  <p> This method will set the Raw Production Data to Pixels
   */

  private void setRawProductionData()
  {
	int    i = 0;
	int    iRatio   = 0;
	double dYmin    = 0.0;
	double dYmax    = 0.0;
	double dMinimum = 0.0;
	double dMaximum = 0.0;
	double data     = 0.0;

	if (stPlot != null)
	{
	  dMinimum = stPlot.dYmin;
	  dMaximum = stPlot.dYmax;

	  iRawData = new int[stPlot.iRawRows];

	  if (stPlot.iYLog == _LINEAR)
	  {
		for (i=0; i<stPlot.iRawRows; i++)
		{
		  data        = stPlot.dRaw[i];
		  iRatio      = (int) ((double) iHeight * (data-dMinimum) / (dMaximum-dMinimum));
          iRawData[i] = iHeight + iYBegin - iRatio;
		}
	  }
	  else
	  {
		dYmin = Math.log(dMinimum) / Math.log(10.0);
		dYmax = Math.log(dMaximum) / Math.log(10.0);

		for (i=0; i<stPlot.iRawRows; i++)
		{
		  data        = Math.log(stPlot.dRaw[i])/Math.log(10.0);
		  iRatio      = (int) ((double) iHeight * (data-dYmin) / (dYmax-dYmin));
          iRawData[i] = iHeight + iYBegin - iRatio;
		}
	  }
	}
  }

  /** Method setMonthlyAvgData()
   *  <p> This method will set the Production Monthly Average Data to Pixels
   */

  private void setMonthlyAvgData()
  {
	int    i = 0;
	int    iRatio   = 0;
	double dYmin    = 0.0;
	double dYmax    = 0.0;
	double dMinimum = 0.0;
	double dMaximum = 0.0;
	double dAvg     = 0.0;

	if (stPlot != null)
	{
	  dMinimum = stPlot.dYmin;
	  dMaximum = stPlot.dYmax;

	  iMonthlyAvg = new int[stPlot.iRows];

	  if (stPlot.iYLog == _LINEAR)
	  {
		for (i=0; i<stPlot.iRows; i++)
		{
		  dAvg = stPlot.dProduction[i]/12.0;
		  iRatio = (int) ((double) iHeight * (dAvg-dMinimum) / (dMaximum-dMinimum));
          iMonthlyAvg[i] = iHeight + iYBegin - iRatio;
		}
	  }
	  else
	  {
		dYmin = Math.log(dMinimum) / Math.log(10.0);
		dYmax = Math.log(dMaximum) / Math.log(10.0);

		for (i=0; i<stPlot.iRows; i++)
		{
		  dAvg = stPlot.dProduction[i]/12.0;
		  iRatio = (int) ((double) iHeight *
		                  ((Math.log(dAvg)/Math.log(10.0))-dYmin) / (dYmax-dYmin));
          iMonthlyAvg[i] = iHeight + iYBegin - iRatio;
		}
	  }
	}
  }

  /** Method setDailyAvgData()
   *  <p> This method will set the Production Daily Average Data to Pixels
   */

  private void setDailyAvgData()
  {
	int    i = 0;
	int    iRatio   = 0;
	double dYmin    = 0.0;
	double dYmax    = 0.0;
	double dMinimum = 0.0;
	double dMaximum = 0.0;
	double dAvg     = 0.0;

	if (stPlot != null)
	{
	  dMinimum = stPlot.dYmin;
	  dMaximum = stPlot.dYmax;

	  iDailyAvg = new int[stPlot.iRows];

	  if (stPlot.iYLog == _LINEAR)
	  {
		for (i=0; i<stPlot.iRows; i++)
		{
		  dAvg = stPlot.dProduction[i]/365.0;
		  iRatio = (int) ((double) iHeight * (dAvg-dMinimum) / (dMaximum-dMinimum));
          iDailyAvg[i] = iHeight + iYBegin - iRatio;
		}
	  }
	  else
	  {
		dYmin = Math.log(dMinimum) / Math.log(10.0);
		dYmax = Math.log(dMaximum) / Math.log(10.0);

		for (i=0; i<stPlot.iRows; i++)
		{
		  dAvg = stPlot.dProduction[i]/365.0;
		  iRatio = (int) ((double) iHeight *
		                  ((Math.log(dAvg)/Math.log(10.0))-dYmin) / (dYmax-dYmin));
          iDailyAvg[i] = iHeight + iYBegin - iRatio;
		}
	  }
	}
  }

  /** Method setCumulativeData()
   *  <p> This method will set the Cumulative Data to Pixels
   */

  private void setCumulativeData()
  {
	int    i = 0;
	int    iRatio   = 0;
	double dYmin    = 0.0;
	double dYmax    = 0.0;
	double dMinimum = 0.0;
	double dMaximum = 0.0;
    int    iStart   = 0;
    int    iDim     = 0;

	if (stPlot != null)
	{
      dMinimum = stPlot.dYmin;
      dMaximum = stPlot.dYmax;
      iStart   = iYBegin;
      iDim     = iHeight;

	  iCumulative = new int[stPlot.iRows];

	  if (stPlot.iYLog == _LINEAR)
	  {
		for (i=0; i<stPlot.iRows; i++)
		{
		  iRatio = (int) ((double) iDim *
		                  (stPlot.dCumulative[i]-dMinimum) / (dMaximum-dMinimum));
          iCumulative[i] = iDim + iStart - iRatio;
		}
	  }
	  else
	  {
		dYmin = Math.log(dMinimum) / Math.log(10.0);
		dYmax = Math.log(dMaximum) / Math.log(10.0);

		for (i=0; i<stPlot.iRows; i++)
		{
		  iRatio = (int) ((double) iDim *
		                  ((Math.log(stPlot.dCumulative[i])/Math.log(10.0))-dYmin) / (dYmax-dYmin));
          iCumulative[i] = iDim + iStart - iRatio;
		}
	  }
	}
  }

  /** Method setWellData()
   *  <p> This method will set the Well Data to Pixels
   */

  private void setWellData()
  {
	int    i=0;
	int    iRatio = 0;
	double dXmin  = 0.0;
	double dYmin  = 0.0;

	if (stPlot != null)
	{
	  iWells = new int[stPlot.iRows];

	  for (i=0; i<stPlot.iRows; i++)
	  {
		iWells[i] = 1;

		if (i== 0)
		{
		  iWellsMin = stPlot.iSources[i];
		  iWellsMax = stPlot.iSources[i];
		}

		if (iWellsMin > stPlot.iSources[i])
		  iWellsMin = stPlot.iSources[i];

		if (iWellsMax < stPlot.iSources[i])
		  iWellsMax = stPlot.iSources[i];
	  }

	  if (iWellsMax == iWellsMin) { iWellsMax = iWellsMin+1; }

	  for (i=0; i<stPlot.iRows; i++)
	  {
		iRatio = (int) (iHeight * (stPlot.iSources[i]-iWellsMin) / (iWellsMax-iWellsMin));
		iWells[i] = iHeight + iYBegin - iRatio;
	  }
	}
  }

  /** Method setDeclineData()
   *  <p> This method will set the Decline Curve Data to Pixels
   */

  private void setDeclineData()
  {
	int    i=0;
	int    iRatio   = 0;
	double dXmin    = 0.0;
	double dXmax    = 0.0;
	double dYmin    = 0.0;
	double dYmax    = 0.0;
	double dMinimum = 0.0;
	double dMaximum = 0.0;

	double iYears   = 0;
	double di       = 0.0;
	double dYear    = 0.0;
	int    iCount   = 0;
	double dQ       = 0.0;
	double data     = 0.0;

	if (stPlot != null)
	{
	  dMinimum     = stPlot.dYmin;
	  dMaximum     = stPlot.dYmax;
	  iDeclineRows = 0;

	  if (stDecline != null)
	  {
		iYears = (int) (stPlot.dXmax - stPlot.dXmin);
		di     = stDecline.Qi_year;

		for (i=0; i<iYears; i++)
		{
		  dYear = stPlot.dXmin + i;
		  if (dYear >= di)
		    iDeclineRows++;
		}

		iCount       = 0;
		iDecline     = new int[iDeclineRows];
		iDeclineTime = new int[iDeclineRows];

		for (i=0; i<iYears; i++)
		{
	      dYear = stPlot.dXmin + i;

	      if (dYear >= di)
	      {
			if (iCount < iYears)
			{
			  iRatio = (int) ((double) iWidth * (dYear-stPlot.dXmin) / (stPlot.dXmax-stPlot.dXmin));
			  iDeclineTime[iCount] = iXBegin + iRatio;

              dQ = production.math.declineMath.computeQ(
				     stDecline.iMethod, stDecline.iRate, stDecline.n, stDecline.K,
                                stDecline.Qi, stDecline.Qi_year, (int) dYear) / iFactor;

			  if (stPlot.iYLog == _LINEAR)
			  {
				iRatio = (int) ((double) iHeight * (dQ-dMinimum) / (dMaximum-dMinimum));
				iDecline[iCount] = iHeight + iYBegin - iRatio;
			  }
			  else
			  {
				dYmin = Math.log(dMinimum) / Math.log(10.0);
				dYmax = Math.log(dMaximum) / Math.log(10.0);

    	 	    data  = Math.log(dQ)/Math.log(10.0);
				iRatio = (int) ((double) iHeight * (data-dYmin) / (dYmax-dYmin));
				iDecline[iCount] = iHeight + iYBegin - iRatio;
			  }

			  iCount++;
			}
		  }
		}
	  }
    }
  }

  /* =============================================================== *
   * ------------------------- DRAW METHODS ------------------------ *
   * =============================================================== */

  /** Method drawTitles()
   *  <p> This method will draw the titles of the Plot
   *  @param g = Graphics pointer
   */

  public void drawTitles(Graphics g)
  {
	int iTemp = 0;

	g.setColor(Color.black);
	g.setFont( new Font("Monospaced", Font.BOLD, 16) );

	if (stPlot.sTitle1.length() > 0)
	{
	  iTemp = (int) (Math.abs(iWidth + iXBegin - (8*stPlot.sTitle1.length())));
	  g.drawString(stPlot.sTitle1, iTemp/2, 30);
	}

	if (stPlot.sTitle2.length() > 0)
	{
	  iTemp = (int) (Math.abs(iWidth + iXBegin - (8*stPlot.sTitle2.length())));
	  g.drawString(stPlot.sTitle2, iTemp/2, 50);
	}

	if (stPlot.sTitle3.length() > 0)
	{
	  iTemp = (int) (Math.abs(iWidth + iXBegin - (8*stPlot.sTitle3.length())));
	  g.drawString(stPlot.sTitle3, iTemp/2, 70);
	}
  }

  /** Method drawTimeAxisGrid()
   *  <p> This method will draw the Linear X-Axis Grid Lines
   *  @param g = graphics pointer
   */

  public void drawTimeAxisGrid(Graphics g)
  {
    int i = 0;
    int    iXinc  = 0;
    int    iLabel = 0;
    String sLabel = new String("");
    int    idraw  = 0;
    int    idel   = 0;
    double dTime  = 0.0;
    int    iRatio = 0;
    int    inc    = 5;

    Font fsb = new Font("Serif", Font.BOLD, 12); //10);
    g.setFont(fsb);

	g.setColor(Color.darkGray);
	g.drawLine(iXBegin, iYBegin, iXBegin, iYBegin+iHeight);
	g.drawLine(iXBegin+iWidth, iYBegin, iXBegin+iWidth, iYBegin+iHeight);

	g.setColor(Color.lightGray);

	iXinc = (int) ((double)iWidth * stPlot.dXincr / (stPlot.dXmax-stPlot.dXmin));

	i = 0;
	idraw = iXBegin;
	while (idraw <= iXBegin+iWidth)
	{
	  dTime  = stPlot.dXmin + (double) i * stPlot.dXincr;
	  iRatio = (int) ((double) iWidth * (dTime-stPlot.dXmin) / (stPlot.dXmax-stPlot.dXmin));
      idraw  = iXBegin + iRatio;

	  if (idraw <= iXBegin+iWidth)
	  {
		g.setColor(Color.black);
//	    g.setColor(Color.lightGray);
	    g.drawLine(idraw, iYBegin, idraw, iYBegin+iHeight);

        iLabel = (int) dTime;
		sLabel = new String("" + iLabel);

//		g.setColor(Color.black);
//		if (sLabel.length() == 4)
//		  g.drawString(sLabel.substring(2), idraw-5, iYBegin+iHeight+15);
//		else
		  g.drawString(sLabel, idraw-5, iYBegin+iHeight+15);

        inc = (int) stPlot.dXincr;

        for (int j=1; j<inc; j++)
        {
          idel = idraw + (j * iXinc) / inc;
          if (idel < iXBegin+iWidth)
          {
            g.setColor(Color.lightGray);
            g.drawLine(idel, iYBegin, idel, iYBegin+iHeight);
          }
        }
	  }

	  i++;
    }
  }

  /** Method drawLinearXAxisGrid()
   *  <p> This method will draw Linear X-Axis Grid Lines
   *  @param g = Graphics pointer
   */

  public void drawLinearXAxisGrid(Graphics g)
  {
    int    i      = 0;
    int    j      = 0;
    int    idraw  = 0;
    int    idel   = 0;
    int    iRatio = 0;
    int    iXinc  = 0;
    double dx     = 0.0;
    double del    = 0.0;

    iXinc = i + (int) ((double) iWidth * stPlot.dXincr / (stPlot.dXmax - stPlot.dXmin));

    idraw = iXBegin;
    while (idraw < iXBegin+iWidth+5)
    {
      dx     = stPlot.dXmin + (double) i * stPlot.dXincr;
      iRatio = (int) ((double) iWidth * (dx - stPlot.dXmin) / (stPlot.dXmax - stPlot.dXmin));
      idraw  = iXBegin + iRatio;

      if (idraw < iXBegin+iWidth+5)
      {
        g.setColor(Color.darkGray);
        g.drawLine(idraw, iYBegin, idraw, iYBegin+iHeight);

        g.setColor(Color.black);
        g.setFont( new Font("Monospaced", Font.BOLD, 10) );

        dx = Math.round(dx*1000)/1000.0;
        g.drawString(""+dx, idraw-10, iYBegin + iHeight + 15);

        for (j=1; j<5; j++)
        {
          idel = idraw + (j * iXinc) / 5;
          if (idel < iXBegin+iWidth)
          {
            g.setColor(Color.lightGray);
            g.drawLine(idel, iYBegin, idel, iYBegin+iHeight);
          }
        }
      }

      i++;
    }
  }

  /** Method drawLogXAxisGrid()
   *  <p> This method will draw Log X-Axis Grid Lines
   *  @param g = Graphics pointer
   */

  public void drawLogXAxisGrid(Graphics g)
  {
    int    i,j;
    int    l=1;
    int    k=0;
    int    iXinc  = 0;
    double dLabel = 0.0;
    double del    = 0.0;
    int    jXinc  = 0;
    double dTemp  = 0.0;
    double dXinc  = 0.0;

    iXinc = 1 + iWidth / stPlot.iXCycles;

    // Draw the X-Axis Grid Lines

    for (i=iXBegin+iXinc; i<iXBegin+iWidth+10; i+=iXinc)
    {
	  // Draw the Major Cycle Grid Lines Dark Gray

	  g.setColor(Color.darkGray);
	  g.drawLine(i, iYBegin, i, iYBegin+iHeight);

	  // Draw the Minor Cycle Grid Lines Light Gray

	  g.setColor(Color.lightGray);

	  for (j=2; j<10; j++)
	  {
		dXinc = (double) j;
		dTemp = 1.0 + (double) iXinc * Math.log(dXinc) / Math.log(10.0);
		jXinc = i - iXinc + (int) dTemp;

		g.drawLine(jXinc, iYBegin, jXinc, iYBegin+iHeight);
	  }
	}

	// Draw the X-Axis Numbers for the Major Axis

	Font fsb = new Font("Serif", Font.BOLD, 10);
	g.setFont( fsb );

	g.setColor(Color.black);
	k=l;
	del = stPlot.dXincr;
	g.drawString("" + stPlot.dXmin, iXBegin-10, iYBegin+iHeight+15);

	for (i=iXBegin+iXinc; i<iXBegin+iWidth+10; i+=iXinc);
	{
	  dLabel = (double) k * 10.0 * stPlot.dXmin;
	  k = k * 10;

	  g.setColor(Color.black);
	  g.drawString("" + dLabel, i-10, iYBegin+iHeight+15);
	  del = stPlot.dXincr + del;
	}
  }

  /** Method drawLinearYAxisGrid()
   *  <p> This method will draw Linear Y-Axis Grid Lines
   *  @param g = Graphics pointer
   *  @param iData = The data to be plotted, Production to left Cumulative to right
   */

  public void drawLinearYAxisGrid(Graphics g, int iData)
  {
    int      i,j;
    int      l=1;
    int      k=0;
    int      iYinc      = 0;
    double   dLabel     = 0.0;
    String   sLabel     = new String("");
    double   del        = 0.0;
    int      iXPosition = 0;
    Color    color      = Color.black;
    Graphics g2         = (Graphics2D)g;

    // Set the Position of the Major Axis Grid Number position and the color

    switch (iData)
    {
	  case PRODUCTION:
	    iXPosition = iXBegin-50;
	    color = Color.black;
	    break;
	  case CUMULATIVE:
	    if (iYAxis == BOTH)
	    {
  	      iXPosition = iWidth+iXBegin+2;
	      color = Color.blue;
		}
		else
		{
	      iXPosition = iXBegin-50;
	      color = Color.black;
		}
	    break;
	}

	iYinc = 1 + (int) (((double) iHeight * stPlot.dYincr) / (stPlot.dYmax-stPlot.dYmin));

	for (i=iYBegin+iYinc; i<iYBegin+iHeight+10; i+=iYinc)
	{
	  // Draw the Minor Cycle Grid Lines Light Gray

	  l=0;
	  g.setColor(Color.lightGray);

	  for (j=i-iYinc+iYinc/5; j<i; j+=iYinc/5)
	  {
		l++;

		// Draw the Major Cycle Grid Lines Dark Gray

		if (l==5)
		  g.setColor(Color.darkGray);

		g.drawLine(iXBegin, j, iXBegin+iWidth, j);
	  }
	}

	// Draw the Upper Major Cycle Grid Lines Dark Gray

	g.setColor(Color.darkGray);
	g.drawLine(iXBegin, iYBegin, iXBegin+iWidth, iYBegin);

	// Draw the Y-Axis Numbers for the Major Axis

	Font fsb = new Font("Serif", Font.BOLD, 10);
	g.setFont( fsb );

	k=l;
	for (i=iYBegin+iYinc; i<iYBegin+iHeight+10; i+=iYinc)
	{
	  dLabel = stPlot.dYmin + (double) k * stPlot.dYincr;
	  sLabel = new String("" + dLabel);
	  k++;

	  g.setColor(color);
	  g2.drawString(sLabel, iXPosition, 2*iYBegin+iHeight-i);
	}
  }

  /** Method drawLogYAxisGrid()
   *  <p> This method will draw Log Y-Axis Grid Lines
   *  @param g = Graphics pointer
   *  @param iData = The data to be plotted, Production to left Cumulative to right
   */

  public void drawLogYAxisGrid(Graphics g, int iData)
  {
    int      i,j;
    int      l=1;
    int      k=0;
    int      iYinc      = 0;
    double   dLabel     = 0.0;
    String   sLabel     = new String("");
    double   del        = 0.0;
    int      jXinc      = 0;
    int      iXPosition = 0;
    Color    color      = Color.black;
    double   dTemp      = 0.0;
    double   dXinc      = 0.0;
    Graphics g2         = (Graphics2D)g;

    // Set the Position of the Major Axis Grid Number position and the color

    switch (iData)
    {
	  case PRODUCTION:
	    iXPosition = iXBegin-50;
	    color = Color.black;
	    break;
	  case CUMULATIVE:
	    if (iYAxis == BOTH)
	    {
  	      iXPosition = iWidth+iXBegin+2;
	      color = Color.blue;
		}
		else
		{
	      iXPosition = iXBegin-50;
	      color = Color.black;
		}
	    break;
	}

	iYinc = 1 + iHeight / stPlot.iYCycles;

	// Draw the Y-Axis Grid Lines

	for (i=iYBegin+iHeight; i>iYBegin+iYinc-10; i-=iYinc)
	{
	  // Draw the Major Cycle Grid Lines Dark Gray

	  g.setColor(Color.darkGray);
	  g.drawLine(iXBegin, i, iXBegin+iWidth, i);

	  // Draw the Minor Cycle Grid Lines Light Gray

	  g.setColor(Color.lightGray);

	  for (j=9; j>1; j--)
	  {
		  dXinc = (double)j;
		  dTemp = 1.0 + (double) iYinc * Math.log(dXinc) / Math.log(10.0);
		  jXinc = i - (int) dTemp;

		  g.drawLine(iXBegin, jXinc, iXBegin+iWidth, jXinc);
	  }
	}

	// Draw the Upper Major Cycle Grid Lines Dark Gray

	g.setColor(Color.darkGray);
	g.drawLine(iXBegin, iYBegin-3, iXBegin+iWidth, iYBegin-3);

	// Draw the Y-Axis Number for the Major Axis

	Font fsb = new Font("Serif", Font.BOLD, 12); //10);
	g.setFont( fsb );
	g.setColor(color);

	k=1;
	del    = stPlot.dYincr;
	dLabel = stPlot.dYmin;
	sLabel = new String("" + dLabel);
	g2.drawString(sLabel, iXPosition, iYBegin+iHeight);

	for (i=iYBegin+iYinc; i<iYBegin+iHeight+10; i+=iYinc)
	{
	  dLabel = (double) k * 10.0 * stPlot.dYmin;
	  sLabel = new String("" + dLabel);
	  k=k*10;

	  g2.drawString(sLabel, iXPosition, 2*iYBegin+iHeight-i);
	}
  }

  /** Method drawYLabels()
   *  <p> This method will draw the Linear Plot Labels
   *  @param g = Graphics pointer
   *  @param iData = The data
   */

  public void drawYLabels(Graphics g, int iData)
  {
	int    ivalue     = 0;
	int    iXPosition = 0;
	String sText      = new String("");
	Color  color      = Color.black;

	switch (iData)
	{
	  case PRODUCTION:
	    iXPosition = -10;
	    color = Color.black;
	    sText = new String("Annual Production " + stPlot.sYlabel);
	    break;

	  case CUMULATIVE:
	    switch (iYAxis)
	    {
		  case CUMULATIVE:
	        iXPosition = -10;
	        color = Color.black;
	        break;
	      case BOTH:
	        iXPosition = -50 - iWidth - iXBegin; //-10 - iWidth - iXBegin;
	        color = Color.blue;
	        break;
		}

		sText = new String("Cumulative " + stPlot.sYlabel);
	    break;
	}

	g.setColor(color);
    g.setFont( new Font("Monospaced", Font.BOLD, 15)); //13) );

	ivalue = (iHeight+100)/2 - 3*(sText).length();

	Graphics2D g2 = (Graphics2D)g;
	g2.rotate((Math.PI / 2.0));
	g2.drawString(sText, ivalue, iXPosition);
	g2.rotate(-1.0*(Math.PI / 2.0));
  }

  /** Method drawXLabels()
   * <p> This method will draw the Linear Plot Labels
   * @param g = Graphics pointer
   */

  public void drawXLabels(Graphics g)
  {
	int    ivalue = 0;
	String sText  = new String("");

	g.setColor(Color.black);
    g.setFont( new Font("Monospaced", Font.BOLD, 15) );

    ivalue = (iWidth+100)/2 - 3*(stPlot.sXlabel).length();
    g.drawString(stPlot.sXlabel, ivalue, iYBegin+iHeight+35);
  }

  /** Method drawProductionVsTimeCurves()
   *  <p> This method will draw the Production curves.
   *  @param g     = Graphics pointer
   *  @param color = The color of the curve
   */

  public void drawProductionVsTimeCurves(Graphics g, Color color)
  {
	g.setColor(color);

	if (stPlot != null)
	{
	  for (int i=0; i<stPlot.iRows-1; i++)
	  {
		if ((iProduction[i]   <= (iHeight+iYBegin)) &&
		    (iProduction[i+1] <= (iHeight+iYBegin)) &&
		    (iProduction[i]   >= iYBegin)           &&
		    (iProduction[i+1] >= iYBegin)           &&
		    (iTime[i]         <= (iWidth+iXBegin))  &&
		    (iTime[i+1]       <= (iWidth+iXBegin))  &&
		    (iTime[i]         >= iXBegin)           &&
		    (iTime[i+1]       >= iXBegin))
		{
		  g.drawLine(iTime[i], iProduction[i], iTime[i+1], iProduction[i+1]);
		}
	  }
	}
  }

  /** Method drawRawProductionVsTimeCurves()
   *  <p> This method will draw the Raw Production curves.
   *  @param g     = Graphics pointer
   *  @param color = The color of the curve
   */

  public void drawRawProductionVsTimeCurves(Graphics g, Color color)
  {
	int iY = 0;

	g.setColor(color);

	if (stPlot != null)
	{
	  for (int i=0; i<stPlot.iRawRows-1; i++)
	  {
		if ((iRawData[i]   <= (iHeight+iYBegin)) &&
		    (iRawData[i+1] <= (iHeight+iYBegin)) &&
		    (iRawData[i]   >= iYBegin)           &&
		    (iRawData[i+1] >= iYBegin)           &&
		    (iRawTime[i]   <= (iWidth+iXBegin))  &&
		    (iRawTime[i+1] <= (iWidth+iXBegin))  &&
		    (iRawTime[i]   >= iXBegin)           &&
		    (iRawTime[i+1] >= iXBegin))
		{
		  g.drawLine(iRawTime[i], iRawData[i], iRawTime[i+1], iRawData[i+1]);
		}
	  }

      iY = iDeclineCurve * 15;

  	  g.drawLine(iWidth-50, iYBegin-25+iY, iWidth-35, iYBegin-25+iY);
	  g.drawString("Raw Data", iWidth-30, iYBegin-25+iY);
	}
  }

  /** Method drawMonthlyAvgVsTimeCurves()
   *  <p> This method will draw the Monthly Average Production curves.
   *  @param g     = Graphics pointer
   *  @param color = The color of the curve
   */

  public void drawMonthlyAvgVsTimeCurves(Graphics g, Color color)
  {
	int iY = 0;

	g.setColor(color);

	if (stPlot != null)
	{
	  for (int i=0; i<stPlot.iRows-1; i++)
	  {
		if ((iMonthlyAvg[i]   <= (iHeight+iYBegin)) &&
		    (iMonthlyAvg[i+1] <= (iHeight+iYBegin)) &&
		    (iMonthlyAvg[i]   >= iYBegin)           &&
		    (iMonthlyAvg[i+1] >= iYBegin)           &&
		    (iTime[i]         <= (iWidth+iXBegin))  &&
		    (iTime[i+1]       <= (iWidth+iXBegin))  &&
		    (iTime[i]         >= iXBegin)           &&
		    (iTime[i+1]       >= iXBegin))
		{
		  g.drawLine(iTime[i], iMonthlyAvg[i], iTime[i+1], iMonthlyAvg[i+1]);
		}
	  }

      iY = (iWellsCurve+i365Avg) * 15;
//	  g.drawLine(iWidth-50, iYBegin-70, iWidth-35, iYBegin-70);
	  g.drawLine(iWidth-50, iYBegin-(40+iY), iWidth-35, iYBegin-(40+iY));
	  g.drawString("Monthly Average", iWidth-30, iYBegin-(40+iY));
	}
  }

  /** Method drawDailyAvgVsTimeCurves()
   *  <p> This method will draw the Daily Average Production curves.
   *  @param g     = Graphics pointer
   *  @param color = The color of the curve
   */

  public void drawDailyAvgVsTimeCurves(Graphics g, Color color)
  {
	int iY = 0;

	g.setColor(color);

	if (stPlot != null)
	{
	  for (int i=0; i<stPlot.iRows-1; i++)
	  {
		if ((iDailyAvg[i]   <= (iHeight+iYBegin)) &&
		    (iDailyAvg[i+1] <= (iHeight+iYBegin)) &&
		    (iDailyAvg[i]   >= iYBegin)           &&
		    (iDailyAvg[i+1] >= iYBegin)           &&
		    (iTime[i]         <= (iWidth+iXBegin))  &&
		    (iTime[i+1]       <= (iWidth+iXBegin))  &&
		    (iTime[i]         >= iXBegin)           &&
		    (iTime[i+1]       >= iXBegin))
		{
		  g.drawLine(iTime[i], iDailyAvg[i], iTime[i+1], iDailyAvg[i+1]);
		}
	  }

      iY = iWellsCurve * 15;
	  g.drawLine(iWidth-50, iYBegin-(40+iY), iWidth-35, iYBegin-(40+iY));
	  g.drawString("Daily Average", iWidth-30, iYBegin-(40+iY));
	}
  }

  /** Method drawCumulativeVsTimeCurves()
   *  <p> This method will draw the Production curves.
   *  @param g     = Graphics pointer
   *  @param color = The color of the curve
   */

  public void drawCumulativeVsTimeCurves(Graphics g, Color color)
  {
	g.setColor(color);

	if (stPlot != null)
	{
	  for (int i=0; i<stPlot.iRows-1; i++)
	  {
		if ((iCumulative[i]   <= (iHeight+iYBegin)) &&
		    (iCumulative[i+1] <= (iHeight+iYBegin)) &&
		    (iCumulative[i]   >= iYBegin)           &&
		    (iCumulative[i+1] >= iYBegin)           &&
		    (iTime[i]         <= (iWidth+iXBegin))  &&
		    (iTime[i+1]       <= (iWidth+iXBegin))  &&
		    (iTime[i]         >= iXBegin)           &&
		    (iTime[i+1]       >= iXBegin))
		{
		  g.drawLine(iTime[i], iCumulative[i], iTime[i+1], iCumulative[i+1]);
		}
	  }
	}
  }

  /** Method drawNumberofWellsVsTimeCurves()
   *  <p> This method will draw the Production curves.
   *  @param g     = Graphics pointer
   *  @param color = The color of the curve
   */

  public void drawNumberofWellsVsTimeCurves(Graphics g, Color color)
  {
	g.setColor(color);

	if (stPlot != null)
	{
	  for (int i=0; i<stPlot.iRows-1; i++)
	  {
		if ((iWells[i]   <= (iHeight+iYBegin)) &&
		    (iWells[i+1] <= (iHeight+iYBegin)) &&
		    (iWells[i]   >= iYBegin)           &&
		    (iWells[i+1] >= iYBegin)           &&
		    (iTime[i]    <= (iWidth+iXBegin))  &&
		    (iTime[i+1]  <= (iWidth+iXBegin))  &&
		    (iTime[i]    >= iXBegin)           &&
		    (iTime[i+1]  >= iXBegin))
		{
		  g.drawLine(iTime[i], iWells[i], iTime[i+1], iWells[i+1]);
		  Font fsb = new Font("Serif", Font.BOLD, 9);
		  g.setFont( fsb );
		  g.drawString(""+stPlot.iSources[i], iTime[i]+5, iWells[i]);
		}
	  }

	  g.drawLine(iWidth-50, iYBegin-40, iWidth-35, iYBegin-40);
	  g.drawString("Number of Wells", iWidth-30, iYBegin-40);
	}
  }

  /** Method drawDeclineVsTimeCurves()
   *  <p> This method will draw the Decline Production curves.
   *  @param g     = Graphics pointer
   *  @param color = The color of the curve
   */

  public void drawDeclineVsTimeCurves(Graphics g, Color color)
  {
	g.setColor(color);

    for (int i=0; i<iDeclineRows-1; i++)
	{
	  if ((iDecline[i]       <= (iHeight+iYBegin)) &&
	      (iDecline[i+1]     <= (iHeight+iYBegin)) &&
	      (iDecline[i]       >= iYBegin)           &&
	      (iDecline[i+1]     >= iYBegin)           &&
	      (iDeclineTime[i]   <= (iWidth+iXBegin))  &&
	      (iDeclineTime[i+1] <= (iWidth+iXBegin))  &&
	      (iDeclineTime[i]   >= iXBegin)           &&
	      (iDeclineTime[i+1] >= iXBegin))
	  {
	    g.drawLine(iDeclineTime[i], iDecline[i], iDeclineTime[i+1], iDecline[i+1]);
	  }
	}

	g.drawLine(iWidth-50, iYBegin-25, iWidth-35, iYBegin-25);
	g.drawString("Decline Curve", iWidth-30, iYBegin-25);
  }

  /** Method drawDeclineData()
   *  <p> This method will draw the Decline curves.
   *  @param g     = Graphics pointer
   *  @param color = The color of the curve
   */

  public void drawDeclineData(Graphics g, Color color)
  {
	int iYo   = 0;
	int iYdel = 18; //12;
	int iXo   = 0;

	Font fsb = new Font("Serif", Font.BOLD, 13); //11);
	g.setFont( fsb );
	g.setColor(color);

	if (stDecline != null)
	{
	  String sMethod   = production.declineStruct.METHOD[stDecline.iMethod];
	  String sEquation = production.declineStruct.EQUATION_TEXT[stDecline.iMethod];
	  String sUnits    = production.declineStruct.UNITS[iProduct];
	  int    iRate_btn = stDecline.iRate;

	  String sRate_btn = new String("Year");
	  if (iRate_btn == MONTHLY)
	    sRate_btn = new String("Month");
	  if (iRate_btn == DAILY)
	    sRate_btn = new String("Day");

	  double dQi       = stDecline.Qi;
	  int    iQi       = stDecline.Qi_year;
	  double dQt       = stDecline.Qt;
	  int    iQt       = stDecline.Qt_year;

	  double decline   = stDecline.decline;
	  double dK        = stDecline.K;
	  double dn        = stDecline.n;

	  double dQf       = stDecline.Qf;
	  int    iQf       = stDecline.Qf_year;
	  String CUMM_A    = new String("" + (int) stDecline.CUMM_A);
	  String CUMM      = new String("" + (int) stDecline.CUMM_C);
	  String EUR       = new String("" + (int) stDecline.EUR);
	  String RR        = new String("" + (int) stDecline.RR);

	  if ((dQi > 0.0) && (iQi > 0) && (dQt > 0.0) && (iQt > 0))
	  {
		iXo = iXBegin - 70;
		iYo = iYBegin + iHeight + 45;

		g.drawString(sMethod + "   (K,n) = (" + dK + ", " + dn + ")", iXo, iYo);
		iYo = iYo + iYdel;

		g.drawString("  Initial Production " + sRate_btn + " Rate (Qi): " +
		             (int) dQi + " " + sUnits + " at " + iQi, iXo, iYo);
		iYo = iYo + iYdel;

		g.drawString("  Present Production " + sRate_btn + " Rate (Qt): " +
		             (int) dQt + " " + sUnits + " at " + iQt, iXo, iYo);
		iYo = iYo + iYdel;

     	g.drawString("  Decline Rate (De): " + decline + " % Per Year", iXo, iYo);
		iYo = iYo + iYdel;

		if ((dQf > 0.0) && (iQf > 0))
		{
		  g.drawString("  Economic Limit " + sRate_btn + " Rate (Qf): " +
		             dQf + " " + sUnits + " at " + iQf, iXo, iYo);
		  iYo = iYo + iYdel;
		}

		iXo = iXBegin + 350; //250;
		iYo = iYBegin + iHeight + 57;
		g.drawString("Cumulative to Data: " + CUMM_A + " " + sUnits, iXo, iYo);
		iYo = iYo + iYdel;

		g.drawString("          Computed: " + CUMM + " " + sUnits, iXo, iYo);
		iYo = iYo + iYdel;

		if ((dQf > 0.0) && (iQf > 0))
		{
		  g.drawString("Cumulative at Limit: " + EUR + " " + sUnits, iXo, iYo);
   		  iYo = iYo + iYdel;
   		  g.drawString("Remaining Reserves: "  + RR + " " + sUnits, iXo, iYo);
		}
	  }
	}
  }

  /** Method drawDataVsTime()
   *  <p> This method will draw the Data vs Time depending on the \
   *      Plot Identifier settings.
   *  @param g     = Graphics Pointer
   *  @param iAxis = The production data identifier to plot on the y-axis
   */

  public void drawDataVsTime(Graphics g, int iAxis)
  {
	// Draw Time X-Axis Labels & Grid

	drawXLabels(g);
	drawTimeAxisGrid(g);

	// Draw the Production Y-Axis Labels & Grid

	switch (iAxis)
	{
	  case PRODUCTION:
	    drawYLabels(g, PRODUCTION);
	    if (stPlot.iYLog == _LOG)
	    {
		  drawLogYAxisGrid(g, PRODUCTION);
		}
		else
	    {
		  drawLinearYAxisGrid(g, PRODUCTION);
		}

		drawProductionVsTimeCurves(g, Color.black);
	    break;

	  case CUMULATIVE:
	    drawYLabels(g, CUMULATIVE);
	    if (stPlot.iYLog == _LOG)
	    {
		  drawLogYAxisGrid(g, CUMULATIVE);
		}
		else
	    {
		  drawLinearYAxisGrid(g, CUMULATIVE);
		}

		drawCumulativeVsTimeCurves(g, Color.black);
	    break;

	  case BOTH:
	    drawYLabels(g, PRODUCTION);
	    drawYLabels(g, CUMULATIVE);
	    if (stPlot.iYLog == _LOG)
	    {
		  drawLogYAxisGrid(g, PRODUCTION);
		  drawLogYAxisGrid(g, CUMULATIVE);
		}
		else
	    {
		  drawLinearYAxisGrid(g, PRODUCTION);
		  drawLinearYAxisGrid(g, CUMULATIVE);
		}

		drawProductionVsTimeCurves(g, Color.black);
		drawCumulativeVsTimeCurves(g, Color.blue);
	    break;
	}

    if (iRawCurve == ON)
    {
      drawRawProductionVsTimeCurves(g, new Color(50, 150, 0));
	}

    if (i12Avg == ON)
    {
	  drawMonthlyAvgVsTimeCurves(g, new Color(62, 200, 62));
	}

    if (i365Avg == ON)
    {
	  drawDailyAvgVsTimeCurves(g, new Color(150, 150, 150));
	}

    if (iWellsCurve == ON)
    {
	  drawNumberofWellsVsTimeCurves(g, Color.red);
	}

	if (iDeclineCurve == ON)
	{
	  drawDeclineVsTimeCurves(g, new Color(180, 90, 0));
	  drawDeclineData(g, new Color(180, 90, 0));
	}
  }

  /** Method refresh()
   *  <p> This method will refresh the plot
   */

  public void refresh() { repaint(); }

  /** Method paint()
   *  <p> This method will create the production plot
   *  @param g = Graphics pointer
   */

  public void paint(Graphics g)
  {
	drawTitles(g);
	drawDataVsTime(g, iYAxis);

	g.dispose();
  }
}

/*
 *  @version 1.1 09/10/2013
 *  @author  John Victorine
 */
