/*
 * @productionPlotUtility.java Version 1.1 01/28/2014
 *
 * Copyright (c) 2013 Kansas Geological Survey
 * 1930 Constant Avenue, Lawrence, Kansas, 66047, U.S.A.
 * All Rights Reserved.
 */

package production.plot;

import iqstrat.iqstratHeadersStruct;              // Well Headers Data Structure

import production.productionListStruct;           // Production List Structure
import production.plot.productionPlotStruct;      // Production Plot Data Structure
import production.plot.productionPlotListStruct;  // Production Plot List Structure

/** Class productionPlotUtility
 *  <p> This Class will provide basic utilities for the Rock Fossil
 *      data structures.
 *
 *  @version 1.1 01/28/2014
 *  @author  John R. Victorine
 */

public class productionPlotUtility
{
  public static final int OIL      = production.productionStruct.OIL;
  public static final int GAS      = production.productionStruct.GAS;

  public static final int CEILING = java.math.BigDecimal.ROUND_CEILING;
  public static final int FLOOR   = java.math.BigDecimal.ROUND_FLOOR;

  /** Method copyList()
   * <p> This method will copy one structure to another
   * @param  stOld = the Old List Structure
   * @return stNew = the New List structure
   */

  public static productionPlotListStruct copyList(productionPlotListStruct stOld)
  {
    productionPlotListStruct stNew = null;

    if (stOld != null)
    {

      stNew.iCount = stOld.iCount;
      stNew.stItem = new productionPlotStruct[stOld.iCount];

      for (int i=0; i<stOld.iCount; i++)
      {
        stNew.stItem[i] = copy(stOld.stItem[i]);
      }
    }

    return (stNew);
  }

  /** Method transfer()
   * <p> This method will copy one structure to another
   * @param  stOld = the Old List Structure
   * @return stNew = the New List structure
   */

  public static productionPlotListStruct transfer(productionPlotListStruct stOld)
  {
    productionPlotListStruct stNew = null;

    if (stOld != null)
    {
	  stNew = copyList( stOld );

	  stOld.delete();
	  stOld = null;
	}

    return (stNew);
  }

  /** Method copy()
   * <p> This method will copy data from one structure to another
   * @param  stOld = The original Fossil Data Structure
   * @return stNew = The Fossil Data Structure
   */

  public static productionPlotStruct copy(productionPlotStruct stOld)
  {
	int i = 0;
    productionPlotStruct stNew = new productionPlotStruct();

    if (stOld != null)
    {
      stNew.iWidth        = stOld.iWidth;  // Width of the plot area
      stNew.iHeight       = stOld.iHeight; // height of the plot area

      // --Titles

      stNew.sTitle1       = new String( stOld.sTitle1 );   // First line of the plot title
      stNew.sTitle2       = new String( stOld.sTitle2 );   // Second line of the plot title
      stNew.sTitle3       = new String( stOld.sTitle3 );   // Third line of the plot title

      // -- X-Axis ( Date Axis )

      stNew.iXLog         = stOld.iXLog;                   // X-Axis Log Grid Indicator
      stNew.iXCycles      = stOld.iXCycles;                // Number of Log Cycles (Only of Log Scales)
      stNew.dXmax         = stOld.dXmax;                   // Maximum X-Axis
      stNew.dXmin         = stOld.dXmin;                   // Minimum X-Axis
      stNew.dXincr        = stOld.dXincr;                  // Increment along X-Axis
      stNew.sXlabel       = new String( stOld.sXlabel );   // X-Axis Label

      // -- Y-Axis ( Date Axis )

      stNew.iYLog         = stOld.iYLog;                   // Y-Axis Log Grid Indicator
      stNew.iYCycles      = stOld.iYCycles;                // Number of Log Cycles (Only of Log Scales)
      stNew.dYmax         = stOld.dYmax;                   // Maximum Y-Axis
      stNew.dYmin         = stOld.dYmin;                   // Minimum Y-Axis
      stNew.dYincr        = stOld.dYincr;                  // Increment along Y-Axis
      stNew.sYlabel       = new String( stOld.sYlabel );   // Y-Axis Label

      // Production Header Data

      stNew.sKID          = new String( stOld.sKID );      // DB Primary Key of the Header Record
      stNew.sName         = new String( stOld.sName );     // Well Name or Outcrop Name
      stNew.status        = new String( stOld.status );    // Status

      // -- Location Information

      stNew.state         = new String( stOld.state );     // State Name
      stNew.iState        = stOld.iState;                  // State Code
      stNew.sCounty       = new String( stOld.sCounty );   // County Name
      stNew.iCounty       = stOld.iCounty;                 // County Code

      stNew.iTownship     = stOld.iTownship;               // Township
      stNew.sTownship     = new String( stOld.sTownship ); // Township Direction
      stNew.iRange        = stOld.iRange;                  // Range
      stNew.sRange        = new String( stOld.sRange );    // Range Direction
      stNew.iSection      = stOld.iSection;                // Section

      // -- XY Position Information

      stNew.dLatitude     = stOld.dLatitude;               // Latitude of Location
      stNew.dLongitude    = stOld.dLongitude;              // Longitude of Location
      stNew.dZone         = stOld.dZone;                   // UTM Zone
      stNew.dUTMx         = stOld.dUTMx;                   // UTM X-Position
      stNew.dUTMy         = stOld.dUTMy;                   // UTM Y-Position

      // Production Plot Data

      stNew.iProduct      = stOld.iProduct; // Product Identifier
      stNew.iFactor       = stOld.iFactor;  // Normalization Factor for data

      stNew.iRows         = stOld.iRows;    // Number of Records
      stNew.iTime         = new int[stNew.iRows];
      stNew.dProduction   = new double[stNew.iRows];
      stNew.dCumulative   = new double[stNew.iRows];
      stNew.iSources      = new int[stNew.iRows];

      for (i=0; i<stNew.iRows; i++)
      {
        stNew.iTime[i]       = stOld.iTime[i];       // Array of time values
        stNew.dProduction[i] = stOld.dProduction[i]; // Array of Production Values
        stNew.dCumulative[i] = stOld.dCumulative[i]; // Array of Cumulative Production Values
        stNew.iSources[i]    = stOld.iSources[i];    // Array of Sources contributing
      }

      stNew.iRawRows      = stOld.iRawRows;  // Number of Raw Records
      stNew.dTime         = new double[stNew.iRawRows];
      stNew.dRaw          = new double[stNew.iRawRows];

      for (i=0; i<stNew.iRawRows; i++)
      {
        stNew.dTime[i] = stOld.dTime[i]; // Array of decimal time values
        stNew.dRaw[i]   = stOld.dRaw[i]; // Array of Raw Production Values
	  }
    }

    return (stNew);
  }

  /** Method add()
   * <p> This method will add the Production Plot Data to an existing list
   * @param  stData = The Production Plot Data Structure
   * @param  st     = The Old Production Plot Data List Data Structure
   * @return st     = The new Production Plot Data List Data Structure.
   */

  public static productionPlotListStruct add( productionPlotStruct stData,
                                              productionPlotListStruct st )
  {
    int    i=0;
    int    iRecords = 0;
    int    iCount   = 0;
    int    iSource  = -1;
    productionPlotListStruct stTemp = new productionPlotListStruct();

    if (st != null)
      iRecords = st.iCount+1;
    else
      iRecords = 1;

    stTemp.stItem = new productionPlotStruct[iRecords];

    if (st != null)
    {
      if (st.iCount > 0)
      {
        for (i=0; i<st.iCount; i++)
        {
          if (iCount < iRecords)
          {
            stTemp.stItem[iCount] = copy(st.stItem[i]);
            iCount++;
          }
        }
      }

      st.delete();
    }

    stTemp.stItem[iCount] = new productionPlotStruct();
    stTemp.stItem[iCount] = copy(stData);
    iCount++;

    stTemp.iCount = iCount;

    st        = new productionPlotListStruct();
    st.stItem = new productionPlotStruct[iCount];
    st.iCount = iCount;

    for (i=0; i<stTemp.iCount; i++)
    {
      st.stItem[i] = copy(stTemp.stItem[i]);
    }

    stTemp.delete();

    return (st);
  }

  /** Method buildProductionPlotData()
   *  <p> This method will convert the data retrieved from the production List
   *      Structure to the production Plot data structure
   *  @param  sProduct = the product to be plotted
   *  @param  stLease  = The Lease Header Data Structure
   *  @param  stList   = The Production List Data Structure
   *  @return stPlot   = The production Plot data Structure
   */

  public static productionPlotStruct buildProductionPlotData(
	                                   String sProduct,
                                       iqstratHeadersStruct stWell,
                                       productionListStruct stList )
  {
	int    i,j;
	int    iCount      = 0;
	int    iFound      = 0;
	int    iYear       = 0;
	int    iMinimum    = 0;
	int    iMaximum    = 0;
	int    iSources    = 0;
	double dProduction = 0.0;
	double dCumulative = 0.0;
	double dTemp       = 0.0;
    productionPlotStruct stPlot = null;

	if (stList != null)
	{
      stPlot = new productionPlotStruct();

      stPlot.iFactor = 1;
      stPlot.iWidth  = 1200; //600;
      stPlot.iHeight = 850; //450;

      stPlot.sTitle2 = new String(stList.sName);
      stPlot.sTitle3 = new String(sProduct);

      // Production Header Data

      stPlot.sKID          = new String( stWell.sKID );      // DB Primary Key of the Header Record
      stPlot.sName         = new String( stWell.sName );     // Well Name or Outcrop Name
      stPlot.status        = new String( stWell.status );    // Status

      // -- Location Information

      stPlot.state         = new String( stWell.state );     // State Name
      stPlot.iState        = stWell.iState;                  // State Code
      stPlot.sCounty       = new String( stWell.sCounty );   // County Name
      stPlot.iCounty       = stWell.iCounty;                 // County Code

      stPlot.iTownship     = stWell.iTownship;               // Township
      stPlot.sTownship     = new String( stWell.sTownship ); // Township Direction
      stPlot.iRange        = stWell.iRange;                  // Range
      stPlot.sRange        = new String( stWell.sRange );    // Range Direction
      stPlot.iSection      = stWell.iSection;                // Section

      // -- XY Position Information

      stPlot.dLatitude     = stWell.dLatitude;               // Latitude of Location
      stPlot.dLongitude    = stWell.dLongitude;              // Longitude of Location
      stPlot.dZone         = stWell.dZone;                   // UTM Zone
      stPlot.dUTMx         = stWell.dUTMx;                   // UTM X-Position
      stPlot.dUTMy         = stWell.dUTMy;                   // UTM Y-Position

      // Production Data

      stPlot.iRows = 0;

      stPlot.dTime    = new double[stList.iCount];
      stPlot.dRaw     = new double[stList.iCount];


      iCount = 0;
      for (i=0; i<stList.iCount; i++)
      {
		if (stList.stItem[i].sProduct.equals(sProduct))
		{
		  if (iFound == 0)
		  {
			iMinimum = stList.stItem[i].iYear;
			iMaximum = stList.stItem[i].iYear;
			iFound = 1;
		  }

		  if (iMinimum > stList.stItem[i].iYear)
		    iMinimum = stList.stItem[i].iYear;

		  if (iMaximum < stList.stItem[i].iYear)
		    iMaximum = stList.stItem[i].iYear;

          stPlot.dTime[iCount] = stList.stItem[i].date;
          stPlot.dRaw[iCount]  = production.math.productionMath.divide(stList.stItem[i].data, stPlot.iFactor);
          iCount++;
		}
	  }

      stPlot.iRawRows = iCount;

      stPlot.iRows = iMaximum - iMinimum;
      iCount       = 0;
      dProduction  = 0.0;
      dCumulative  = 0.0;

      if (stPlot.iRows > 0)
      {
		stPlot.iTime       = new int[stPlot.iRows];    // Array of Time Values
		stPlot.dProduction = new double[stPlot.iRows]; // Array of Production
		stPlot.dCumulative = new double[stPlot.iRows]; // Array of Cumulative
		stPlot.iSources    = new int[stPlot.iRows];    // Array of Sources

		iYear = iMinimum;

		for (i=0; i<stPlot.iRows; i++)
		{
		  dProduction = 0.0;

		  for (j=0; j<stList.iCount; j++)
		  {
			if (stList.stItem[j].sProduct.equals(sProduct))
			{
		      if (iYear == stList.stItem[j].iYear)
		      {
				dTemp       = production.math.productionMath.divide(stList.stItem[j].data, stPlot.iFactor);
				dProduction = production.math.productionMath.add(dProduction, dTemp);
				dCumulative = production.math.productionMath.add(dCumulative, dTemp);
				iSources    = stList.stItem[j].iSources;
			  }
			}
		  }

		  if (i==0)
		  {
			stPlot.dXmin = iYear;
			stPlot.dXmax = iYear;
			stPlot.dYmin = dProduction;
			stPlot.dYmax = dProduction;
		  }

		  if (iCount < stPlot.iRows)
		  {
			stPlot.iTime[iCount]       = iYear;
			stPlot.dProduction[iCount] = dProduction;
			stPlot.dCumulative[iCount] = dCumulative;
			stPlot.iSources[iCount]    = iSources;
//System.out.println(iCount+" "+iYear+" "+dProduction+" "+dCumulative+" "+iSources);

			iYear++;
			iCount++;
		  }

		  if ((double) iYear < stPlot.dXmin)
		  {
			stPlot.dXmin = (double) iYear;
		  }

		  if ((double) iYear > stPlot.dXmax)
		  {
			stPlot.dXmax = (double) iYear;
		  }

		  if ((dProduction < stPlot.dYmin) && (dProduction != 0.0))
		  {
			stPlot.dYmin = dProduction;
		  }

		  if (dCumulative > stPlot.dYmax)
		  {
			stPlot.dYmax = dCumulative;
		  }
		}
	  }

	  stPlot.iXLog    = production.plot.productionPlotStruct._LINEAR;
	  stPlot.iXCycles = 1;

	  stPlot.iYLog    = production.plot.productionPlotStruct._LOG;
	  stPlot.iYCycles = 1;
	  stPlot.dYincr   = 1.0;

	  stPlot.sXlabel  = new String("Year");

	  if (sProduct.equals(production.productionStruct.PRODUCT[OIL]))
	  {
		stPlot.iProduct = OIL;
		stPlot.sYlabel  = new String (sProduct +
		                      "(" + production.productionStruct.UNITS[OIL] + ")");
	  }
	  else if (sProduct.equals(production.productionStruct.PRODUCT[GAS]))
	  {
		stPlot.iProduct = GAS;
		stPlot.sYlabel  = new String (sProduct +
		                      "(" + production.productionStruct.UNITS[GAS] + ")");
	  }
	  else
	  {
		stPlot.sYlabel = new String (sProduct);
	  }

	  stPlot.sYlabel = new String(stPlot.sYlabel + " * " + stPlot.iFactor);

	  // X-Axis Values

	  int iTemp = (int) (Math.ceil((stPlot.dXmax-stPlot.dXmin) / 20.0));

	  if (iTemp == 0)
	    iTemp = 1;

	  if (iTemp == 3)
	    iTemp = 4;

	  stPlot.dXincr = (double) iTemp; // Increment along the X-Axis

	  int iCycleL = (int) Math.floor( Math.log(stPlot.dYmin) / Math.log(10.0));
	  int iCycleU = (int) Math.ceil( Math.log(stPlot.dYmax) / Math.log(10.0));

	  stPlot.iYCycles = (iCycleU-iCycleL);
	  if (stPlot.iYCycles == 0)
	    stPlot.iYCycles = 1;

	  stPlot.dYmin = Math.pow(10.0, iCycleL);
	  stPlot.dYmax = Math.pow(10.0, iCycleU);
	}

	return (stPlot);
  }
}

/*
 *  @version 1.1 01/28/2014
 *  @author  John Victorine
 */
