/*
 * @productionBuildPlotData.java Version 1.1 09/10/2013
 *
 * Copyright (c) 2013 Kansas Geological Survey
 * 1930 Constant Avenue, Lawrence, Kansas, 66047, U.S.A.
 * All Rights Reserved.
 */

package production;

import java.math.BigDecimal;

import production.productionListStruct;       // Production List Structure
import production.plot.productionPlotStruct;  // Production Plot Data Structure

/** Class productionBuildPlotData
 *  <p> This Class holds all production data
 *
 *  @version 1.1 09/10/2013
 *  @author  John R. Victorine
 */

public class productionBuildPlotData
{
  public static final int LEASE    = iqstrat.iqstratInfoStruct.LEASE;
  public static final int FIELD    = iqstrat.iqstratInfoStruct.FIELD;
  public static final int COUNTY   = iqstrat.iqstratInfoStruct.COUNTY;
  public static final int OPERATOR = iqstrat.iqstratInfoStruct.OPERATOR;

  public static final int OIL      = production.productionStruct.OIL;
  public static final int GAS      = production.productionStruct.GAS;

  public static final int CEILING = java.math.BigDecimal.ROUND_CEILING;
  public static final int FLOOR   = java.math.BigDecimal.ROUND_FLOOR;

  // Input Variables

  private int iType   = LEASE;

  // Global Variables

  private productionListStruct stList = null; // Production List Structure
  private productionPlotStruct stPlot = null; // Production Plot Data Structure

  private int iFactor  = 1; // Divide Data By

  /** Constructor productionBuildPlotData()
   *  <p> This is the Constructor for this class.
   *  @param iType = Type of production plot
   */

  public productionBuildPlotData( int iType )
  {
	this.iType = iType;
	setInitialFactor();
  }

  /** Method close()
   * <p> This method will close all dialogs that are opened by this class.
   */

  public void close()
  {
    stList = null; // Production List Structure

    if (stPlot != null)
      stPlot.delete();
    stPlot = null; // Production Plot Data Structure
  }

  /* ===================================================================== *
   * -------------------------- GET METHODS ------------------------------ *
   * ===================================================================== */

  /** Method getProductionData()
   *  <p> This method will get the production list structure
   *  @return stList = The production List Structure
   */

  public productionListStruct getProductionData() { return (stList); }

  /** Method getPlotData()
   *  <p> This method will get the production plot data structure
   *  @return stPlot = The production Plot data Structure
   */

  public productionPlotStruct getPlotData() { return (stPlot); }

  /** Method getFactor()
   *  <p> This method will set the default divide by factor
   */

  public int getFactor() { return (iFactor); }

  /* ===================================================================== *
   * -------------------------- SET METHODS ------------------------------ *
   * ===================================================================== */

  /** Method setInitialFactor()
   *  <p> This method will determine the type of data that is being created
   *      and set a default divide by factor
   */

  public void setInitialFactor()
  {
	switch (iType)
	{
	  case COUNTY:
	    iFactor = 100000;
	    break;
	  case LEASE:
	    iFactor = 1;
	    break;
	  case FIELD:
	    iFactor = 1000;
	    break;
	  case OPERATOR:
	    iFactor = 1000;
	    break;
    }
  }

  /** Method setProductionData()
   *  <p> This method will set the production list structure
   *  @param stList = The production List Structure
   */

  public void setProductionData(productionListStruct st) { stList = st; }

  /** Method setFactor(int iFactor)
   *  <p> This method will set the default divide by factor
   */

  public void setFactor() { this.iFactor = iFactor; }

  /* ===================================================================== *
   * ------------------------- BUILD METHODS ----------------------------- *
   * ===================================================================== */

  /** Method buildProductionPlotData()
   *  <p> This method will convert the data retrieved from the production List
   *      Structure to the production Plot data structure
   *  @param  sProduct = the product to be plotted
   *  @return stPlot   = The production Plot data Structure
   */

  public productionPlotStruct buildProductionPlotData(String sProduct)
  {
	int    i,j;
	int    iCount      = 0;
	int    iFound      = 0;
	int    iYear       = 0;
	int    iMinimum    = 0;
	int    iMaximum    = 0;
	int    iSources    = 0;
	double dProduction = 0.0;
	double dCumulative = 0.0;
	double dTemp       = 0.0;

	if (stList != null)
	{
      stPlot = new productionPlotStruct();

      stPlot.iFactor = iFactor;
      stPlot.iWidth  = 1200; //600;
      stPlot.iHeight = 850; //450;

      // Titles

  	  switch (iType)
	  {
	    case COUNTY:
	      stPlot.sTitle1 = new String("County Production");
	      break;
	    case LEASE:
	      stPlot.sTitle1 = new String("Lease Production");
	      break;
	    case FIELD:
	      stPlot.sTitle1 = new String("Field Production");
	      break;
	    case OPERATOR:
	      stPlot.sTitle1 = new String("Operator Production");
	      break;
      }

      stPlot.sTitle2 = new String(stList.sName);
      stPlot.sTitle3 = new String(sProduct);

      stPlot.iRows = 0;

      stPlot.dTime    = new double[stList.iCount];
      stPlot.dRaw     = new double[stList.iCount];

      iCount = 0;
      for (i=0; i<stList.iCount; i++)
      {
		if (stList.stItem[i].sProduct.equals(sProduct))
		{
		  if (iFound == 0)
		  {
			iMinimum = stList.stItem[i].iYear;
			iMaximum = stList.stItem[i].iYear;
			iFound = 1;
		  }

		  if (iMinimum > stList.stItem[i].iYear)
		    iMinimum = stList.stItem[i].iYear;

		  if (iMaximum < stList.stItem[i].iYear)
		    iMaximum = stList.stItem[i].iYear;

          stPlot.dTime[iCount] = stList.stItem[i].date;
          stPlot.dRaw[iCount]  = production.math.productionMath.divide(stList.stItem[i].data, iFactor);
          iCount++;
		}
	  }

      stPlot.iRawRows = iCount;

      stPlot.iRows = iMaximum - iMinimum;
      iCount       = 0;
      dProduction  = 0.0;
      dCumulative  = 0.0;

      if (stPlot.iRows > 0)
      {
		stPlot.iTime       = new int[stPlot.iRows];    // Array of Time Values
		stPlot.dProduction = new double[stPlot.iRows]; // Array of Production
		stPlot.dCumulative = new double[stPlot.iRows]; // Array of Cumulative
		stPlot.iSources    = new int[stPlot.iRows];    // Array of Sources

		iYear = iMinimum;

		for (i=0; i<stPlot.iRows; i++)
		{
		  dProduction = 0.0;

		  for (j=0; j<stList.iCount; j++)
		  {
			if (stList.stItem[j].sProduct.equals(sProduct))
			{
		      if (iYear == stList.stItem[j].iYear)
		      {
				dTemp       = production.math.productionMath.divide(stList.stItem[j].data, iFactor);
				dProduction = production.math.productionMath.add(dProduction, dTemp);
				dCumulative = production.math.productionMath.add(dCumulative, dTemp);
				iSources    = stList.stItem[j].iSources;
			  }
			}
		  }

		  if (i==0)
		  {
			stPlot.dXmin = iYear;
			stPlot.dXmax = iYear;
			stPlot.dYmin = dProduction;
			stPlot.dYmax = dProduction;
		  }

		  if (iCount < stPlot.iRows)
		  {
			stPlot.iTime[iCount]       = iYear;
			stPlot.dProduction[iCount] = dProduction;
			stPlot.dCumulative[iCount] = dCumulative;
			stPlot.iSources[iCount]    = iSources;
//System.out.println(iCount+" "+iYear+" "+dProduction+" "+dCumulative+" "+iSources);

			iYear++;
			iCount++;
		  }

		  if ((double) iYear < stPlot.dXmin)
		  {
			stPlot.dXmin = (double) iYear;
		  }

		  if ((double) iYear > stPlot.dXmax)
		  {
			stPlot.dXmax = (double) iYear;
		  }

		  if ((dProduction < stPlot.dYmin) && (dProduction != 0.0))
		  {
			stPlot.dYmin = dProduction;
		  }

		  if (dCumulative > stPlot.dYmax)
		  {
			stPlot.dYmax = dCumulative;
		  }
		}
	  }

	  stPlot.iXLog    = production.plot.productionPlotStruct._LINEAR;
	  stPlot.iXCycles = 1;

	  stPlot.iYLog    = production.plot.productionPlotStruct._LOG;
	  stPlot.iYCycles = 1;
	  stPlot.dYincr   = 1.0;

	  stPlot.sXlabel  = new String("Year");

	  if (sProduct.equals(production.productionStruct.PRODUCT[OIL]))
	  {
		stPlot.iProduct = OIL;
		stPlot.sYlabel  = new String (sProduct +
		                      "(" + production.productionStruct.UNITS[OIL] + ")");
	  }
	  else if (sProduct.equals(production.productionStruct.PRODUCT[GAS]))
	  {
		stPlot.iProduct = GAS;
		stPlot.sYlabel  = new String (sProduct +
		                      "(" + production.productionStruct.UNITS[GAS] + ")");
	  }
	  else
	  {
		stPlot.sYlabel = new String (sProduct);
	  }

	  stPlot.sYlabel = new String(stPlot.sYlabel + " * " + iFactor);

	  // X-Axis Values

	  int iTemp = (int) (Math.ceil((stPlot.dXmax-stPlot.dXmin) / 20.0));

	  if (iTemp == 0)
	    iTemp = 1;

	  if (iTemp == 3)
	    iTemp = 4;

	  stPlot.dXincr = (double) iTemp; // Increment along the X-Axis

	  int iCycleL = (int) Math.floor( Math.log(stPlot.dYmin) / Math.log(10.0));
	  int iCycleU = (int) Math.ceil( Math.log(stPlot.dYmax) / Math.log(10.0));

	  stPlot.iYCycles = (iCycleU-iCycleL);
	  if (stPlot.iYCycles == 0)
	    stPlot.iYCycles = 1;

	  stPlot.dYmin = Math.pow(10.0, iCycleL);
	  stPlot.dYmax = Math.pow(10.0, iCycleU);
	}

	return (stPlot);
  }
}

/*
 *  @version 1.1 09/10/2013
 *  @author  John Victorine
 */
